@@ -25,7 +25,7 @@
 
 @TOP@
 
-
+/* Always defined */
 #undef D_INO_IN_DIRENT
 #undef IS_AIX
 #undef MOUNTED_FREAD
@@ -164,6 +164,18 @@
 /* Define if your system uses PAM for auth stuff */
 #undef HAVE_PAM
 
+/* Define if you have the pmap_getmaps function */
+#undef HAVE_PMAP_GETMAPS
+
+/* Define if you have the <sys/select.h> header file.  */
+#undef HAVE_SYS_SELECT_H
+
+/* Define if you have the get_process_stats function and have to use that instead of gettimeofday  */
+#undef HAVE_GET_PROCESS_STATS
+
+/* Define if you want to call the internal routine edit() for the editor */
+#undef USE_INTERNAL_EDIT
+
 @BOTTOM@
 
 #ifdef HAVE_LIBPT
