@@ -34,12 +34,8 @@
 #include <sys/ioctl.h>
 #endif
 
-#ifdef __QNX__
-#include <sys/select.h>
-#endif
-
-#ifdef IS_AIX
-#include <sys/select.h>
+#ifdef HAVE_SYS_SELECT_H
+#    include <sys/select.h>
 #endif
 
 #include <sys/types.h>
@@ -99,6 +95,9 @@ TTY_Termio_Type Old_TTY;
 
 static int TTY_Inited = 0;
 
+/* Used to be 255, but breaks with some input languages that required 255 */
+#define IGNORE_THING 0
+
 int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 {
    TTY_Termio_Type newtty;
@@ -151,10 +150,10 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
    newtty.c_cc[VEOF] = 1;
    newtty.c_lflag = ISIG | NOFLSH;
    newtty.c_cc[VINTR] = SLang_Abort_Char;   /* ^G */
-   newtty.c_cc[VQUIT] = 255;
-   newtty.c_cc[VSUSP] = 255;   /* to ignore ^Z */
+   newtty.c_cc[VQUIT] = IGNORE_THING;
+   newtty.c_cc[VSUSP] = IGNORE_THING;   /* to ignore ^Z */
 #ifdef VSWTCH
-   newtty.c_cc[VSWTCH] = 255;   /* to ignore who knows what */
+   newtty.c_cc[VSWTCH] = IGNORE_THING;   /* to ignore who knows what */
 #endif
 #endif /*sequent*/
    SET_TERMIOS(SLang_TT_Read_FD, &newtty);
