@@ -29,6 +29,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>		/* For malloc() */
+#include <errno.h>	/* For errno on SunOS systems	      */
 #include "mad.h"
 #include "tty.h"
 #include "util.h"		/* Needed for the externs */
@@ -45,6 +46,7 @@
 #include "file.h"
 #include "chmod.h"
 #include "main.h"
+#include "../vfs/vfs.h"
 
 #define BX		5
 #define BY		6
@@ -268,7 +270,7 @@ static void do_enter_key (Dlg_head *h, int f_pos)
 			      "[Chown-advanced]", "achown_enter", DLG_NONE);
 	
 	/* get new listboxes */
-	chl_list = listbox_new (1, 1, 15, 11, 0, l_call);
+	chl_list = listbox_new (1, 1, 15, 11, 0, l_call, NULL);
 	
 	listbox_add_item (chl_list, 0, 0, "<Unknown>", NULL);
 	
@@ -546,7 +548,7 @@ static void init_chown_advanced (void)
 
 #define XTRACT(i) BY+chown_advanced_but[i].y, BX+chown_advanced_but[i].x, \
                   chown_advanced_but[i].ret_cmd, chown_advanced_but[i].text, \
-                  chown_advanced_but[i].hkey, chown_advanced_but[i].hpos, 0, 0
+                  chown_advanced_but[i].hkey, chown_advanced_but[i].hpos, 0, 0, NULL
 
     for (i = 0; i < BUTTONS - 5; i++)
 	if (!single_set || i < 2)
@@ -595,11 +597,18 @@ static void apply_advanced_chowns (struct stat *sf)
     gid_t a_gid = sf->st_gid;
     uid_t a_uid = sf->st_uid;
 
-    fname = cpanel->dir.list[current_file].fname, get_mode ();
+    fname = cpanel->dir.list[current_file].fname;
     need_update = end_chown = 1;
-    chmod (cpanel->dir.list[current_file].fname, get_mode ());
-    chown (fname, (ch_flags[9] == '+') ? sf->st_uid : -1,
-	   (ch_flags[10] == '+') ? sf->st_gid : -1);
+    if (mc_chmod (fname, get_mode ()) == -1)
+	message (1, " Error ", " Couldn't chmod \"%s\" \n %s ",
+		 fname, unix_error_string (errno));
+    /* call mc_chown only, if mc_chmod didn't fail */
+    else if (mc_chown (fname, (ch_flags[9] == '+') ? sf->st_uid : -1,
+		       (ch_flags[10] == '+') ? sf->st_gid : -1) == -1)
+	message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
+		 fname, unix_error_string (errno));
+    file_mark (cpanel, current_file, 0);
+    cpanel->marked--;
 
     do {
 	fname = next_file ();
@@ -607,8 +616,13 @@ static void apply_advanced_chowns (struct stat *sf)
 	if (!stat_file (fname, sf))
 	    break;
 	ch_cmode = sf->st_mode;
-	chmod (fname, get_mode ());
-	chown (fname, (ch_flags[9] == '+') ? a_uid : -1, (ch_flags[10] == '+') ? a_gid : -1);
+	if (mc_chmod (fname, get_mode ()) == -1)
+	    message (1, " Error ", " Couldn't chmod \"%s\" \n %s ",
+		     fname, unix_error_string (errno));
+	/* call mc_chown only, if mc_chmod didn't fail */
+	else if (mc_chown (fname, (ch_flags[9] == '+') ? a_uid : -1, (ch_flags[10] == '+') ? a_gid : -1) == -1)
+	    message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
+		     fname, unix_error_string (errno));
 
 	file_mark (cpanel, current_file, 0);
 	cpanel->marked--;
@@ -620,6 +634,18 @@ void chown_advanced_cmd (void)
 
     files_on_begin = cpanel->marked;
 
+    if (!vfs_current_is_local ()) {
+	if (vfs_current_is_extfs ()) {
+	    message (1, " Oops... ",
+		     " I can't run the Advanced Chown command on an extfs ");
+	    return;
+	} else if (vfs_current_is_tarfs ()) {
+	    message (1, " Oops... ",
+		     " I can't run the Advanced Chown command on a tarfs ");
+	    return;
+	}
+    }
+
     do {			/* do while any files remaining */
 	init_chown_advanced ();
 
@@ -628,8 +654,10 @@ void chown_advanced_cmd (void)
 	else
 	    fname = selection (cpanel)->fname;	/* single file */
 
-	if (!stat_file (fname, sf_stat))	/* get status of file */
+	if (!stat_file (fname, sf_stat)){	/* get status of file */
+	    destroy_dlg (ch_dlg);
 	    break;
+	}
 	ch_cmode = sf_stat->st_mode;
 
 	chown_refresh ();
@@ -646,10 +674,14 @@ void chown_advanced_cmd (void)
 
 	case B_ENTER:
 	    need_update = 1;
-	    chmod (fname, get_mode ());
-	    chown (fname, (ch_flags[9] == '+') ? sf_stat->st_uid : -1, (ch_flags[10] == '+') ? sf_stat->st_gid : -1);
+	    if (mc_chmod (fname, get_mode ()) == -1)
+		message (1, " Error ", " Couldn't chmod \"%s\" \n %s ",
+			 fname, unix_error_string (errno));
+	    /* call mc_chown only, if mc_chmod didn't fail */
+	    else if (mc_chown (fname, (ch_flags[9] == '+') ? sf_stat->st_uid : -1, (ch_flags[10] == '+') ? sf_stat->st_gid : -1) == -1)
+		message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
+			 fname, unix_error_string (errno));
 	    break;
-
 	case B_SETALL:
 	    apply_advanced_chowns (sf_stat);
 	    break;
