@@ -55,6 +55,7 @@ static WInput *user;
 static WInput *status;
 static WCheck *check_status;
 static int current_mode;
+extern int ftpfs_always_use_proxy;
 
 #define VIEW_TYPES 4
 
@@ -166,7 +167,7 @@ static int display_callback (struct Dlg_head *h, int id, int Msg)
 	    && h->current->widget != (Widget *) status){
 	    my_radio->sel = 3;
 	    dlg_select_widget (h, my_radio); /* force redraw */
-	    dlg_select_widget (h, user);
+    dlg_select_widget (h, user);
 	    return MSG_HANDLED;
 	}
     }
@@ -179,37 +180,31 @@ static void display_init (int radio_sel, char *init_text,
 {
     dd = create_dlg (0, 0, DISPLAY_Y, DISPLAY_X, dialog_colors,
 		     display_callback, "[Left and Right Menus]", "display",
-		     DLG_CENTER);
+		     DLG_CENTER | DLG_GRID);
 
     x_set_dialog_title (dd, "Listing mode");
-    tk_new_frame (dd, "b.");
     add_widgetl (dd,
-        button_new (3, 32, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0),
+        button_new (3, 32, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0, "cancel-button"),
 	XV_WLAY_RIGHTOF);
 
     add_widgetl (dd,
-	button_new (5, 32, B_ENTER,  "[[ Ok ]]", 'o', 3, 0, 0),
+	button_new (5, 32, B_ENTER,  "[[ Ok ]]", 'o', 3, 0, 0, "ok-button"),
 	 XV_WLAY_CENTERROW);
-    tk_end_frame ();
 
-    tk_new_frame (dd, "m.");
-    status = input_new (9, 8, NORMAL_COLOR, 34, _status);
+    status = input_new (9, 8, NORMAL_COLOR, 34, _status, "mini-input");
     add_widgetl (dd, status, XV_WLAY_RIGHTDOWN);
     input_set_point (status, 0);
 
-    check_status = check_new (8, 4, _check_status, "user Mini status",'m', 5);
+    check_status = check_new (8, 4, _check_status, "user Mini status",'m', 5, "mini-status");
     add_widgetl (dd, check_status, XV_WLAY_NEXTROW);
-    tk_end_frame ();
     
-    tk_new_frame (dd, "r.");
-    user = input_new  (6, 14, NORMAL_COLOR, 29, init_text);
+    user = input_new  (6, 14, NORMAL_COLOR, 29, init_text, "user-fmt-input");
     add_widgetl (dd, user, XV_WLAY_RIGHTDOWN);
     input_set_point (user, 0);
 
-    my_radio = radio_new (3, 4, VIEW_TYPES, displays, 1);
+    my_radio = radio_new (3, 4, VIEW_TYPES, displays, 1, "radio");
     my_radio->sel = my_radio->pos = current_mode;
     add_widgetl (dd, my_radio, XV_WLAY_BELOWCLOSE);
-    tk_end_frame ();
 }
 
 int display_box (WPanel *panel, char **userp, char **minip, int *use_msformat,
@@ -263,23 +258,6 @@ int display_box (WPanel *panel, char **userp, char **minip, int *use_msformat,
 #define SORT_X 40
 #define SORT_Y 14
 
-static int sort_callback (struct Dlg_head *h, int id, int Msg)
-{
-#ifndef HAVE_X
-    switch (Msg){
-    case DLG_DRAW:
-	attrset (REVERSE_COLOR);
-	dlg_erase (h);
-	draw_box (h, 1, 1, SORT_Y-2, SORT_X-2);
-	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 2);
-	addstr (" Sort order ");
-	break;
-    }
-#endif
-    return 0;
-}
-
 char *sort_orders_names [SORT_TYPES];
 
 sortfn *sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive)
@@ -296,33 +274,29 @@ sortfn *sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive)
 	    break;
 	}
     
-    dd = create_dlg (0, 0, SORT_Y, SORT_X, dialog_colors, sort_callback,
-		     "[Left and Right Menus]", "sort", DLG_CENTER);
+    dd = create_dlg (0, 0, SORT_Y, SORT_X, dialog_colors, common_dialog_callback,
+		     "[Left and Right Menus]", "sort", DLG_CENTER | DLG_GRID);
 		     
     x_set_dialog_title (dd, "Sort order");
 
-    tk_new_frame (dd, "b.");
-    add_widgetl (dd, button_new (6, 25, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0),
+    add_widgetl (dd, button_new (6, 25, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0, "cancel-button"),
         XV_WLAY_CENTERROW);
 
-    add_widgetl (dd, button_new (8, 25, B_ENTER, "[ Ok ]", 'o', 2, 0, 0),
+    add_widgetl (dd, button_new (8, 25, B_ENTER, "[ Ok ]", 'o', 2, 0, 0, "ok-button"),
 	XV_WLAY_RIGHTDOWN);
-    tk_end_frame ();
 
-    tk_new_frame (dd, "r.");
-    case_sense = check_new (4, 19, *case_sensitive, "case sensitive", 't', 10);
+    case_sense = check_new (4, 19, *case_sensitive, "case sensitive", 't', 10, "case-check");
     add_widgetl (dd, case_sense, XV_WLAY_RIGHTDOWN);
-    c = check_new (3, 19, *reverse, "Reverse", 'r', 0);
+    c = check_new (3, 19, *reverse, "Reverse", 'r', 0, "reverse-check");
     add_widgetl (dd, c, XV_WLAY_RIGHTDOWN);
 
     for (i = SORT_TYPES-1; i >= 0; i--){
 	sort_orders_names [i] = sort_orders [i].sort_name;
     }
-    my_radio = radio_new (3, 3, SORT_TYPES, sort_orders_names, 1);
+    my_radio = radio_new (3, 3, SORT_TYPES, sort_orders_names, 1, "radio-1");
     my_radio->sel = my_radio->pos = current_mode;
     
     add_widget (dd, my_radio);
-    tk_end_frame ();
     run_dlg (dd);
 
     r = dd->ret_value;
@@ -346,16 +320,16 @@ static int my_exit;
 
 static QuickWidget conf_widgets [] = {
 { quick_button,   4, 6, 4, CONFY, " [ Cancel ]",
-      'c', 3, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "b." },
+      'c', 3, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "c" },
 { quick_button,   4, 6, 3, CONFY, " [ oK ] ",
-      'k', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "" },
+      'k', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
 
 { quick_checkbox, 1, 13, 5, CONFY, " confirm Exit ",
-      'e', 9, 0, &my_exit, 0, XV_WLAY_BELOWCLOSE, "c." },
+      'e', 9, 0, &my_exit, 0, XV_WLAY_BELOWCLOSE, "e" },
 { quick_checkbox, 1, 13, 4, CONFY, " confirm Overwrite ",
-      'o', 9, 0, &my_overwrite, 0, XV_WLAY_BELOWCLOSE, "" },
+      'o', 9, 0, &my_overwrite, 0, XV_WLAY_BELOWCLOSE, "ov" },
 { quick_checkbox, 1, 13, 3, CONFY, " confirm Delete ",
-      'd', 9, 0, &my_delete, 0, XV_WLAY_BELOWCLOSE, "" },
+      'd', 9, 0, &my_delete, 0, XV_WLAY_BELOWCLOSE, "de" },
 { 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE }
 };
 
@@ -385,11 +359,11 @@ char *display_bits_str [] =
 
 static QuickWidget display_widgets [] = {
 { quick_button,   4,  6,    3, DISPY, " [ Ok ]",
-      'o', 3, B_ENTER, 0, 0, XV_WLAY_CENTERROW },
+      'o', 3, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
 { quick_button,   4,  6,    4, DISPY, " [ Cancel ]",
-      'c', 3, B_CANCEL, 0, 0, XV_WLAY_CENTERROW },
+      'c', 3, B_CANCEL, 0, 0, XV_WLAY_CENTERROW, "c" },
 { quick_radio,    4, DISPX, 3, DISPY, "", 3, 0, 0,
-      &new_mode, display_bits_str, XV_WLAY_BELOWCLOSE },
+      &new_mode, display_bits_str, XV_WLAY_BELOWCLOSE, "r" },
 { 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE }
 };
 
@@ -441,14 +415,8 @@ static int tree_callback (struct Dlg_head *h, int id, int msg)
 	return MSG_HANDLED;
 	
     case DLG_DRAW:
-	attrset (REVERSE_COLOR);
-	dlg_erase (h);
-	draw_box (h, 1, 1, TREE_Y-2, TREE_X-2 );
-	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 2);
-	addstr (" Select directory ");
-	attrset (COLOR_NORMAL);
-
+	common_dialog_repaint (h);
+	break;
     }
     return MSG_NOT_HANDLED;
 }
@@ -495,6 +463,7 @@ char *tree (char *current_dir)
 
 extern int vfs_timeout;
 extern int tar_gzipped_memlimit;
+extern int ftpfs_always_use_proxy;
 
 #if defined(USE_NETCODE)
 extern char *ftpfs_anonymous_passwd;
@@ -525,37 +494,37 @@ char *confvfs_str [] =
 
 static QuickWidget confvfs_widgets [] = {
 { quick_button,   VFSX - 10 - 4,  VFSX,    VFSY - 3, VFSY, "[ Cancel ]",
-      'c', 2, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "b." },
+      'c', 2, B_CANCEL, 0, 0, XV_WLAY_RIGHTOF, "button-cancel" },
 { quick_button,   4, VFSX,    VFSY - 3, VFSY, "[ Ok ]",
-      'o', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "" },
+      'o', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "button-ok" },
 #if defined(USE_NETCODE)
-{ quick_input,    23, VFSX, 10, VFSY, 0, 0, 10, 0, 0, &ret_ftp_proxy,
-      XV_WLAY_RIGHTDOWN, "" },
-{ quick_label,     4, VFSX, 10, VFSY, "ftp proxy host:",
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, ""},
+{ quick_input,    30, VFSX, 10, VFSY, 0, 0, 10, 0, 0, &ret_ftp_proxy,
+      XV_WLAY_RIGHTDOWN, "input-ftp-proxy" },
+{ quick_checkbox,    4, VFSX, 10, VFSY, "always use ftp proxy", 0, 0, 0,
+      &ftpfs_always_use_proxy, 0, XV_WLAY_RIGHTDOWN, "check-ftp-proxy" },
 { quick_label,    46, VFSX, 9, VFSY, "sec",
-      0, 0, 0, 0, 0, XV_WLAY_RIGHTOF, "" },
+      0, 0, 0, 0, 0, XV_WLAY_RIGHTOF, "label-sec" },
 { quick_input,    35, VFSX, 9, VFSY, 0, 0, 10, 0, 0, &ret_directory_timeout,
-      XV_WLAY_RIGHTDOWN, "" },
+      XV_WLAY_RIGHTDOWN, "input-timeout" },
 { quick_label,     4, VFSX, 9, VFSY, "ftpfs directory cache timeout:",
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, ""},
+      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "label-cache"},
 { quick_input,    28, VFSX, 8, VFSY, 0, 0, 21, 0, 0, &ret_passwd,
-      XV_WLAY_RIGHTDOWN, "" },
+      XV_WLAY_RIGHTDOWN, "input-passwd" },
 { quick_label,     4, VFSX, 8, VFSY, "ftp anonymous password:",
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, ""},
+      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "label-pass"},
 #endif
 { quick_input,    26, VFSX, 6, VFSY, 0, 0, 10, 0, 0, &ret_limit, 
-      XV_WLAY_RIGHTDOWN, "l." },
+      XV_WLAY_RIGHTDOWN, "input-limit" },
 { quick_radio,    4, VFSX, 5, VFSY, "", 2, 0, 0,
-      &vfs_use_limit, confvfs_str, XV_WLAY_BELOWCLOSE, "" },
+      &vfs_use_limit, confvfs_str, XV_WLAY_BELOWCLOSE, "radio" },
 { quick_label,    4,  VFSX, 4, VFSY, "Gzipped tar archive extract:", 
-      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "" },
+      0, 0, 0, 0, 0, XV_WLAY_NEXTROW, "label-tar" },
 { quick_label,    46, VFSX, 3, VFSY, "sec",
-      0, 0, 0, 0, 0, XV_WLAY_RIGHTOF, "t." },
+      0, 0, 0, 0, 0, XV_WLAY_RIGHTOF, "label-sec2" },
 { quick_input,    35, VFSX, 3, VFSY, 0, 0, 10, 0, 0, &ret_timeout, 
-      XV_WLAY_RIGHTOF, "" },
+      XV_WLAY_RIGHTOF, "input-timo-vfs" },
 { quick_label,    4,  VFSX, 3, VFSY, "Timeout for freeing VFSs:", 
-      0, 0, 0, 0, 0, XV_WLAY_BELOWCLOSE, "" },
+      0, 0, 0, 0, 0, XV_WLAY_BELOWCLOSE, "label-vfs" },
 { 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, 0 }
 };
 
@@ -646,14 +615,14 @@ char *cd_dialog (void)
 #ifdef HAVE_TK
 #define INPUT_INDEX 2
     { quick_button, 0, 1, 0, 1, "Cancel", 'c', 0, B_CANCEL, 0, 0,
-	  XV_WLAY_DONTCARE, "b." },
+	  XV_WLAY_DONTCARE, "cancel" },
     { quick_button, 0, 1, 0, 1, "Ok", 'o', 0, B_ENTER, 0, 0,
-	  XV_WLAY_DONTCARE, "" },
+	  XV_WLAY_DONTCARE, "ok" },
 #else
 #define INPUT_INDEX 0
 #endif
-    { quick_input,  8, 80, 5, 0, 0, 0, 50, 0, 0, 0, XV_WLAY_RIGHTOF, 0 },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, 0 },
+    { quick_input,  8, 80, 5, 0, 0, 0, 50, 0, 0, 0, XV_WLAY_RIGHTOF, "input" },
+    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
     { 0 } };
     
     char *my_str;
@@ -688,16 +657,16 @@ void symlink_dialog (char *existing, char *new, char **ret_existing,
 #ifdef HAVE_TK
 #define INPUT_INDEX 2
     { quick_button, 0, 1, 0, 1, "Cancel", 'c', 0, B_CANCEL, 0, 0,
-	  XV_WLAY_DONTCARE, "b." },
+	  XV_WLAY_DONTCARE, "cancel" },
     { quick_button, 0, 1, 0, 1, "Ok", 'o', 0, B_ENTER, 0, 0,
-	  XV_WLAY_DONTCARE, "" },
+	  XV_WLAY_DONTCARE, "ok" },
 #else
 #define INPUT_INDEX 0
 #endif
-    { quick_input,  4, 80, 5, 8, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, 0 },
-    { quick_label,  4, 80, 4, 8, 0, 0, 0, 0, 0, 0, XV_WLAY_BELOWOF, 0 },
-    { quick_input,  4, 80, 3, 8, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, 0 },
-    { quick_label,  4, 80, 2, 8, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, 0 },
+    { quick_input,  4, 80, 5, 8, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-1" },
+    { quick_label,  4, 80, 4, 8, 0, 0, 0, 0, 0, 0, XV_WLAY_BELOWOF, "label-1" },
+    { quick_input,  4, 80, 3, 8, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-2" },
+    { quick_label,  4, 80, 2, 8, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-2" },
     { 0 } };
     
     Quick_input.xlen  = 64;
