@@ -19,6 +19,7 @@
 #include <config.h>
 #include <string.h>
 #include <stdio.h>
+#include <errno.h>	/* For errno on SunOS systems	      */
 /* Needed for the extern declarations of integer parameters */
 #include <sys/types.h>
 #include <sys/param.h>
@@ -44,6 +45,7 @@
 #include "chmod.h"
 #include "achown.h"
 #include "chown.h"
+#include "../vfs/vfs.h"
 
 static int single_set;
 struct Dlg_head *ch_dlg;
@@ -274,7 +276,7 @@ static void init_chmod (void)
     x_set_dialog_title (ch_dlg, "Chmod command");
 
 #define XTRACT(i) BY+chmod_but[i].y-single_set, BX+chmod_but[i].x, \
-                  chmod_but[i].ret_cmd, chmod_but[i].text, chmod_but[i].hkey,                    chmod_but[i].hpos, 0, 0
+                  chmod_but[i].ret_cmd, chmod_but[i].text, chmod_but[i].hkey,                    chmod_but[i].hpos, 0, 0, NULL
 
     tk_new_frame (ch_dlg, "b.");
     for (i = 0; i < BUTTONS; i++) {
@@ -285,7 +287,7 @@ static void init_chmod (void)
     }
 
 
-#define XTRACT2(i) 0, check_perm [i].text, 0,-1
+#define XTRACT2(i) 0, check_perm [i].text, 0,-1, NULL
     tk_new_frame (ch_dlg, "c.");
     for (i = 0; i < PERMISSIONS; i++) {
 	check_perm[i].check = check_new (PY + (PERMISSIONS - i), PX + 2,
@@ -296,7 +298,7 @@ static void init_chmod (void)
 
 int stat_file (char *filename, struct stat *st)
 {
-    if (stat (filename, st))
+    if (mc_stat (filename, st))
 	return 0;
     if (!(S_ISREG(st->st_mode) || S_ISDIR(st->st_mode) ||
 	  S_ISLNK(st->st_mode)))
@@ -324,7 +326,10 @@ static void do_chmod (struct stat *sf)
 {
     sf->st_mode &= and_mask;
     sf->st_mode |= or_mask;
-    chmod (cpanel->dir.list [c_file].fname, sf->st_mode);
+    if (mc_chmod (cpanel->dir.list [c_file].fname, sf->st_mode) == -1)
+	message (1, " Error ", " Couldn't chmod \"%s\" \n %s ",
+	     cpanel->dir.list [c_file].fname, unix_error_string (errno));
+
     file_mark (cpanel, c_file, 0);
 }
 
@@ -354,6 +359,18 @@ void chmod_cmd (void)
     int i;
     struct stat sf_stat;
 
+    if (!vfs_current_is_local ()) {
+	if (vfs_current_is_extfs ()) {
+	    message (1, " Oops... ",
+		     " I can't run the Chmod command on an extfs ");
+	    return;
+	} else if (vfs_current_is_tarfs ()) {
+	    message (1, " Oops... ",
+		     " I can't run the Chmod command on a tarfs ");
+	    return;
+	}
+    }
+
     do {			/* do while any files remaining */
 	init_chmod ();
 	if (cpanel->marked)
@@ -361,8 +378,10 @@ void chmod_cmd (void)
 	else
 	    fname = selection (cpanel)->fname;	/* single file */
 
-	if (!stat_file (fname, &sf_stat))	/* get status of file */
+	if (!stat_file (fname, &sf_stat)){	/* get status of file */
+	    destroy_dlg (ch_dlg);
 	    break;
+	}
 	
 	c_stat = sf_stat.st_mode;
 	mode_change = 0;	/* clear changes flag */
@@ -376,13 +395,13 @@ void chmod_cmd (void)
 	tk_new_frame (ch_dlg, "l.");
 	/* Set the labels */
 	c_fname = name_trunc (fname, 21);
-	add_widget (ch_dlg, label_new (FY+2, FX+2, c_fname));
+	add_widget (ch_dlg, label_new (FY+2, FX+2, c_fname, NULL));
 	c_fown = name_trunc (get_owner (sf_stat.st_uid), 21);
-	add_widget (ch_dlg, label_new (FY+6, FX+2, c_fown));
+	add_widget (ch_dlg, label_new (FY+6, FX+2, c_fown, NULL));
 	c_fgrp = name_trunc (get_group (sf_stat.st_gid), 21);
-	add_widget (ch_dlg, label_new (FY+8, FX+2, c_fgrp));
+	add_widget (ch_dlg, label_new (FY+8, FX+2, c_fgrp, NULL));
 	sprintf (buffer, "%o", c_stat);
-	statl = label_new (FY+4, FX+2, buffer);
+	statl = label_new (FY+4, FX+2, buffer, NULL);
 	add_widget (ch_dlg, statl);
 	tk_end_frame ();
 	
@@ -392,7 +411,9 @@ void chmod_cmd (void)
 	switch (ch_dlg->ret_value){
 	case B_ENTER:
 	    if (mode_change)
-		chmod (fname, c_stat);
+		if (mc_chmod (fname, c_stat) == -1)
+		    message (1, " Error ", " Couldn't chmod \"%s\" \n %s ",
+	 		 fname, unix_error_string (errno));
 	    need_update = 1;
 	    break;
 	    
