@@ -20,6 +20,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>	/* For malloc() */
+#include <errno.h>	/* For errno on SunOS systems	      */
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -47,6 +48,7 @@
 #include "main.h"
 #include "chown.h"
 #include "wtools.h"		/* For init_box_colors */
+#include "../vfs/vfs.h"
 
 #define UX		5
 #define UY		2
@@ -169,14 +171,14 @@ static void init_chown (void)
     ch_dlg = create_dlg (0, 0, 18, 74, dialog_colors, chown_callback,
 			 "[Chown]", "chown", DLG_CENTER);
 
-#define XTRACT(i) BY+chown_but[i].y, BX+chown_but[i].x, chown_but[i].ret_cmd, chown_but[i].text, chown_but[i].hkey, chown_but[i].hpos, 0, 0
+#define XTRACT(i) BY+chown_but[i].y, BX+chown_but[i].x, chown_but[i].ret_cmd, chown_but[i].text, chown_but[i].hkey, chown_but[i].hpos, 0, 0, NULL
 
     tk_new_frame (ch_dlg, "b.");
     for (i = 0; i < BUTTONS-single_set; i++)
 	add_widget (ch_dlg, button_new (XTRACT (i)));
 
     /* Add the widgets for the file information */
-#define LX(i) chown_label [i].y, chown_label [i].x, ""
+#define LX(i) chown_label [i].y, chown_label [i].x, "", NULL
     tk_new_frame (ch_dlg, "l.");
     for (i = 0; i < LABELS; i++){
 	chown_label [i].l = label_new (LX (i));
@@ -184,8 +186,8 @@ static void init_chown (void)
     }
 
     /* get new listboxes */
-    l_user = listbox_new (UY + 1, UX + 1, 19, 10, 0, l_call);
-    l_group = listbox_new (GY + 1, GX + 1, 19, 10, 0, l_call);
+    l_user = listbox_new (UY + 1, UX + 1, 19, 10, 0, l_call, NULL);
+    l_group = listbox_new (GY + 1, GX + 1, 19, 10, 0, l_call, NULL);
 
     listbox_add_item (l_user, 0, 0, "<Unknown user>", NULL);	/* add fields for unknown names (numbers) */
     listbox_add_item (l_group, 0, 0, "<Unknown group>", NULL);
@@ -216,7 +218,10 @@ void chown_done (void)
 
 static inline void do_chown (uid_t u, gid_t g)
 {
-    chown (cpanel->dir.list [current_file].fname, u, g);
+    if (mc_chown (cpanel->dir.list [current_file].fname, u, g) == -1)
+	message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
+	     cpanel->dir.list [current_file].fname, unix_error_string (errno));
+
     file_mark (cpanel, current_file, 0);
 }
 
@@ -247,6 +252,18 @@ void chown_cmd (void)
     gid_t new_group;
     char  buffer [15];
 
+    if (!vfs_current_is_local ()) {
+	if (vfs_current_is_extfs ()) {
+	    message (1, " Oops... ",
+		     " I can't run the Chown command on an extfs ");
+	    return;
+	} else if (vfs_current_is_tarfs ()) {
+	    message (1, " Oops... ",
+		     " I can't run the Chown command on a tarfs ");
+	    return;
+	}
+    }
+
     do {			/* do while any files remaining */
 	init_chown ();
 	new_user = new_group = -1;
@@ -256,8 +273,10 @@ void chown_cmd (void)
 	else
 	    fname = selection (cpanel)->fname;	        /* single file */
 	
-	if (!stat_file (fname, &sf_stat))	/* get status of file */
+	if (!stat_file (fname, &sf_stat)){	/* get status of file */
+	    destroy_dlg (ch_dlg);
 	    break;
+	}
 	
 	/* select in listboxes */
 	fe = listbox_search_text (l_user, get_owner(sf_stat.st_uid));
@@ -310,7 +329,9 @@ void chown_cmd (void)
 	    new_user = (getpwnam (l_user->current->text))->pw_uid;
 	    if (ch_dlg->ret_value==B_ENTER) {
 		need_update = 1;
-		chown (fname, new_user, new_group);
+		if (mc_chown (fname, new_user, new_group) == -1)
+		    message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
+	 		 fname, unix_error_string (errno));
 	    } else
 		apply_chowns (new_user, new_group);
 	    break;
