@@ -82,6 +82,13 @@ int output_starts_shell = 0;
 /* Source routing destination */
 int source_route = 0;
 
+/* If set, use the builtin editor */
+int use_internal_edit = 0;
+
+#ifndef HAVE_TK
+#    define x_filter_changed(p)
+#endif
+
 /* This is used since the parameter panel on some of the commands */
 /* defined in this file may receive a 0 parameter if they are invoked */
 /* The drop down menu */
@@ -104,7 +111,7 @@ WPanel *get_a_panel (WPanel *panel)
  *               always invokes the internal viewer.
  *   internal:   If set uses the internal viewer, otherwise an external viewer.
  */
-int view_file (char *filename, int plain_view, int internal)
+int view_file_at_line (char *filename, int plain_view, int internal, int start_line)
 {
     static char *viewer = 0;
     int move_dir = 0;
@@ -122,7 +129,7 @@ int view_file (char *filename, int plain_view, int internal)
             changed_nroff_flag = 1;
         default_hex_mode = 0;
         default_nroff_flag = 0;
-        view (0, filename, &move_dir);
+        view (0, filename, &move_dir, start_line);
         if (changed_hex_mode && !altered_hex_mode)
             default_hex_mode = 1;
         if (changed_nroff_flag && !altered_nroff_flag)
@@ -131,8 +138,15 @@ int view_file (char *filename, int plain_view, int internal)
         return move_dir;
     }
     if (internal){
-	if (!regex_command (filename, "View", NULL, &move_dir)){
-	    view (0, filename, &move_dir);
+	char view_entry [32];
+
+	if (start_line != 0)
+	    sprintf (view_entry, "View:%d", start_line);
+	else
+	    strcpy (view_entry, "View");
+	
+	if (!regex_command (filename, view_entry, NULL, &move_dir)){
+	    view (0, filename, &move_dir, start_line);
 	    repaint_screen ();
 	}
     } else {
@@ -158,6 +172,12 @@ int view_file (char *filename, int plain_view, int internal)
     return move_dir;
 }
 
+int
+view_file (char *filename, int plain_view, int internal)
+{
+    return view_file_at_line (filename, plain_view, internal, 0);
+}
+
 /* scan_for_file (panel, idx, direction)
  *
  * Inputs:
@@ -234,7 +254,7 @@ void filtered_view_cmd (WPanel *panel)
     if (!command)
 	return;
 
-    view (command, "", 0);
+    view (command, "", 0, 0);
 
     free (command);
 }
@@ -248,6 +268,12 @@ void do_edit (const char *what)
 {
     static char *editor = 0;
 
+#ifdef USE_INTERNAL_EDIT
+    if (use_internal_edit){
+	edit (what);
+	return;
+    }
+#endif
     if (!editor){
 	editor = getenv ("EDITOR");
 	if (!editor)
@@ -263,6 +289,12 @@ void edit_cmd (WPanel *panel)
         do_edit (selection (panel)->fname);
 }
 
+void edit_cmd_new (WPanel *panel)
+{
+    panel = get_a_panel(panel);
+    do_edit ("");
+}
+
 void copy_cmd (void)
 {
     save_cwds_stat ();
@@ -327,35 +359,37 @@ void find_cmd (void)
     do_find ();
 }
 
-void filter_cmd (void)
+/* Set a given panel filter expression */
+void set_panel_filter (WPanel *p)
 {
     char *reg_exp;
     char *x;
+    
+    x = p->filter ? p->filter : easy_patterns ? "*" : ".";
+	
+    reg_exp = input_dialog (" Filter ", "Set expression for filtering filenames", x);
+    if (!reg_exp)
+	return;
+    if (p->filter){
+	free (p->filter);
+	p->filter = 0;
+    }
+    if (!(reg_exp [0] == '*' && reg_exp [1] == 0))
+	p->filter = reg_exp;
+    reread_cmd ();
+    x_filter_changed (p);
+}
+
+/* Invoked from the left/right menus */
+void filter_cmd (void)
+{
     WPanel *p;
 
     if (!SELECTED_IS_PANEL)
 	return;
 
     p = MENU_PANEL;
-
-    x = p->filter ? p->filter : easy_patterns ? "*" : ".";
-	
-    reg_exp = input_dialog (" Filter ", "", x);
-    mad_check (__FILE__, __LINE__);
-    if (!reg_exp)
-	return;
-    mad_check (__FILE__, __LINE__);
-    if (p->filter){
-	mad_check (__FILE__, __LINE__);
-	free (p->filter);
-	mad_check (__FILE__, __LINE__);
-	p->filter = 0;
-    }
-    mad_check (__FILE__, __LINE__);
-    if (!(reg_exp [0] == '*' && reg_exp [1] == 0))
-	p->filter = reg_exp;
-    mad_check (__FILE__, __LINE__);
-    reread_cmd ();
+    set_panel_filter (p);
 }
 
 void reread_cmd (void)
@@ -605,7 +639,7 @@ void quick_chdir_cmd (void)
 {
     char *target;
 
-    target = hotlist_cmd (0);
+    target = hotlist_cmd (LIST_HOTLIST);
     if (!target)
 	return;
 
@@ -613,6 +647,7 @@ void quick_chdir_cmd (void)
 	tree_chdir (the_tree, target);
     else
 	do_cd (target, cd_exact);
+    free (target);
 }
 
 #ifdef USE_VFS
@@ -620,7 +655,7 @@ void reselect_vfs (void)
 {
     char *target;
 
-    target = hotlist_cmd (1);
+    target = hotlist_cmd (LIST_VFSLIST);
     if (!target)
 	return;
 
@@ -659,7 +694,12 @@ static int compare_files (char *name1, char *name2, long size)
     return result;
 }
 
-static void compare_dir (WPanel *panel, WPanel *other, int thorough_flag)
+enum CompareMode {
+    compare_quick, compare_size_only, compare_thourough
+};
+
+static void
+compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 {
     int i, j;
     char *src_name, *dst_name;
@@ -695,28 +735,41 @@ static void compare_dir (WPanel *panel, WPanel *other, int thorough_flag)
 	    /* Found */
 	    file_entry *target = &other->dir.list[j];
 
-	    /* Older version is not marked */
-	    if (source->buf.st_mtime < target->buf.st_mtime)
-		continue;
+	    if (mode != compare_size_only){
+		/* Older version is not marked */
+		if (source->buf.st_mtime < target->buf.st_mtime)
+		    continue;
+	    }
+	    
 	    /* Newer version with different size is marked */
 	    if (source->buf.st_size != target->buf.st_size){
 		file_mark (panel, i, 1);
-	    } else if (!thorough_flag){
+	        panel->marked++;
+		panel->total += source->buf.st_size;
+		continue;
+		
+	    }
+	    if (mode == compare_size_only)
+		continue;
+	    
+	    if (mode == compare_quick){
 		/* Thorough compare off, compare only time stamps */
 		/* Mark newer version, don't mark version with the same date */
-		if (source->buf.st_mtime > target->buf.st_mtime)
+		if (source->buf.st_mtime > target->buf.st_mtime){
 		    file_mark (panel, i, 1);
-		else
-		    continue;
-	    } else {
-		/* Thorough compare on, do byte-by-byte comparison */
-		src_name = get_full_name (panel->cwd, source->fname);
-		dst_name = get_full_name (other->cwd, target->fname);
-		if (compare_files (src_name, dst_name, source->buf.st_size))
-		    file_mark (panel, i, 1);
-		free (src_name);
-		free (dst_name);
+		    panel->marked++;
+		    panel->total += source->buf.st_size;
+		}
+		continue;
 	    }
+
+	    /* Thorough compare on, do byte-by-byte comparison */
+	    src_name = get_full_name (panel->cwd, source->fname);
+	    dst_name = get_full_name (other->cwd, target->fname);
+	    if (compare_files (src_name, dst_name, source->buf.st_size))
+		file_mark (panel, i, 1);
+	    free (src_name);
+	    free (dst_name);
 	}
 
 	/* If marked increase counts */
@@ -729,12 +782,12 @@ static void compare_dir (WPanel *panel, WPanel *other, int thorough_flag)
 
 void compare_dirs_cmd (void)
 {
-    int thorough_flag = 0;
+    enum CompareMode thorough_flag = compare_quick;
 
 #ifdef HAVE_MMAP
     thorough_flag = query_dialog (" Compare directories ", " Select compare method: ",
-				  0, 3, " Quick ", " Thorough ", " Cancel ");
-    if (thorough_flag < 0 || thorough_flag > 1)
+				  0, 3, " Quick ", " Size only ", " Thorough ", " Cancel ");
+    if (thorough_flag < 0 || thorough_flag > 2)
 	return;
 #endif
     if (get_current_type () == view_listing &&
@@ -839,7 +892,7 @@ void view_other_cmd (void)
 	if (use_subshell){
 	    new_dir_p = vfs_current_is_local () ? &new_dir : NULL;
 	    if (invoke_subshell (NULL, VISIBLY, new_dir_p))
-		return;  /* User did `exit' or `logout': quit MC quietly */
+		quiet_quit_cmd();  /* User did `exit' or `logout': quit MC quietly */
 	} else
 #endif
 	{
@@ -878,7 +931,11 @@ void view_other_cmd (void)
 	}
 #endif
         touchwin (stdscr);
-	repaint_screen ();
+
+	/* prevent screen flash when user did 'exit' or 'logout' within
+	   subshell */
+	if (!quit)
+	    repaint_screen ();
     }
 }
 
@@ -1140,7 +1197,11 @@ static void nice_cd (char *text, char *xtext, char *help, char *prefix, int to_h
     if (!machine)
 	return;
 
-    cd_path = copy_strings (prefix, machine, to_home ? "/~/" : NULL, NULL);
+    if (strncmp (prefix, machine, strlen (prefix)) == 0)
+	cd_path = copy_strings (machine, to_home ? "/~/" : NULL, NULL);
+    else 
+	cd_path = copy_strings (prefix, machine, to_home ? "/~/" : NULL, NULL);
+    
     do_panel_cd (MENU_PANEL, cd_path, 0);
     free (cd_path);
     free (machine);
@@ -1269,7 +1330,7 @@ void dirsizes_cmd (void)
                 break;
         }
         if (pclose (f) < 0)
-	    message (2, "Show directory sizes", "Pipe close failed");
+	    message (0, "Show directory sizes", "Pipe close failed");
 	else
 	    panel->has_dir_sizes = 1;
 	close_error_pipe (0, 0);
