@@ -12,6 +12,7 @@ void filtered_view_cmd (WPanel *panel);
 void filtered_view_cmd_cpanel (void);
 void do_edit (const char *what);
 void edit_cmd (WPanel *panel);
+void edit_cmd_new (WPanel *panel);
 void copy_cmd (void);
 void ren_cmd (void);
 void reselect_vfs (void);
@@ -21,6 +22,7 @@ void delete_cmd (void);
 void find_cmd (void);
 void tree_mode_cmd (void);
 void filter_cmd (void);
+void set_panel_filter (WPanel *panel);
 void reread_cmd (void);
 void do_re_sort (WPanel *panel);
 void quick_view_cmd (void);
@@ -46,4 +48,6 @@ void view_panel_cmd (void);
 void quick_cd_cmd (void);
 char *get_random_hint (void);
 void source_routing (void);
+
+extern int use_internal_edit;
 #endif /* __CMD_H */
