@@ -226,7 +226,7 @@ static int enter (WCommand *cmdline)
 	    free (command);
 	    
 #ifdef HAVE_SUBSHELL_SUPPORT
-	    if (quit == SUBSHELL_EXIT){
+	    if (quit & SUBSHELL_EXIT){
 	        quiet_quit_cmd ();
 		return MSG_HANDLED;
 	    }
@@ -258,9 +258,12 @@ static int command_callback (Dlg_head *h, WCommand *cmd, int msg, int par)
 
 WCommand *command_new (int y, int x, int cols)
 {
+    WInput *in;
     WCommand *cmd = xmalloc (sizeof (WCommand), "command_new");
 
-    cmd->input = *input_new (y, x, DEFAULT_COLOR, cols, "");
+    in = input_new (y, x, DEFAULT_COLOR, cols, "", NULL);
+    cmd->input = *in;
+    free (in);
 
     /* Add our hooks */
     cmd->old_callback = (callback_fn) cmd->input.widget.callback;
