@@ -12,8 +12,8 @@
     };
 
     extern signed char console_flag;
-    void show_console_contents (int starty, int begin_line, int end_line);
-    void handle_console (int action);
+    void show_console_contents (int starty, unsigned char begin_line, unsigned char end_line);
+    void handle_console (unsigned char action);
 
 #else
 
