@@ -51,7 +51,7 @@ static int case_sensitive = 1;
 
 #define MY_ISDIR(x) ( (S_ISDIR (x->buf.st_mode) || x->f.link_to_dir) ? 1 : 0)
 
-sort_orders_t sort_orders [SORT_TYPES] = {
+sort_orders_t sort_orders [SORT_TYPES_TOTAL] = {
     { "Unsorted",    unsorted },
     { "Name",        sort_name },
     { "Extension",   sort_ext },
@@ -59,17 +59,27 @@ sort_orders_t sort_orders [SORT_TYPES] = {
     { "Access time", sort_atime },
     { "Change time", sort_ctime },
     { "Size",        sort_size },
-    { "Inode",       sort_inode }
+    { "Inode",       sort_inode },
+
+    /* New sort orders */
+    { "Type",        sort_type },
+    { "Links",       sort_links },
+    { "NGID",        sort_ngid },
+    { "NUID",        sort_nuid },
+    { "Owner",       sort_owner },
+    { "Group",       sort_group }
 };
 
-#define string_sortcomp(a,b) case_sensitive ? strcmp (a,b) : strcasecmp (a,b)
+#define string_sortcomp(a,b) (case_sensitive ? strcmp (a,b) : strcasecmp (a,b))
 
-int unsorted (const file_entry *a, const file_entry *b)
+int
+unsorted (const file_entry *a, const file_entry *b)
 {
     return 0;
 }
 
-int sort_name (const file_entry *a, const file_entry *b)
+int
+sort_name (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -79,7 +89,8 @@ int sort_name (const file_entry *a, const file_entry *b)
     return bd-ad;
 }
 
-int sort_ext (const file_entry *a, const file_entry *b)
+int
+sort_ext (const file_entry *a, const file_entry *b)
 {
     char *exta, *extb;
     int r;
@@ -98,7 +109,30 @@ int sort_ext (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-int sort_time (const file_entry *a, const file_entry *b)
+int
+sort_owner (const file_entry *a, const file_entry *b)
+{
+    int ad = MY_ISDIR (a);
+    int bd = MY_ISDIR (b);
+    
+    if (ad == bd || mix_all_files)
+	return string_sortcomp (get_owner (a->buf.st_uid), get_owner (a->buf.st_uid)) * reverse;
+    return bd-ad;
+}
+
+int
+sort_group (const file_entry *a, const file_entry *b)
+{
+    int ad = MY_ISDIR (a);
+    int bd = MY_ISDIR (b);
+    
+    if (ad == bd || mix_all_files)
+	return string_sortcomp (get_group (a->buf.st_gid), get_group (a->buf.st_gid)) * reverse;
+    return bd-ad;
+}
+
+int
+sort_time (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -109,7 +143,8 @@ int sort_time (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-int sort_ctime (const file_entry *a, const file_entry *b)
+int
+sort_ctime (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -120,7 +155,8 @@ int sort_ctime (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-int sort_atime (const file_entry *a, const file_entry *b)
+int
+sort_atime (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -131,7 +167,8 @@ int sort_atime (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-int sort_inode (const file_entry *a, const file_entry *b)
+int
+sort_inode (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -142,7 +179,8 @@ int sort_inode (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-int sort_size (const file_entry *a, const file_entry *b)
+int
+sort_size (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -153,8 +191,97 @@ int sort_size (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-void do_sort (dir_list *list, sortfn *sort, int top, int reverse_f, int case_sensitive_f)
+int
+sort_links (const file_entry *a, const file_entry *b)
 {
+    int ad = MY_ISDIR (a);
+    int bd = MY_ISDIR (b);
+
+    if (ad == bd || mix_all_files)
+	return (b->buf.st_nlink - a->buf.st_nlink) * reverse;
+    else
+	return bd-ad;
+}
+
+int
+sort_ngid (const file_entry *a, const file_entry *b)
+{
+    int ad = MY_ISDIR (a);
+    int bd = MY_ISDIR (b);
+
+    if (ad == bd || mix_all_files)
+	return (b->buf.st_gid - a->buf.st_gid) * reverse;
+    else
+	return bd-ad;
+}
+
+int
+sort_nuid (const file_entry *a, const file_entry *b)
+{
+    int ad = MY_ISDIR (a);
+    int bd = MY_ISDIR (b);
+
+    if (ad == bd || mix_all_files)
+	return (b->buf.st_uid - a->buf.st_uid) * reverse;
+    else
+	return bd-ad;
+}
+
+inline static int
+file_type_to_num (const file_entry *fe)
+{
+    const struct stat *s = &fe->buf;
+    
+    if (S_ISDIR (s->st_mode))
+	return 0;
+    if (S_ISLNK (s->st_mode)){
+	if (fe->f.link_to_dir)
+	    return 1;
+	if (fe->f.stalled_link)
+	    return 2;
+	else
+	    return 3;
+    }
+    if (S_ISSOCK (s->st_mode))
+	return 4;
+    if (S_ISCHR (s->st_mode))
+	return 5;
+    if (S_ISBLK (s->st_mode))
+	return 6;
+    if (S_ISFIFO (s->st_mode))
+	return 7;
+    if (is_exe (s->st_mode))
+	return 8;
+    return 9;
+}
+
+int
+sort_type (const file_entry *a, const file_entry *b)
+{
+    int aa  = file_type_to_num (a);
+    int bb  = file_type_to_num (b);
+    
+    return bb-aa;
+}
+
+
+void
+do_sort (dir_list *list, sortfn *sort, int top, int reverse_f, int case_sensitive_f)
+{
+    int i;
+    file_entry tmp_fe;
+
+    for (i = 0; i < top + 1; i++) {             /* put ".." first in list */
+	if (!strcmp (list->list [i].fname, "..")) {
+            if (i > 0) {                        /* swap [i] and [0] */
+                memcpy (&tmp_fe, &(list->list [0]), sizeof (file_entry));
+                memcpy (&(list->list [0]), &(list->list [i]), sizeof (file_entry));
+                memcpy (&(list->list [i]), &tmp_fe, sizeof (file_entry));
+            }
+            break;
+        }
+    }
+
     reverse = reverse_f ? -1 : 1;
     case_sensitive = case_sensitive_f;
     qsort (&(list->list) [1], top, sizeof (file_entry), sort);
@@ -363,8 +490,10 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
 
     start_tree_check (NULL);
     dirp = mc_opendir (".");
-    if (!dirp)
+    if (!dirp) {
+ 	clean_dir (list, count);
 	return set_zero_dir (list);
+    }
 
     alloc_dir_copy (list->size);
     for (i = 0; i < count; i++){
@@ -381,8 +510,20 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
 	    &stalled_link);
 	if (status == 0)
 	    continue;
-	if (status == -1)
+	if (status == -1) {
+	    mc_closedir (dirp);
+	    /* Norbert (Feb 12, 1997): 
+	     Just in case someone finds this memory leak:
+	     -1 means big trouble (at the moment no memory left), 
+	     I don't bother with further cleanup because if one gets to
+	     this point he will have more problems than a few memory
+	     leaks and because one 'clean_dir' would not be enough (and
+	     because I don't want to spent the time to make it working, 
+             IMHO it's not worthwhile). 
+	    clean_dir (&dir_copy, count);
+             */
 	    return next_free;
+	}
 	
 	tmp_len = NLENGTH (dp);
 	for (found = i = 0; i < count; i++)
