@@ -41,22 +41,36 @@ int handle_dirent (dir_list *list, char *filter, struct dirent *dp,
 #endif
 
 /* Sorting functions */
-int unsorted (const file_entry *a, const file_entry *b);
-int sort_name (const file_entry *a, const file_entry *b);
-int sort_ext (const file_entry *a, const file_entry *b);
-int sort_time (const file_entry *a, const file_entry *b);
+int unsorted   (const file_entry *a, const file_entry *b);
+int sort_name  (const file_entry *a, const file_entry *b);
+int sort_ext   (const file_entry *a, const file_entry *b);
+int sort_time  (const file_entry *a, const file_entry *b);
 int sort_atime (const file_entry *a, const file_entry *b);
 int sort_ctime (const file_entry *a, const file_entry *b);
-int sort_size (const file_entry *a, const file_entry *b);
+int sort_size  (const file_entry *a, const file_entry *b);
 int sort_inode (const file_entry *a, const file_entry *b);
+int sort_type  (const file_entry *a, const file_entry *b);
+int sort_links (const file_entry *a, const file_entry *b);
+int sort_nuid  (const file_entry *a, const file_entry *b);
+int sort_ngid  (const file_entry *a, const file_entry *b);
+int sort_owner (const file_entry *a, const file_entry *b);
+int sort_group (const file_entry *a, const file_entry *b);
 
+/* SORT_TYPES is used to build the nice dialog box entries */
 #define SORT_TYPES 8
+
+/* This is the number of sort types not available in that dialog box */
+#define SORT_TYPES_EXTRA 6
+
+/* The total, used by Tk version */
+#define SORT_TYPES_TOTAL (SORT_TYPES + SORT_TYPES_EXTRA)
+
 typedef struct {
     char    *sort_name;
     int     (*sort_fn)(const file_entry *, const file_entry *);
 } sort_orders_t;
 
-extern sort_orders_t sort_orders [SORT_TYPES];
+extern sort_orders_t sort_orders [SORT_TYPES_TOTAL];
 
 int link_isdir (file_entry *);
 int if_link_is_exe (file_entry *file);
