@@ -134,7 +134,7 @@ void dlg_erase (Dlg_head *h)
 
 void init_widget (Widget *w, int y, int x, int lines, int cols,
 		  int (*callback)(Dlg_head *, void *, int, int),
-		  destroy_fn destroy, mouse_h mouse_handler)
+		  destroy_fn destroy, mouse_h mouse_handler, char *tkname)
 {
     w->x = x;
     w->y = y;
@@ -148,6 +148,13 @@ void init_widget (Widget *w, int y, int x, int lines, int cols,
     w->wcontainer = 0;
     w->frame = "";
     w->parent = 0;
+    w->tkname = tkname;
+
+    if (tkname && *tkname == 0){
+	fprintf (stderr, "Got an null string for the tkname (pid=%d)\n", getpid ());
+	
+	abort ();
+    }
     /* Almost all widgets want to put the cursor in a suitable place */
     w->options = W_WANT_CURSOR; 
 }
@@ -239,7 +246,7 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
     new_d->initfocus = NULL;
 #ifdef HAVE_X
     if (callback != midnight_callback)
-        new_d->wdata = xtoolkit_create_dialog (new_d);
+        new_d->wdata = xtoolkit_create_dialog (new_d, flags & DLG_GRID);
     else
     	new_d->wdata = xtoolkit_get_main_dialog (new_d);
 #endif    
@@ -486,12 +493,13 @@ void update_cursor (Dlg_head *h)
     }
 }
 
+/* Redraw the widgets in reverse order, leaving the current widget
+ * as the last one
+ */
 void dlg_redraw (Dlg_head *h)
 {
     (h->callback)(h, 0, DLG_DRAW);
 
-    /* Redraw the widgets in reverse order, leaving the current widget
-       as the last one */
     dlg_broadcast_msg (h, WIDGET_DRAW, 1);
     
     update_cursor (h);
@@ -534,7 +542,7 @@ static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
 #else
 	touchwin (stdscr);
 #endif
-	refresh ();
+	mc_refresh ();
 	doupdate ();
 	break;
 	
