@@ -51,7 +51,8 @@ int use_file_to_check_type = 1;
  */
 static char *data = NULL;
 
-void flush_extension_file (void)
+void
+flush_extension_file (void)
 {
     if (data){
         free (data);
@@ -60,7 +61,8 @@ void flush_extension_file (void)
    
 }
 
-void exec_extension (char *filename, char *data, char *drops, int *move_dir)
+static void
+exec_extension (char *filename, char *data, char *drops, int *move_dir, int start_line)
 {
     char *file_name;
     FILE *cmd_file;
@@ -199,9 +201,9 @@ void exec_extension (char *filename, char *data, char *drops, int *move_dir)
 	 * into view
 	 */
     	if (written_nonspace) 
-    	    view (file_name, filename, move_dir);
+    	    view (file_name, filename, move_dir, start_line);
     	else
-    	    view (0, filename, move_dir);
+    	    view (0, filename, move_dir, start_line);
     	if (changed_hex_mode && !altered_hex_mode)
     	    default_hex_mode = def_hex_mode;
     	if (changed_nroff_flag && !altered_nroff_flag)
@@ -257,6 +259,9 @@ void exec_extension (char *filename, char *data, char *drops, int *move_dir)
  * If action == "Icon", we are doing again something special. We return
  * icon name and we set the variable regex_command_title to Title for
  * that icon.
+ *
+ * If action == "View" then a parameter is checked in the form of "View:%d",
+ * if the value for %d exists, then the viewer is started up at that line number.
  */
 char *regex_command_title = NULL;
 char *regex_command (char *filename, char *action, char *drops, int *move_dir)
@@ -272,13 +277,21 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir)
     int old_patterns;
     struct stat mystat;
     int asked_file;
-
+    int view_at_line_number;
+    
 #ifdef FILE_STDIN
     int file_supports_stdin = 1;
 #else
     int file_supports_stdin = 0;
 #endif    
-    
+
+    /* Check for the special View:%d parameter */
+    if (action && strncmp (action, "View:", 5) == 0){
+	view_at_line_number = atoi (action + 5);
+	action [4] = 0;
+    } else {
+	view_at_line_number = 0;
+    }
     /* Have we asked file for the file contents? */
     asked_file = 0;
 
@@ -361,7 +374,7 @@ file as an example of how to write it.\n\
 	        if (regexp_match (p, filename, match_normal))
 	            found = 1;
 	    } else if (!strncmp (p, "directory/", 10)) {
-	        if (S_ISDIR (mystat.st_mode) && regexp_match (p, filename, match_normal))
+	        if (S_ISDIR (mystat.st_mode) && regexp_match (p+10, filename, match_normal))
 	            found = 1;
 	    } else if (!strncmp (p, "shell/", 6)) {
 	        p += 6;
@@ -483,7 +496,7 @@ match_file_output:
     	                            to_return = xmalloc (512, "Action list");
     	                            q = to_return;
     	                        } else 
-    	                            *(q++) = ' ';
+    	                            *(q++) = '='; /* Mark separator */
     	                        strcpy (q, p);
     	                        q = strchr (q, 0);
     	                }
@@ -522,7 +535,7 @@ match_file_output:
     	                if (p < q) { 
 			    char *filename_copy = strdup (filename);
 			    
-    	                    exec_extension (filename_copy, r + 1, drops, move_dir);
+    	                    exec_extension (filename_copy, r + 1, drops, move_dir, view_at_line_number);
 			    free (filename_copy);
 			    
     	                    to_return = "Success";
