@@ -1,12 +1,29 @@
 /* {{{ Copyright */
 
-/* File managing
-   Copyright (C) 1994, 1995 The Free Software Foundation
+/* File managing.  Important notes on this file:
    
-   Written by: 1994, 1995 Janne Kukonlehto
-               1994, 1995 Fred Leeflang
-               1994, 1995 Miguel de Icaza
-               1995       Jakub Jelinek
+   About the use of dialogs in this file:
+     If you want to add a new dialog box (or call a routine that pops
+     up a dialog box), you have to provide a wrapper for background
+     operations (ie, background operations have to up-call to the parent
+     process).
+
+     For example, instead of using the message() routine, in this
+     file, you should use one of the stubs that call message with the
+     proper number of arguments (ie, message_1s, message_2s and so on).
+
+     Actually, that is a rule that should be followed by any routines
+     that may be called from this module.
+
+*/
+
+/* File managing
+   Copyright (C) 1994, 1995, 1996 The Free Software Foundation
+   
+   Written by: 1994, 1995       Janne Kukonlehto
+               1994, 1995       Fred Leeflang
+               1994, 1995, 1996 Miguel de Icaza
+               1995, 1996       Jakub Jelinek
 
    The copy code was based in GNU's cp, and was written by:
    Torbjorn Granlund, David MacKenzie, and Jim Meyering.
@@ -16,7 +33,7 @@
 
    Janne Kukonlehto added much error recovery to them for being used
    in an interactive program.
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -33,12 +50,12 @@
 
 /* }}} */
 
+/* {{{ Include files */
+
 #include <config.h>
 /* Hack: the vfs code should not rely on this */
 #define WITH_FULL_PATHS 1
 
-/* {{{ Include files */
-
 #include <sys/types.h>
 #include <dirent.h>
 #include <stdio.h>
@@ -75,14 +92,7 @@
 #include "layout.h"
 #include "widget.h"
 #include "wtools.h"
-
-/* This for the background code, we currenlty only support one
- * way of comunicating the background and foreground process
- * by using the socketpair system call
- */
-#ifdef USE_NETCODE
-#    include <sys/socket.h>
-#endif
+#include "background.h"
 
 /* Needed for current_panel, other_panel and WTree */
 #include "dir.h"
@@ -97,6 +107,24 @@
 /* rcsid [] = "$Id: file.c,v 1.12 1995/02/21 19:05:47 miguel Exp $" */
 int verbose = 1;
 
+/* Recursive operation on subdirectories */
+int dive_into_subdirs = 1;
+
+/* If running as root, preserve the original uid/gid on file copy */
+int preserve_uidgid = 1;
+
+/* If off, it gets a little scrict with dangerous operations */
+int know_what_am_i_doing = 1;
+
+int stable_symlinks = 0;
+
+/* The next two are not static, since they are used on background.c */
+/* Controls appending to files, shared with filequery.c */
+int do_append = 0;
+
+/* result from the recursive query */
+int recursive_result;
+
 /* This is a hard link cache */
 struct link {
     struct link *next;
@@ -113,14 +141,6 @@ struct re_registers regs;
 static char *dest_mask = NULL;
 static int op_follow_symlinks = 0;
 
-/* Recursive operation on subdirectories */
-int dive_into_subdirs = 1;
-
-/* If running as root, preserve the original uid/gid on file copy */
-int preserve_uidgid = 1;
-
-int stable_symlinks = 0;
-
 #define WX 60
 #define WY 10
 #define BY 10
@@ -133,8 +153,16 @@ static char *op_names [] = { " Copy ", " Move ", " Delete " };
 static int selected_button;
 static int last_percentage [3];
 
-enum { REPLACE_YES = B_USER, REPLACE_NO, REPLACE_APPEND, REPLACE_ALWAYS,
-	   REPLACE_UPDATE, REPLACE_NEVER, REPLACE_ABORT };
+/* Used for button result values */
+enum {
+    REPLACE_YES = B_USER,
+    REPLACE_NO,
+    REPLACE_APPEND,
+    REPLACE_ALWAYS,
+    REPLACE_UPDATE,
+    REPLACE_NEVER,
+    REPLACE_ABORT
+};
 
 static int replace_colors [4];
 static Dlg_head *replace_dlg;
@@ -142,12 +170,14 @@ static char *replace_filename;
 static struct stat *s_stat, *d_stat;
 static int replace_result;
 
-int know_what_am_i_doing = 1;
+enum {
+    RECURSIVE_YES,
+    RECURSIVE_NO,
+    RECURSIVE_ALWAYS,
+    RECURSIVE_NEVER,
+    RECURSIVE_ABORT
+};
 
-enum { RECURSIVE_YES, RECURSIVE_NO, RECURSIVE_ALWAYS,
-	   RECURSIVE_NEVER, RECURSIVE_ABORT };
-
-static int recursive_result;
 static int recursive_erase (char *s);
 
 /* Describe the components in the panel operations window */
@@ -172,7 +202,7 @@ static void update_buttons (void)
     attrset ((selected_button == FILE_ABORT) ? FOCUSC : NORMALC);
     dlg_move (op_dlg, BY-minus, WX - 19);
     addstr ("[ Abort ]");
-    refresh ();
+    mc_refresh ();
 #undef h
 #endif    
 }
@@ -282,38 +312,38 @@ void create_op_win (int op)
     x_set_dialog_title (op_dlg, "");
 
 #ifndef HAVE_X
-    add_widget (op_dlg, button_new (0, 0, 0, "", 0, -1, 0, 0));
+    add_widget (op_dlg, button_new (0, 0, 0, "", 0, -1, 0, 0, NULL));
 #else
     tk_new_frame (op_dlg, "b.");
-    add_widgetl (op_dlg, button_new (0, 0, FILE_ABORT, "Abort", 0, -1, 0, 0),
+    add_widgetl (op_dlg, button_new (0, 0, FILE_ABORT, "Abort", 0, -1, 0, 0, NULL),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, button_new (0, 0, FILE_SKIP, "Skip", 0, -1, 0, 0),
+    add_widgetl (op_dlg, button_new (0, 0, FILE_SKIP, "Skip", 0, -1, 0, 0, NULL),
         XV_WLAY_CENTERROW);
 #endif        
     tk_new_frame (op_dlg, "2.");
-    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (8, 14, 0, 100, 0), 
+    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (8, 14, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [2] = label_new (8, 5, fifteen), 
+    add_widgetl (op_dlg, ProgressLabel [2] = label_new (8, 5, fifteen, NULL), 
         XV_WLAY_NEXTROW);
     tk_new_frame (op_dlg, "1.");
-    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (7, 14, 0, 100, 0), 
+    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (7, 14, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [1] = label_new (7, 5, fifteen), 
+    add_widgetl (op_dlg, ProgressLabel [1] = label_new (7, 5, fifteen, NULL), 
         XV_WLAY_NEXTROW);
     tk_new_frame (op_dlg, "0.");
-    add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, 14, 0, 100, 0), 
+    add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, 14, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [0] = label_new (6, 5, fifteen), 
+    add_widgetl (op_dlg, ProgressLabel [0] = label_new (6, 5, fifteen, NULL), 
         XV_WLAY_NEXTROW);
     tk_new_frame (op_dlg, "f1.");
-    add_widgetl (op_dlg, FileString [1] = label_new (4, 14, sixty),
+    add_widgetl (op_dlg, FileString [1] = label_new (4, 14, sixty, NULL),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, FileLabel [1] = label_new (4, 5, fifteen), 
+    add_widgetl (op_dlg, FileLabel [1] = label_new (4, 5, fifteen, NULL), 
         XV_WLAY_NEXTROW);
     tk_new_frame (op_dlg, "f0.");
-    add_widgetl (op_dlg, FileString [0] = label_new (3, 14, sixty),
+    add_widgetl (op_dlg, FileString [0] = label_new (3, 14, sixty, NULL),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, FileLabel [0] = label_new (3, 5, fifteen), 
+    add_widgetl (op_dlg, FileLabel [0] = label_new (3, 5, fifteen, NULL), 
         XV_WLAY_NEXTROW);
 	
     /* We will manage the dialog without any help, that's why
@@ -437,217 +467,6 @@ static int show_bytes_progress (long done, long total)
 
 /* }}} */
 
-/* {{{ Query/status report routines */
-
-static int do_file_error (char *error)
-{
-    int result;
-
-    result = query_dialog (" Error ", error, 3, 3, " Skip ", " Retry ", " Abort ");
-
-    switch (result){
-    case 0:
-	do_refresh ();
-	return FILE_SKIP;
-    case 1:
-	do_refresh ();
-	return FILE_RETRY;
-    case 2:
-    default:
-	return FILE_ABORT;
-    }
-}
-
-/* Report error with one file */
-static int file_error (char *format, char *file)
-{
-    sprintf (cmd_buf, format,
-	     name_trunc (file, 30), unix_error_string (errno));
-    return do_file_error (cmd_buf);
-}
-
-/* Report error with two files */
-static int files_error (char *format, char *file1, char *file2)
-{
-    sprintf (cmd_buf, format, name_trunc (file1, 15),
-	     name_trunc (file2, 15), unix_error_string (errno));
-    return do_file_error (cmd_buf);
-}
-
-static char *format = "Target file \"%s\" already exists!";
-static int replace_callback (struct Dlg_head *h, int Id, int Msg)
-{
-#ifndef HAVE_X
-
-    switch (Msg){
-    case DLG_DRAW:
-	attrset (ERROR_COLOR);
-	dlg_erase (h);
-	draw_box (h, 1, 2, 12, 56);
-
-	dlg_move (h, 1, 24);
-	addstr (" File exists ");
-	break;
-    }
-#endif
-    return 0;
-}
-
-static void init_replace (void)
-{
-    char buffer [128];
-    
-    replace_colors [0] = ERROR_COLOR;
-    replace_colors [1] = REVERSE_COLOR;
-    replace_colors [2] = ERROR_COLOR;
-    replace_colors [3] = REVERSE_COLOR;
-    
-    replace_dlg = create_dlg (0, 0, 14, 60, replace_colors, replace_callback,
-			      "[Replace]", "replace", DLG_CENTER);
-			      
-    x_set_dialog_title (replace_dlg, "File exists");
-
-#ifdef HAVE_X
-#define X_TRUNC 128
-#else
-#define X_TRUNC 52
-#endif
-    sprintf (buffer, format, name_trunc (replace_filename, X_TRUNC - strlen (format)));
-    add_widgetl (replace_dlg, label_new (3, 5, buffer), XV_WLAY_CENTERROW);
-    
-    add_widgetl (replace_dlg,
-		button_new (BY + 1, 46, REPLACE_ABORT, "[ Abort ]", 'a', 2, 0, 0),
-		XV_WLAY_CENTERROW);
-    
-    tk_new_frame (replace_dlg, "a.");
-    add_widgetl (replace_dlg,
-		button_new (BY - 1, 47, REPLACE_NEVER, "[ nonE ]", 'e', 5, 0, 0),
-		XV_WLAY_RIGHTOF);
-    add_widgetl (replace_dlg,
-		button_new (BY - 1, 36, REPLACE_UPDATE, "[ Update ]", 'u', 2, 0, 0),
-		XV_WLAY_RIGHTOF);
-    add_widgetl (replace_dlg,
-		button_new (BY - 1, 28, REPLACE_ALWAYS, "[ alL ]", 'l', 4, 0, 0),
-		XV_WLAY_RIGHTOF);
-
-    add_widgetl (replace_dlg, label_new (BY-1, 5, "Overwrite all targets?"),
-	        XV_WLAY_CENTERROW);        
-
-    /* "this target..." widgets */
-    tk_new_frame (replace_dlg, "p.");
-    if (!S_ISDIR (d_stat->st_mode))
-        add_widgetl (replace_dlg,
-		    button_new (BY - 2, 43, REPLACE_APPEND, "[ apPend ]", 'p', 2, 0, 0),
-		    XV_WLAY_RIGHTOF);
-    add_widgetl (replace_dlg,
-		button_new (BY - 2, 36, REPLACE_NO, "[ No ]", 'n', 2, 0, 0),
-		XV_WLAY_RIGHTOF);
-    add_widgetl (replace_dlg,
-		button_new (BY - 2, 28, REPLACE_YES, "[ Yes ]", 'y', 2, 0, 0), 
-		XV_WLAY_RIGHTOF);
-    add_widgetl (replace_dlg, label_new (BY-2,5, "Overwrite this target?"),
-		 XV_WLAY_CENTERROW);
-    
-    tk_new_frame (replace_dlg, "i.");
-    sprintf (buffer, "Target date: %s, size %d",
-	     file_date (d_stat->st_mtime), (int) d_stat->st_size);
-    add_widgetl (replace_dlg, label_new (6, 5, buffer), XV_WLAY_CENTERROW);
-    sprintf (buffer, "Source date: %s, size %d",
-	     file_date (s_stat->st_mtime), (int) s_stat->st_size);
-    add_widgetl (replace_dlg, label_new (5, 5, buffer), XV_WLAY_CENTERROW);
-    tk_end_frame ();
-}
-
-static int do_append = 0;
-
-static int query_replace (char *destname, struct stat *_s_stat,
-			  struct stat *_d_stat)
-{
-    if (replace_result < REPLACE_ALWAYS){
-	replace_filename = destname;
-	s_stat = _s_stat;
-	d_stat = _d_stat;
-	init_replace ();
-	run_dlg (replace_dlg);
-	replace_result = replace_dlg->ret_value;
-	if (replace_result == B_CANCEL)
-	    replace_result = REPLACE_ABORT;
-	destroy_dlg (replace_dlg);
-    }
-
-    switch (replace_result){
-    case REPLACE_UPDATE:
-	do_refresh ();
-	if (s_stat->st_mtime > d_stat->st_mtime)
-	    return FILE_CONT;
-	else
-	    return FILE_SKIP;
-	
-    case REPLACE_APPEND:
-        do_append = 1;
-    case REPLACE_YES:
-    case REPLACE_ALWAYS:
-	do_refresh ();
-	return FILE_CONT;
-    case REPLACE_NO:
-    case REPLACE_NEVER:
-	do_refresh ();
-	return FILE_SKIP;
-    case REPLACE_ABORT:
-    default:
-	return FILE_ABORT;
-    }
-}
-
-int query_recursive (char *s)
-{
-    char *confirm, *text;
-
-    if (recursive_result < RECURSIVE_ALWAYS){
-	text = copy_strings (" Delete: ", name_trunc (s, 30), " ", 0);
-
-        if (know_what_am_i_doing){
-	    query_set_sel (1);
-	    recursive_result = query_dialog (text, "\n   Directory not empty.   "
-					     "\n   Delete it recursively? ",
-					     3 | WITH_HOTKEYS, 5,
-					     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
-	} else
-	    recursive_result = query_dialog (text, "\n   Directory not empty.   "
-					     "\n   Delete it recursively? ",
-					     3 | WITH_HOTKEYS, 5, 
-					     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
-	if (recursive_result != RECURSIVE_ABORT)
-	    do_refresh ();
-	free (text);
-	if (!know_what_am_i_doing && (recursive_result == RECURSIVE_YES
-	    || recursive_result == RECURSIVE_ALWAYS)){
-	    text = copy_strings (" Please type 'yes' to confirm recursive delete of ",
-				 recursive_result == RECURSIVE_YES
-				 ? name_trunc (s, 20) : "all the directories", " ", 0);
-	    confirm = input_dialog (" Recursive Delete ",
-				    text, "no");
-	    do_refresh ();
-	    if (!confirm || strcmp (confirm, "yes"))
-		recursive_result = RECURSIVE_NEVER;
-	    free (confirm);
-	    free (text);
-	}
-    }
-    switch (recursive_result){
-    case RECURSIVE_YES:
-    case RECURSIVE_ALWAYS:
-	return FILE_CONT;
-    case RECURSIVE_NO:
-    case RECURSIVE_NEVER:
-	return FILE_SKIP;
-    case RECURSIVE_ABORT:
-    default:
-	return FILE_ABORT;
-    }
-}
-
-/* }}} */
 
 /* {{{ Copy routines */
 
@@ -722,7 +541,7 @@ static char *do_transform_source (char *source)
 	case '*':
 	    if (next_reg < 0 || next_reg >= RE_NREGS
 		|| regs.start [next_reg] < 0) {
-		message (1, " Error ", " Invalid target mask ");
+		message_1s (1, " Error ", " Invalid target mask ");
 		transform_error = FILE_ABORT;
 		return NULL;
 	    }
@@ -839,7 +658,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     if (show_source (src_path) == FILE_ABORT
 	|| show_target (dst_path) == FILE_ABORT)
 	return FILE_ABORT;
-    refresh ();
+    mc_refresh ();
 
  retry_dst_stat:
     if (!mc_lstat (dst_path, &sb2)){
@@ -866,13 +685,13 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	/* Destination already exists */
 	if (sb.st_dev == sb2.st_dev
 	    && sb.st_ino == sb2.st_ino){
-	    message (1, " Error ", " `%s' and `%s' are the same file. ",
+	    message_3s (1, " Error ", " `%s' and `%s' are the same file. ",
 		     src_path, dst_path);
 	    do_refresh ();
 	    return FILE_SKIP;
 	}
 	if (S_ISDIR (sb2.st_mode)){
-	    message (1, " Error ", " Cannot overwrite directory `%s' ",
+	    message_2s (1, " Error ", " Cannot overwrite directory `%s' ",
 		     dst_path);
 	    do_refresh ();
 	    return FILE_SKIP;
@@ -1074,7 +893,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     }
 
     return_status = show_file_progress (0, file_size);
-    refresh ();
+    mc_refresh ();
     if (return_status != FILE_CONT)
 	goto ret3;
 
@@ -1104,7 +923,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	        goto ret3;
 	    }
 	    return_status = show_file_progress (n_read_total, file_size);
-	    refresh ();
+	    mc_refresh ();
 	    if (return_status != FILE_CONT)
 	        goto ret3;
         }
@@ -1129,7 +948,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
                     break;
             }
 	    if (size == MCERR_TARGETOPEN){
-		message (1, " Error ", " Can't open target file ");
+		message_1s (1, " Error ", " Can't open target file ");
 		goto ret3;
 	    }
 	    if (size == MCERR_READ)
@@ -1157,7 +976,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 		tv_last_update = tv_current;
 	    }
 	    return_status = show_file_progress (n_read_total, file_size);
-	    refresh ();
+	    mc_refresh ();
 	    if (return_status != FILE_CONT)
 	        goto ret3;
         }
@@ -1198,7 +1017,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
         }
     }
     if (!appending)
-        utime (dst_path, &utb);
+        mc_utime (dst_path, &utb);
 
  ret2:
  retry_src_close:
@@ -1352,7 +1171,7 @@ int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
         free (path);
     }
     mc_closedir (reading);
-    chmod (dest_dir, cbuf.st_mode);
+    mc_chmod (dest_dir, cbuf.st_mode);
     free (dest_dir);
     return return_status;
 }
@@ -1369,8 +1188,8 @@ static int move_file_file (char *s, char *d)
     if (show_source (s) == FILE_ABORT
 	|| show_target (d) == FILE_ABORT)
 	return FILE_ABORT;
-	
-    refresh ();
+
+    mc_refresh ();
 
  retry_src_lstat:
     if (mc_lstat (s, &src_stats) != 0){
@@ -1391,13 +1210,13 @@ static int move_file_file (char *s, char *d)
 		msize = 40;
 	    msize /= 2;
 	    
-	    message (1, " Error ", " `%s' and `%s' are the same file ",
+	    message_3s (1, " Error ", " `%s' and `%s' are the same file ",
 		     name_trunc (s, msize), name_trunc (d, msize) );
 	    do_refresh ();
 	    return FILE_SKIP;
 	}
 	if (S_ISDIR (dst_stats.st_mode)){
-	    message (1, " Error ", " Cannot overwrite directory `%s' ", d);
+	    message_2s (1, " Error ", " Cannot overwrite directory `%s' ", d);
 	    do_refresh ();
 	    return FILE_SKIP;
 	}
@@ -1437,8 +1256,8 @@ static int move_file_file (char *s, char *d)
     if ((return_status = show_source (NULL)) != FILE_CONT
 	|| (return_status = show_file_progress (0, 0)) != FILE_CONT)
 	return return_status;
-	
-    refresh ();
+
+    mc_refresh ();
 
  retry_src_remove:
     if (mc_unlink (s)){
@@ -1461,8 +1280,8 @@ int move_dir_dir (char *s, char *d)
     if (show_source (s) == FILE_ABORT
 	|| show_target (d) == FILE_ABORT)
 	return FILE_ABORT;
-	
-    refresh ();
+
+    mc_refresh ();
 
     mc_stat (s, &sbuf);
     if (mc_stat (d, &dbuf))
@@ -1513,8 +1332,8 @@ oktoret:
     if ((return_status = show_source (NULL)) != FILE_CONT
 	|| (return_status = show_file_progress (0, 0)) != FILE_CONT)
 	goto ret;
-	
-    refresh ();
+
+    mc_refresh ();
 
     return_status = recursive_erase (s);
 
@@ -1533,8 +1352,8 @@ static int erase_file (char *s)
 
     if (show_deleting (s) == FILE_ABORT)
 	return FILE_ABORT;
-	
-    refresh ();
+
+    mc_refresh ();
 
  retry_unlink:
     if (mc_unlink (s)){
@@ -1583,7 +1402,7 @@ static int recursive_erase (char *s)
 	return return_status;
     if (show_deleting (s) == FILE_ABORT)
 	return FILE_ABORT;
-    refresh ();
+    mc_refresh ();
  retry_rmdir:
     if (my_rmdir (s)){
 	return_status = file_error (" Cannot remove directory \"%s\" \n %s ", s);
@@ -1606,7 +1425,7 @@ int erase_dir (char *s)
 
     if (show_deleting (s) == FILE_ABORT)
 	return FILE_ABORT;
-    refresh ();
+    mc_refresh ();
 
  retry_rmdir:
     error = my_rmdir (s);
@@ -1671,80 +1490,7 @@ static int is_wildcarded (char *p)
     return 0;
 }
 
-#ifdef BACKGROUND
-
-int we_are_background = 0;
-
-int
-background_attention (int fd)
-{
-    char c;
-    
-    read (fd, &c, 1);
-    if (c == ATTENTION){
-	
-    }
-}
-
-int background_processor (int fd)
-{
-    int c;
-    char buffer [1024];
-    
-    while ((c = read (fd, buffer, sizeof (buffer))) > 0){
-	if (attention)
-	    write (1, buffer, c);
-    }
-}
-
-void
-set_nonblocking (int fd)
-{
-    int val;
-
-    val = fcntl (fd, F_GETFL, 0);
-    val |= O_NONBLOCK;
-    fcntl (fd, F_SETFL, val);
-}
-
-/* Try to make the Midnight Commander a background job */
-/* return -1 on failure */
-static int
-do_background ()
-{
-    int fds [2];		/* stdin/stdout streams */
-    int comm [2];		/* control connection stream */
-    int pid;
-    
-    if (socketpair (AF_UNIX, SOCK_STREAM, 0, fd) == -1)
-	return -1;
-    if (socketpair (AF_UNIX, SOCK_STREAM, 0, comm) == -1){
-	close (fd [0]);
-	close (fd [1]);
-	return -1;
-    }
-    
-    if ((pid = fork ()) == -1)
-	return -1;
-    
-    if (pid == 0){
-	close (fd [0]);
-	close (comm [0]);
-	dup2 (fd [1], 0);	/* stdin */
-	dup2 (fd [1], 1);	/* stdout */
-	dup2 (fd [1], 2);	/* stderr */
-	we_are_background = 1;
-    } else {
-	close (fd [1]);
-	close (comm [1]);
-	add_select_channel (fd [0], background_processor, 0);
-	set_non_blocking (fd [0]);
-	add_select_channel (comm [0], background_attention, 0);
-    }
-}
-#endif
-
-char *file_mask_dialog (char *header, char *text, char *def_text, int only_one)
+char *file_mask_dialog (char *header, char *text, char *def_text, int only_one, int *do_background)
 {
 #define FMDY 13
     int source_easy_patterns = easy_patterns;
@@ -1757,13 +1503,13 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one)
     QuickWidget quick_widgets [] = {
 /* preserve UID/GID must be the first one */    
     { quick_checkbox, 3, 64, 8, FMDY, "preserve UIDs/GIDs", 'U', 0, 0,
-      &preserve_uidgid, 0, XV_WLAY_BELOWCLOSE, "t." },
+      &preserve_uidgid, 0, XV_WLAY_BELOWCLOSE, "preserve" },
     { quick_checkbox, 42, 64, 8, FMDY, "Stable Symlinks", 'S', 0, 0,
-      &stable_symlinks, 0, XV_WLAY_BELOWCLOSE, "" },
+      &stable_symlinks, 0, XV_WLAY_BELOWCLOSE, "stab-sym" },
     { quick_checkbox, 23, 64, 7, FMDY, "Dive into subdir if exists", 'D', 0, 0, 
-      &dive_into_subdirs, 0, XV_WLAY_RIGHTOF, "" },
+      &dive_into_subdirs, 0, XV_WLAY_RIGHTOF, "dive" },
     { quick_checkbox, 3, 64, 7, FMDY, "follow sYmlinks", 'y', 0, 0, 
-      &op_follow_symlinks, 0, XV_WLAY_BELOWOF, "" },
+      &op_follow_symlinks, 0, XV_WLAY_BELOWOF, "follow" },
 #ifdef HAVE_XVIEW
 #define FMDI1 7
 #define FMDI2 4
@@ -1771,27 +1517,29 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one)
     { quick_input, 3, 64, 6, FMDY, 0, 0, 58, 0, 
       0, 0, XV_WLAY_BELOWCLOSE, "" },
 #endif
-    { quick_label, 3, 64, 5, FMDY, "to:", 0, 0, 0, 0, 0, XV_WLAY_BELOWOF,"d."},
+    { quick_label, 3, 64, 5, FMDY, "to:", 0, 0, 0, 0, 0, XV_WLAY_BELOWOF,"to"},
     { quick_checkbox, 37, 64, 4, FMDY, "using shell Patterns", 'p', 0, 0, 
-      0/* &source_easy_patterns */, 0, XV_WLAY_BELOWCLOSE, "s." },
+      0/* &source_easy_patterns */, 0, XV_WLAY_BELOWCLOSE, "using-shell" },
     { quick_input, 3, 64, 3, FMDY, 0 /* default text */, 0, 58, 
-      0, 0, 0, XV_WLAY_BELOWCLOSE, "" },
+      0, 0, 0, XV_WLAY_BELOWCLOSE, "input-def" },
 #ifndef HAVE_XVIEW      
 #define FMDI1 6
 #define FMDI2 7
 #define FMDC 5
     { quick_input, 3, 64, 6, FMDY, 0, 0, 58, 0, 
-      0, 0, XV_WLAY_BELOWCLOSE },
+      0, 0, XV_WLAY_BELOWCLOSE, "input2" },
 #endif      
 #define FMDI0 8	  
-    { quick_label, 2, 64, 2, FMDY, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE },
+    { quick_label, 2, 64, 2, FMDY, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "ql" },
 #define SKIP 2
-    { quick_button, 34, 64, 9, FMDY, "[ Cancel ]", 'C', 2, B_CANCEL, 0, 0, 0,
-	  "b." },
-    { quick_button, 20, 64, 9, FMDY, "[ Ok ]", 'O', 2, B_ENTER, 0, 0, 0, "" },
-#ifdef BACKGROUND
-    { quick_button, 48, 64, 9, FMDY, "[ & ]", '&', 2, B_USER, 0, 0, 0, 0, "" },
+    { quick_button, 42, 64, 9, FMDY, "[ Cancel ]", 'C', 2, B_CANCEL, 0, 0, 0,
+	  "cancel" },
+#ifdef WITH_BACKGROUND
+#undef SKIP
+#define SKIP 3
+    { quick_button, 25, 64, 9, FMDY, "[ Background ]", 'B', 2, B_USER, 0, 0, 0, "back" },
 #endif
+    { quick_button, 15, 64, 9, FMDY, "[ Ok ]", 'O', 2, B_ENTER, 0, 0, 0, "ok" },
     { 0 } };
 
     stable_symlinks = 0;
@@ -1802,7 +1550,10 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one)
     Quick_input.xpos  = -1;
     Quick_input.title = header;
     Quick_input.help  = "[Mask Copy/Rename]";
-    Quick_input.class = "quick_file_mask";
+    if (notroot)
+	Quick_input.class = "quick_file_mask";
+    else
+	Quick_input.class = "quick_file_mask_root";
     quick_widgets [FMDI0].text = text;
     quick_widgets [FMDI2].text = def_text;
     quick_widgets [FMDI2].str_result = &dest_dir;
@@ -1813,23 +1564,15 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one)
         int i;
         
         preserve_uidgid = 0;
-	quick_widgets [1].tk_frame = "t.";
+	quick_widgets [1].tkname = "uidgid";
     }
 
+    *do_background = 0;
 ask_file_mask:
 
     if ((val = quick_dialog_skip (&Quick_input, SKIP)) == B_CANCEL)
 	return 0;
 
-#ifdef BACKGROUND
-    /* If it is a background operation */
-    if (val == B_USER){
-	if (do_background () != -1)
-	    return 0;
-	/* If it did fail, continue as if no background support */
-    }
-#endif
-    
     orig_mask = source_mask;
     if (!dest_dir || !*dest_dir) {
 	if (source_mask)
@@ -1845,7 +1588,7 @@ ask_file_mask:
 
     error = re_compile_pattern (source_mask, strlen (source_mask), &rx);
     if (error) {
-	message (1, " Error ", "Invalid source pattern \"%s\" \n %s ",
+	message_3s (1, " Error ", "Invalid source pattern \"%s\" \n %s ",
 		 orig_mask, error);
 	if (orig_mask)
 	    free (orig_mask);
@@ -1871,6 +1614,8 @@ ask_file_mask:
 	free (dest_dir);
 	dest_dir = strdup ("./");
     }
+    if (val == B_USER)
+	*do_background = 1;
     return dest_dir;
 }
 
@@ -1892,7 +1637,9 @@ int panel_operate (int operation, char *thedefault)
     long marked, total;
     long count = 0, bytes = 0;
     int  dst_result;
+    int  do_bg;			/* do background operation? */
 
+    do_bg = 0;
     rx.buffer = NULL;
     if (linklist)
     	free_linklist ();
@@ -1940,7 +1687,7 @@ int panel_operate (int operation, char *thedefault)
 	rx.allocated = MC_MAXPATHLEN;
 	rx.translate = 0;
 	dest = file_mask_dialog (op_names [operation],
-				    cmd_buf, dest_dir, only_one);
+				    cmd_buf, dest_dir, only_one, &do_bg);
 	if (!dest) {
 	    free (rx.buffer);
 	    return 0;
@@ -1952,6 +1699,24 @@ int panel_operate (int operation, char *thedefault)
 	}
     }
 
+#ifdef WITH_BACKGROUND
+    /* Did the user select to do a background operation? */
+    if (do_bg){
+	int v;
+	
+	v = do_background ();
+	if (v == -1){
+	    message (1, " Error ", " Sorry, I could not put the job in background ");
+	}
+
+	/* If we are the parent */
+	if (v == 1){
+	    vfs_force_expire (cpanel->cwd);
+	    vfs_force_expire (dest);
+	    return 0;
+	}
+    }
+#endif
     /* Initialize things */
     create_op_win (operation);
     ftpfs_hint_reread (0);
@@ -1999,7 +1764,7 @@ int panel_operate (int operation, char *thedefault)
 		    break;
 	        default:
 		    value = FILE_CONT;
-		    message (1, " Internal failure ", " Unknown file operation ");
+		    message_1s (1, " Internal failure ", " Unknown file operation ");
 	        }
 	    }
 	} /* Copy or move operation */
@@ -2065,7 +1830,7 @@ int panel_operate (int operation, char *thedefault)
 		            value = move_file_file (source_with_path, temp);
 		        break;
 		    default:
-		    	message (1, " Internal failure ",
+		    	message_1s (1, " Internal failure ",
 			         " Unknown file operation ");
 		    	goto clean_up;
 		    }
@@ -2091,7 +1856,7 @@ int panel_operate (int operation, char *thedefault)
 	    if (operation != OP_DELETE && verbose
 		&& show_file_progress (0, 0) == FILE_ABORT)
 		goto clean_up;
-	    refresh ();
+	    mc_refresh ();
 	} /* Loop for every file */
     } /* Many files */
 
@@ -2117,11 +1882,277 @@ int panel_operate (int operation, char *thedefault)
 	free (dest_mask);
 	dest_mask = NULL;
     }
+
+#ifdef WITH_BACKGROUND
+    /* Let our parent know we are saying bye bye */
+    if (we_are_background){
+	vfs_shut ();
+	tell_parent (MSG_CHILD_EXITING);
+	exit (1);
+    } 
+#endif
     return 1;
 }
 
 /* }}} */
 
+/* {{{ Query/status report routines */
+
+static int
+real_do_file_error (enum OperationMode mode, char *error)
+{
+    int result;
+    char *msg;
+
+    msg = mode == Foreground ? " Error " : " Background process error ";
+    result = query_dialog (msg, error, 3, 3, " Skip ", " Retry ", " Abort ");
+
+    switch (result){
+    case 0:
+	do_refresh ();
+	return FILE_SKIP;
+    case 1:
+	do_refresh ();
+	return FILE_RETRY;
+    case 2:
+    default:
+	return FILE_ABORT;
+    }
+}
+
+/* Report error with one file */
+int
+file_error (char *format, char *file)
+{
+    sprintf (cmd_buf, format,
+	     name_trunc (file, 30), unix_error_string (errno));
+    return do_file_error (cmd_buf);
+}
+
+/* Report error with two files */
+int
+files_error (char *format, char *file1, char *file2)
+{
+    sprintf (cmd_buf, format, name_trunc (file1, 15),
+	     name_trunc (file2, 15), unix_error_string (errno));
+    return do_file_error (cmd_buf);
+}
+
+static char *format = "Target file \"%s\" already exists!";
+static int
+replace_callback (struct Dlg_head *h, int Id, int Msg)
+{
+#ifndef HAVE_X
+
+    switch (Msg){
+    case DLG_DRAW:
+	dialog_repaint (h, ERROR_COLOR, ERROR_COLOR);
+	break;
+    }
+#endif
+    return 0;
+}
+
+static void
+init_replace (enum OperationMode mode)
+{
+    char buffer [128];
+    
+    replace_colors [0] = ERROR_COLOR;
+    replace_colors [1] = REVERSE_COLOR;
+    replace_colors [2] = ERROR_COLOR;
+    replace_colors [3] = REVERSE_COLOR;
+    
+    replace_dlg = create_dlg (0, 0, 14, 60, replace_colors, replace_callback,
+			      "[Replace]", "replace", DLG_CENTER);
+
+    
+    x_set_dialog_title (replace_dlg,
+        mode == Foreground ? " File exists " : " Background process: File exists ");
+
+#ifdef HAVE_X
+#define X_TRUNC 128
+#else
+#define X_TRUNC 52
+#endif
+    sprintf (buffer, format, name_trunc (replace_filename, X_TRUNC - strlen (format)));
+    add_widgetl (replace_dlg, label_new (3, 5, buffer, NULL), XV_WLAY_CENTERROW);
+    
+    add_widgetl (replace_dlg,
+		button_new (BY + 1, 46, REPLACE_ABORT, "[ Abort ]", 'a', 2, 0, 0, NULL),
+		XV_WLAY_CENTERROW);
+    
+    tk_new_frame (replace_dlg, "a.");
+    add_widgetl (replace_dlg,
+		button_new (BY - 1, 47, REPLACE_NEVER, "[ nonE ]", 'e', 5, 0, 0, NULL),
+		XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg,
+		button_new (BY - 1, 36, REPLACE_UPDATE, "[ Update ]", 'u', 2, 0, 0, NULL),
+		XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg,
+		button_new (BY - 1, 28, REPLACE_ALWAYS, "[ alL ]", 'l', 4, 0, 0, NULL),
+		XV_WLAY_RIGHTOF);
+
+    add_widgetl (replace_dlg, label_new (BY-1, 5, "Overwrite all targets?", NULL),
+	        XV_WLAY_CENTERROW);        
+
+    /* "this target..." widgets */
+    tk_new_frame (replace_dlg, "p.");
+    if (!S_ISDIR (d_stat->st_mode))
+        add_widgetl (replace_dlg,
+		    button_new (BY - 2, 43, REPLACE_APPEND, "[ apPend ]", 'p', 2, 0, 0, NULL),
+		    XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg,
+		button_new (BY - 2, 36, REPLACE_NO, "[ No ]", 'n', 2, 0, 0, NULL),
+		XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg,
+		button_new (BY - 2, 28, REPLACE_YES, "[ Yes ]", 'y', 2, 0, 0, NULL), 
+		XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg, label_new (BY-2,5, "Overwrite this target?", NULL),
+		 XV_WLAY_CENTERROW);
+    
+    tk_new_frame (replace_dlg, "i.");
+    sprintf (buffer, "Target date: %s, size %d",
+	     file_date (d_stat->st_mtime), (int) d_stat->st_size);
+    add_widgetl (replace_dlg, label_new (6, 5, buffer, NULL), XV_WLAY_CENTERROW);
+    sprintf (buffer, "Source date: %s, size %d",
+	     file_date (s_stat->st_mtime), (int) s_stat->st_size);
+    add_widgetl (replace_dlg, label_new (5, 5, buffer, NULL), XV_WLAY_CENTERROW);
+    tk_end_frame ();
+}
+
+static int
+real_query_replace (enum OperationMode mode, char *destname, struct stat *_s_stat,
+		    struct stat *_d_stat)
+{
+    if (replace_result < REPLACE_ALWAYS){
+	replace_filename = destname;
+	s_stat = _s_stat;
+	d_stat = _d_stat;
+	init_replace (mode);
+	run_dlg (replace_dlg);
+	replace_result = replace_dlg->ret_value;
+	if (replace_result == B_CANCEL)
+	    replace_result = REPLACE_ABORT;
+	destroy_dlg (replace_dlg);
+    }
+
+    switch (replace_result){
+    case REPLACE_UPDATE:
+	do_refresh ();
+	if (s_stat->st_mtime > d_stat->st_mtime)
+	    return FILE_CONT;
+	else
+	    return FILE_SKIP;
+	
+    case REPLACE_APPEND:
+        do_append = 1;
+    case REPLACE_YES:
+    case REPLACE_ALWAYS:
+	do_refresh ();
+	return FILE_CONT;
+    case REPLACE_NO:
+    case REPLACE_NEVER:
+	do_refresh ();
+	return FILE_SKIP;
+    case REPLACE_ABORT:
+    default:
+	return FILE_ABORT;
+    }
+}
+
+int
+real_query_recursive (enum OperationMode mode, char *s)
+{
+    char *confirm, *text;
+
+    if (recursive_result < RECURSIVE_ALWAYS){
+	char *msg =
+	    mode == Foreground ? "\n   Directory not empty.   \n   Delete it recursively? "
+	                       : "\n   Background process: Directory not empty \n   Delete it recursively? ";
+	text = copy_strings (" Delete: ", name_trunc (s, 30), " ", 0);
+
+        if (know_what_am_i_doing){
+	    query_set_sel (1);
+	    recursive_result = query_dialog (text, msg, 3 | WITH_HOTKEYS, 5,
+					     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
+	} else
+	    recursive_result = query_dialog (text, msg, 3 | WITH_HOTKEYS, 5, 
+					     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
+	if (recursive_result != RECURSIVE_ABORT)
+	    do_refresh ();
+	free (text);
+	if (!know_what_am_i_doing && (recursive_result == RECURSIVE_YES
+	    || recursive_result == RECURSIVE_ALWAYS)){
+	    text = copy_strings (" Please type 'yes' to confirm recursive delete of ",
+				 recursive_result == RECURSIVE_YES
+				 ? name_trunc (s, 20) : "all the directories", " ", 0);
+	    confirm = input_dialog (mode == Foreground ? " Recursive Delete "
+				                       : " Background process: Recursive Delete ",
+				    text, "no");
+	    do_refresh ();
+	    if (!confirm || strcmp (confirm, "yes"))
+		recursive_result = RECURSIVE_NEVER;
+	    free (confirm);
+	    free (text);
+	}
+    }
+    switch (recursive_result){
+    case RECURSIVE_YES:
+    case RECURSIVE_ALWAYS:
+	return FILE_CONT;
+    case RECURSIVE_NO:
+    case RECURSIVE_NEVER:
+	return FILE_SKIP;
+    case RECURSIVE_ABORT:
+    default:
+	return FILE_ABORT;
+    }
+}
+
+#ifdef WITH_BACKGROUND
+int
+do_file_error (char *str)
+{
+    return call_1s (real_do_file_error, str);
+}
+
+int
+query_recursive (char *s)
+{
+    return call_1s (real_query_recursive, s);
+}
+
+int
+query_replace (char *destname, struct stat *_s_stat, struct stat *_d_stat)
+{
+    if (we_are_background)
+	return parent_call (real_query_replace, 3, strlen(destname), destname,
+			    sizeof (struct stat), _s_stat, sizeof(struct stat), _d_stat);
+    else
+	return real_query_replace (Foreground, destname, _s_stat, _d_stat);
+}
+
+#else
+do_file_error (char *str)
+{
+    return real_do_file_error (Foreground, str);
+}
+
+int
+query_recursive (char *s)
+{
+    return real_query_recursive (Foreground, s);
+}
+
+int
+query_replace (char *destname, struct stat *_s_stat, struct stat *_d_stat)
+{
+    return real_query_replace (Foreground, destname, _s_stat, _d_stat);
+}
+
+#endif
+
 /*
   Cause emacs to enter folding mode for this file:
   Local variables:
