@@ -51,6 +51,8 @@
 
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <fcntl.h>
+#include <ctype.h>
 #include "global.h"
 #include "mad.h"
 #include "util.h"
@@ -79,19 +81,27 @@ extern int verbose;		/* Should be in a more sensible header file */
 
 
 /* Size of the find parameters window */
-#define FIND_Y 10
+#define FIND_Y 12
 #define FIND_X 50
 
 /* Size of the find window */
-#define FIND2_Y 20
-#define FIND2_X 50
+#define FIND2_Y LINES-4
+#define FIND2_X COLS-16
+
+#ifdef HAVE_X
+#   define FIND2_X_USE 35
+#else
+#   define FIND2_X_USE FIND2_X-20
+#endif
 
 /* A couple of extra messages we need */
-#define B_STOP   B_USER+1
-#define B_AGAIN  B_USER+2
-#define B_PANELIZE	B_USER+3
-
-#define B_TREE	 B_USER+1
+enum {
+    B_STOP = B_USER + 1,
+    B_AGAIN,
+    B_PANELIZE,
+    B_TREE,
+    B_VIEW
+};
 
 /* A list of directories to be ignores, separated with ':' */
 char *find_ignore_dirs = 0;
@@ -99,80 +109,91 @@ char *find_ignore_dirs = 0;
 static Dlg_head *find_dlg;	/* The dialog */
 static WInput *in_start;	/* Start path */
 static WInput *in_name;		/* Pattern to search */
+static WInput *in_with;		/* text inside filename */
 static WListbox *find_list;	/* Listbox with the file list */
 static int running = 0;		/* nice flag */
 static WButton *stop_button;	/* pointer to the stop button */
 static WLabel *status_label;	/* Finished, Searching etc. */
 static char *find_pattern;	/* Pattern to search */
+static char *content_pattern;	/* pattern to search inside files */
 static int count;		/* Number of files displayed */
 static int matches;		/* Number of matches */
 static int is_start;		/* Status of the start/stop toggle button */
 int max_loops_in_idle = 10;
 static char *old_dir;
 
-static int find_par_callback (struct Dlg_head *h, int id, int Msg)
-{
-#ifndef HAVE_X
-    switch (Msg){
-    case DLG_DRAW:
-	attrset (REVERSE_COLOR);
-	dlg_erase (h);
-	draw_box (h, 1, 1, FIND_Y-2, FIND_X-2);
-	break;
-    }
-#endif
-    return 0;
-}
+/* For nice updating */
+static char *rotating_dash = "|/-\\";
 
-static char *in_start_dir = NULL;
-static char *in_start_name = NULL;
+/* This keeps track of the directory stack */
+typedef struct dir_stack {
+    char *name;
+    struct dir_stack *prev;
+} dir_stack ;
 
-static int find_parameters (char **start_dir, char **pattern)
+dir_stack *dir_stack_base = 0;
+
+/*
+ * find_parameters: gets information from the user
+ *
+ * If the return value is true, then the following holds:
+ *
+ * START_DIR and PATTERN are pointers to char * and upon return they
+ * contain the information provided by the user.
+ *
+ * CONTENT holds a strdup of the contents specified by the user if he
+ * asked for them or 0 if not (note, this is different from the
+ * behavior for the other two parameters.
+ *
+ */
+
+static int
+find_parameters (char **start_dir, char **pattern, char **content)
 {
     int return_value;
     char *temp_dir;
+    static char *in_contents = NULL;
+    static char *in_start_dir = NULL;
+    static char *in_start_name = NULL;
 
- find_par_start:
+find_par_start:
     if (!in_start_dir)
 	in_start_dir = strdup (".");
     if (!in_start_name)
 	in_start_name = strdup (easy_patterns ? "*" : ".");
+    if (!in_contents)
+	in_contents = strdup ("");
+    
     find_dlg = create_dlg (0, 0, FIND_Y, FIND_X, dialog_colors,
-			   find_par_callback, "[Find File]", "findfile",
-			   DLG_CENTER);
+			   common_dialog_callback, "[Find File]", "findfile",
+			   DLG_CENTER | DLG_GRID);
     x_set_dialog_title (find_dlg, "Find File");
 
-    in_start = input_new (3, 14, INPUT_COLOR, 30, in_start_dir);
-    in_name  = input_new (5, 14, INPUT_COLOR, 30, in_start_name);
-
+    in_start = input_new (3, 14, INPUT_COLOR, 30, in_start_dir, "start");
+    in_name  = input_new (5, 14, INPUT_COLOR, 30, in_start_name, "name");
+    in_with  = input_new (7, 14, INPUT_COLOR, 30, in_contents, "content");
+    
 #ifndef HAVE_XVIEW
     add_widget (find_dlg, in_start);
 #endif
-    tk_new_frame (find_dlg, "b.");
-    add_widgetl (find_dlg, button_new (7, 36, B_CANCEL,"[ Cancel ]",'c',2,0,0),
+    add_widgetl (find_dlg, button_new (9, 36, B_CANCEL,"[ Cancel ]",'c',2,0,0, "cancel"),
         XV_WLAY_RIGHTOF);
     
-#if HAVE_AVOID_DIRS
-    add_widgetl (find_dlg, button_new (7, 22, B_USER, "[ Avoid... ]",
-				      'a', 2,
-				      avoid_box, 0),
-	         XV_WLAY_RIGHTOF);
-#endif
-
-    add_widgetl (find_dlg, button_new (7, 12, B_TREE, "[ Tree ]", 't', 2,0,0),
+    add_widgetl (find_dlg, button_new (9, 12, B_TREE, "[ Tree ]", 't', 2,0,0, "tree"),
         XV_WLAY_RIGHTOF);
-    add_widgetl (find_dlg, button_new (7, 4, B_ENTER,"[ Ok ]",'o', 2, 0, 0),
+    add_widgetl (find_dlg, button_new (9, 4, B_ENTER,"[ Ok ]",'o', 2, 0, 0, "ok"),
         XV_WLAY_CENTERROW);
-    tk_end_frame ();
+
+    add_widget (find_dlg, in_with);
     add_widgetl (find_dlg, in_name, XV_WLAY_RIGHTOF);
 
-    
-    add_widgetl (find_dlg, label_new (5, 3, "Filename:"), XV_WLAY_NEXTROW);
+    add_widgetl (find_dlg, label_new (7, 3, "Content: ", "label-cont"), XV_WLAY_NEXTROW);
+    add_widgetl (find_dlg, label_new (5, 3, "Filename:", "label-file"), XV_WLAY_NEXTROW);
 
 #ifdef HAVE_XVIEW
     add_widgetl (find_dlg, in_start, XV_WLAY_RIGHTOF);
 #endif    
-    add_widget (find_dlg, label_new (3, 3, "Start at:"));
+    add_widget (find_dlg, label_new (3, 3, "Start at:", "label-start"));
 
     run_dlg (find_dlg);
     if (find_dlg->ret_value == B_CANCEL)
@@ -196,6 +217,11 @@ static int find_parameters (char **start_dir, char **pattern)
 	return_value = 1;
 	*start_dir = strdup (in_start->buffer);
 	*pattern   = strdup (in_name->buffer);
+
+	if (in_with->buffer [0])
+	    *content   = strdup (in_with->buffer);
+	else
+	    *content   = 0;
 	free (in_start_dir);
 	in_start_dir = strdup (*start_dir);
 	free (in_start_name);
@@ -207,14 +233,8 @@ static int find_parameters (char **start_dir, char **pattern)
     return return_value;
 }
 
-typedef struct dir_stack {
-    char *name;
-    struct dir_stack *prev;
-} dir_stack ;
-
-dir_stack *dir_stack_base = 0;
-
-static void push_directory (char *dir)
+static void
+push_directory (char *dir)
 {
     dir_stack *new;
 
@@ -224,7 +244,8 @@ static void push_directory (char *dir)
     dir_stack_base = new;
 }
 
-static char *pop_directory (void)
+static char*
+pop_directory (void)
 {
     char *name; 
     dir_stack *next;
@@ -239,18 +260,19 @@ static char *pop_directory (void)
 	return 0;
 }
 
-static void insert_file (char *dir, char *file)
+static void
+insert_file (char *dir, char *file)
 {
     char *tmp_name;
     static char *dirname;
     int i;
 
-    if (dir [0] == '/' && dir [1] == '/')
+    if (dir [0] == PATH_SEP && dir [1] == PATH_SEP)
 	dir++;
     i = strlen (dir);
     if (i){
-	if (dir [i - 1] != '/'){
-	    dir [i] = '/';
+	if (dir [i - 1] != PATH_SEP){
+	    dir [i] = PATH_SEP;
 	    dir [i + 1] = 0;
 	}
     }
@@ -271,9 +293,114 @@ static void insert_file (char *dir, char *file)
     free (tmp_name);
 }
 
-static char *rotating_dash = "|/-\\";
+static void
+find_add_match (Dlg_head *h, char *dir, char *file)
+{
+    int p = ++matches & 7;
 
-static void do_search (struct Dlg_head *h)
+    insert_file (dir, file);
+    
+    /* Scroll nicely */
+    if (!p)
+	listbox_select_last (find_list, 1);
+    else
+	listbox_select_last (find_list, 0);
+    
+#ifndef HAVE_X
+	/* Updates the current listing */
+	send_message (h, &find_list->widget, WIDGET_DRAW, 0);
+	if (p == 7)
+	    mc_refresh ();
+#endif	    
+}
+
+/* 
+ * search_content:
+ *
+ * Search with egrep the global (FIXME) content_pattern string in the
+ * DIRECTORY/FILE.  It will add the found entries to the find listbox.
+ */
+void
+search_content (Dlg_head *h, char *directory, char *filename)
+{
+    struct stat s;
+    char buffer [128];
+    char *fname, *p;
+    int file_fd, pipe, ignoring;
+    char c, i;
+    pid_t pid;
+
+    fname = get_full_name (directory, filename);
+
+    if (mc_stat (fname, &s) != 0 && !S_ISREG (s.st_mode)){
+	free (fname);
+	return;
+    }
+    if (!S_ISREG (s.st_mode)){
+	free (fname);
+	return;
+    }
+    
+    file_fd = mc_open (fname, O_RDONLY);
+    free (fname);
+    
+    if (file_fd == -1)
+	return;
+
+    pipe = mc_doublepopen (file_fd, -1, &pid, "/usr/bin/egrep",
+			   "/usr/bin/egrep", "-n", content_pattern, "-", NULL);
+	
+    if (pipe == -1){
+	mc_close (file_fd);
+	return;
+    }
+    
+    sprintf (buffer, "Grepping in %s", name_trunc (filename, FIND2_X_USE));
+
+    label_set_text (status_label, buffer);
+    mc_refresh ();
+    p = buffer;
+    ignoring = 0;
+    
+    enable_interrupt_key ();
+    got_interrupt ();
+    while (1){
+	i = read (pipe, &c, 1);
+	if (i != 1)
+	    break;
+	
+	if (c == '\n'){
+	    p = buffer;
+	    ignoring = 0;
+	}
+	if (ignoring)
+	    continue;
+	
+	if (c == ':'){
+	    char *the_name;
+	    
+	    *p = 0;
+	    ignoring = 1;
+	    the_name = copy_strings (buffer, ":", filename, NULL);
+	    find_add_match (h, directory, the_name);
+	    free (the_name);
+	} else {
+	    if (p - buffer < (sizeof (buffer)-1) && ISASCII (c) && isdigit (c))
+		*p++ = c;
+	    else
+		*p = 0;
+	} 
+    }
+    disable_interrupt_key ();
+    if (i == -1)
+	message (1, " Find/read ", " Problem reading from child ");
+
+    mc_doublepclose (pipe, pid);
+    mc_close (file_fd);
+}
+
+static void
+do_search (struct Dlg_head *h)
 {
     static struct dirent *dp   = 0;
     static DIR  *dirp = 0;
@@ -282,7 +409,15 @@ static void do_search (struct Dlg_head *h)
     static int pos;
     static int subdirs_left = 0;
     char *tmp_name;		/* For bulding file names */
-    int p;
+
+    if (!h) { /* someone forces me to close dirp */
+	if (dirp) {
+	    mc_closedir (dirp);
+	    dirp = 0;
+	}
+        dp = 0;
+	return;
+    }
 
  do_search_begin:
     while (!dp){
@@ -321,22 +456,16 @@ static void do_search (struct Dlg_head *h)
 
 	    if (verbose){
 		    char buffer [50];
-		    
-		    sprintf (buffer, "Searching %s", name_trunc (directory, 
-#ifndef HAVE_X
-		    FIND2_X-20
-#else		    
-		    35
-#endif
-		    ));
+
+		    sprintf (buffer, "Searching %s", name_trunc (directory, FIND2_X_USE));
 		    label_set_text (status_label, buffer);
 	    }
 	    dirp = mc_opendir (directory);
 	    mc_stat (directory, &tmp_stat);
 	    subdirs_left = tmp_stat.st_nlink - 2;
 	    /* Commented out as unnecessary
-	    if (subdirs_left < 0)
-	        subdirs_left = MAXINT;
+	       if (subdirs_left < 0)
+	       subdirs_left = MAXINT;
 	    */
 	}
 	dp = mc_readdir (dirp);
@@ -360,24 +489,12 @@ static void do_search (struct Dlg_head *h)
 	    subdirs_left--;
 	}
     }
-    
+
     if (regexp_match (find_pattern, dp->d_name, match_file)){
-	insert_file (directory, dp->d_name);
-	matches++;
-
-	p = matches & 7;
-
-	if (!p)
-	    listbox_select_last (find_list, 1);
-	else
-	    listbox_select_last (find_list, 0);
-
-#ifndef HAVE_X
-	/* Updates the current listing */
-	send_message (h, &find_list->widget, WIDGET_DRAW, 0);
-	if (p == 7)
-	    refresh ();
-#endif	    
+	if (content_pattern)
+	    search_content (h, directory, dp->d_name);
+	else 
+	    find_add_match (h, directory, dp->d_name);
     }
     
     free (tmp_name);
@@ -392,7 +509,7 @@ static void do_search (struct Dlg_head *h)
 	    attrset (NORMALC);
 	    dlg_move (h, FIND2_Y-6, FIND2_X - 4);
 	    addch (rotating_dash [pos]);
-	    refresh ();
+	    mc_refresh ();
 	}
     } else
 	goto do_search_begin;
@@ -403,76 +520,100 @@ static void do_search (struct Dlg_head *h)
     xv_post_proc (h, (void (*)(void *))do_search, (void *)h);
 #endif		
 #endif
-
 }
 
-static int find_callback (struct Dlg_head *h, int id, int Msg)
+static int
+view_currently_selected_file (int unparsed_view)
+{
+    WLEntry *entry = find_list->current;
+    char *dir, *fullname, *filename;
+    int line;
+
+    if (!entry)
+        return MSG_NOT_HANDLED;
+
+    dir = entry->data;
+
+    if (!entry->text || !dir)
+	return MSG_NOT_HANDLED;
+
+    if (content_pattern){
+	filename = strchr (entry->text + 4, ':') + 1;
+	line = atoi (entry->text + 4);
+    } else {
+	 filename = entry->text + 4;
+	 line = 0;
+    }
+    if (dir [0] == '.' && dir [1] == 0)
+	 fullname = strdup (filename);
+    else if (dir [0] == '.' && dir [1] == PATH_SEP)
+	 fullname = get_full_name (dir+2, filename);
+    else
+	 fullname = get_full_name (dir, filename);
+
+    view_file_at_line (fullname, unparsed_view, use_internal_view, line);
+    free (fullname);
+    return MSG_HANDLED;
+}
+
+static int
+find_callback (struct Dlg_head *h, int id, int Msg)
 {
     switch (Msg){
 #ifndef HAVE_X    
     case DLG_DRAW:
-	attrset (REVERSE_COLOR);
-	dlg_erase (h);
-	draw_box (h, 1, 1, FIND2_Y-2, FIND2_X-2);
-	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 2);
-	addstr (" Find file ");
+        common_dialog_repaint (h);
 	break;
 #endif
 
     case DLG_KEY:
 	if (id == KEY_F(3) || id == KEY_F(13)){
 	    int unparsed_view = (id == KEY_F(13));
-	    WLEntry *entry = find_list->current;
-	    char *dir, *name;
+	    return view_currently_selected_file (unparsed_view);
+	 }
+	 return MSG_NOT_HANDLED;
 
-	    dir = entry->data;
-	    
-	    if (!entry->text || !dir)
-		return MSG_NOT_HANDLED;
-
-	    if (dir [0] == '.' && dir [1] == 0)
-		name = strdup (entry->text + 4);
-	    else if (dir [0] == '.' && dir [1] == '/')
-		name = get_full_name (dir+2, entry->text + 4);
-	    else
-		name = get_full_name (dir, entry->text+4);
-
-	    view_file (name, unparsed_view, use_internal_view);
-	    free (name);
-	    return MSG_HANDLED;
-	}
-	return MSG_NOT_HANDLED;
-	
-    case DLG_IDLE:
-	do_search (h);
-	break;
-    }
-    return 0;
+     case DLG_IDLE:
+	 do_search (h);
+	 break;
+     }
+     return 0;
 }
 
-#define BUTTON_NEW(i,msg,str) \
-   button_new (FIND2_Y-4, 3+i*12, msg, str, str [2], 2, 0, 0)
+#define BUTTON_NEW(i,msg,str,name) \
+   button_new (FIND2_Y-4, 3+i*12, msg, str, str [2], 2, 0, 0,name)
 
-#define BUTTON_NEW_CBACK(i,msg,str,a,b) \
-   button_new (FIND2_Y-4, 3+i*12, msg, str, str [2], 2, a, b)
+#define BUTTON_NEW_CBACK(i,msg,str,a,b,name) \
+   button_new (FIND2_Y-4, 3+i*12, msg, str, str [2], 2, a, b, name)
+
+#define BUTTON_NEW_CBACKY(i,j,msg,str,a,b,name) \
+   button_new (FIND2_Y-4+j, 3+i*12, msg, str, str [2], 2, a, b, name)
 
 /* Handles the Stop/Start button in the find window */
-static int start_stop (int button, void *extra)
+static int
+start_stop (int button, void *extra)
 {
     char *button_labels [2] = { "[ Stop ] ", "[ Start ]" };
-    
     running = is_start;
     set_idle_proc (find_dlg, running);
     is_start = !is_start;
 
     label_set_text (status_label, is_start ? "Stopped" : "Searching");
     button_set_text (stop_button, button_labels [is_start]);
-    
+
     return 0;
 }
 
-static void init_find_vars (void)
+/* Handle view command, when invoked as a button */
+static int
+find_do_view_file (int button, void *extra)
+{
+    view_currently_selected_file (0);
+    return 0;
+}
+
+static void
+init_find_vars (void)
 {
     char *dir;
     
@@ -488,49 +629,59 @@ static void init_find_vars (void)
 	free (dir);
 }
 
-static int find_file (char *start_dir, char *pattern,
-		      char **dirname,  char **filename)
+
+struct spaced_dirent {
+    struct dirent dp;
+#ifdef NEED_EXTRA_DIRENT_BUFFER
+    char extra_buffer [MC_MAXPATHLEN];
+#endif
+};
+
+static int
+find_file (char *start_dir, char *pattern, char *content, char **dirname,  char **filename)
 {
     int return_value = 0;
     char *dir;
     char *dir_tmp, *file_tmp;
     
     find_dlg = create_dlg (0, 0, FIND2_Y, FIND2_X, dialog_colors,
-			   find_callback, "[Find File]", "mfind", DLG_CENTER);
+			   find_callback, "[Find File]", "mfind", DLG_CENTER | DLG_GRID);
     
     x_set_dialog_title (find_dlg, "Find file");
 
-    find_list = listbox_new (2, 2, FIND2_X-4, FIND2_Y-9, listbox_finish, 0);
+    find_list = listbox_new (2, 2, FIND2_X-4, FIND2_Y-9, listbox_finish, 0, "listbox");
 
 #ifndef HAVE_X
     add_widget (find_dlg, find_list);
 #endif
 
-    tk_new_frame (find_dlg, "b.");
     add_widgetl (find_dlg,
-		button_new (FIND2_Y-3, 3, B_PANELIZE, "[ Panelize ]", 'p', 2, 0, 0),
+		button_new (FIND2_Y-3, 3, B_PANELIZE, "[ Panelize ]", 'p', 2, 0, 0, "button-panelize"),
 		XV_WLAY_CENTERROW);
-    add_widgetl (find_dlg, BUTTON_NEW (3, B_CANCEL, "[ Quit ]"),
+    add_widgetl (find_dlg,
+		 BUTTON_NEW_CBACKY(2, 1,B_VIEW, "[ View - F3 ]", find_do_view_file, find_dlg, "button-view"),
+		 0);
+    add_widgetl (find_dlg, BUTTON_NEW (3, B_CANCEL, "[ Quit ]", "button-quit"),
         XV_WLAY_RIGHTOF);
     stop_button = BUTTON_NEW_CBACK (2, B_STOP,   "[ Stop  ]", start_stop,
-				    find_dlg);
+				    find_dlg, "start-stop");
     add_widgetl (find_dlg, stop_button,
         XV_WLAY_RIGHTOF);
-    add_widgetl (find_dlg, BUTTON_NEW (1, B_AGAIN,  "[ Again ]"),
+    add_widgetl (find_dlg, BUTTON_NEW (1, B_AGAIN,  "[ Again ]", "button-again"),
         XV_WLAY_RIGHTOF);
-    add_widgetl (find_dlg, BUTTON_NEW (0, B_ENTER,  "[ Chdir ]"), 
+    add_widgetl (find_dlg, BUTTON_NEW (0, B_ENTER,  "[ Chdir ]", "button-chdir"), 
         XV_WLAY_CENTERROW);
-    tk_end_frame ();
 
-    status_label = label_new (FIND2_Y-6, 4, "Searching");
+    status_label = label_new (FIND2_Y-6, 4, "Searching", "label-search");
     add_widgetl (find_dlg, status_label, XV_WLAY_BELOWOF);
 #ifdef HAVE_X
     add_widgetl (find_dlg, find_list, XV_WLAY_EXTENDWIDTH);
 #endif
 
-    /* Need to cleanup this */
-    find_pattern = pattern;
-
+    /* FIXME: Need to cleanup this, this ought to be passed non-globaly */
+    find_pattern    = pattern;
+    content_pattern = content;
+    
     set_idle_proc (find_dlg, 1);
     init_find_vars ();
     push_directory (start_dir);
@@ -553,39 +704,54 @@ static int find_file (char *start_dir, char *pattern,
     if (file_tmp)
 	*filename = strdup (file_tmp);
     if (return_value == B_PANELIZE && *dirname && *filename){
-	struct dirent dp;
+	struct spaced_dirent dp;
 	int status, link_to_dir, stalled_link;
 	int next_free = 0;
-	int i;
+	int i, k;
 	struct stat buf;
 	WLEntry *entry = find_list->list;
 	dir_list *list = &cpanel->dir;
 	char *dir, *name;
 
-	clean_dir (list, cpanel->count);
 	for (i = 0; entry && i < find_list->count; entry = entry->next, i++){
+	    char *filename;
+
+	    if (content_pattern)
+		filename = strchr (entry->text+4, ':')+1;
+	    else
+		filename = entry->text+4;
+
 	    if (!entry->text || !entry->data)
 		continue;
 	    dir = entry->data;
 	    if (dir [0] == '.' && dir [1] == 0)
-		name = strdup (entry->text + 4);
-	    else if (dir [0] == '.' && dir [1] == '/')
-		name = get_full_name (dir + 2, entry->text + 4);
+		name = strdup (filename);
+	    else if (dir [0] == '.' && dir [1] == PATH_SEP)
+		name = get_full_name (dir + 2, filename);
 	    else
-		name = get_full_name (dir, entry->text + 4);
-	    strcpy (dp.d_name, name);
+		name = get_full_name (dir, filename);
+	    strcpy (dp.dp.d_name, name);
 	    free (name);
 #if !defined(HAVE_DIRENT_H) && !defined(_POSIX_VERSION)
-	    NLENGTH (&dp) = strlen (dp.d_name);
+	    NLENGTH (&dp.dp) = strlen (dp.dp.d_name);
 #endif
-	    status = handle_dirent (list, NULL, &dp, &buf, next_free, &link_to_dir,
+	    status = handle_dirent (list, NULL, &dp.dp, &buf, next_free, &link_to_dir,
 	        &stalled_link);
 	    if (status == 0)
 		continue;
 	    if (status == -1)
 		break;
-	    list->list [next_free].fnamelen = strlen (dp.d_name);
-	    list->list [next_free].fname = strdup (dp.d_name);
+
+	    /* don't add files more than once to the panel */
+	    if (content_pattern && next_free > 0){
+		if (strcmp (list->list [next_free-1].fname, dp.dp.d_name) == 0)
+		    continue;
+	    }
+
+	    if (!next_free) /* first turn i.e clean old list */
+    		clean_dir (list, cpanel->count);
+	    list->list [next_free].fnamelen = NLENGTH (&(dp.dp));
+	    list->list [next_free].fname = strdup (dp.dp.d_name);
 	    list->list [next_free].cache = NULL;
 	    file_mark (cpanel, next_free, 0);
 	    list->list [next_free].f.link_to_dir = link_to_dir;
@@ -598,14 +764,22 @@ static int find_file (char *start_dir, char *pattern,
 	if (next_free){
 	    cpanel->count = next_free;
 	    cpanel->is_panelized = 1;
-	    if (start_dir [0] == '/'){
-		strcpy (cpanel->cwd, "/");
-		chdir ("/");
+	    cpanel->dirs_marked = 0;
+	    cpanel->marked = 0;
+	    cpanel->total = 0;
+
+	    if (start_dir [0] == PATH_SEP){
+		strcpy (cpanel->cwd, PATH_SEP_STR);
+		chdir (PATH_SEP_STR);
 	    }
 	}
     }
 
+    /* cpanel->top_file and cpanel->selected must be set to a valid
+     * value before call of destroy_dlg which calls paint_panel  */
+    try_to_select (cpanel, NULL);
     destroy_dlg (find_dlg);
+    do_search (0); /* force do_search to release resources */
     if (old_dir){
 	free (old_dir);
 	old_dir = 0;
@@ -613,21 +787,25 @@ static int find_file (char *start_dir, char *pattern,
     return return_value;
 }
 
-void do_find (void)
+void
+do_find (void)
 {
-    char *start_dir, *pattern;
+    char *start_dir, *pattern, *content;
     char *filename, *dirname;
     int  v, dir_and_file_set;
     int done = 0;
     
     while (!done){
-	if (!find_parameters (&start_dir, &pattern))
+	if (!find_parameters (&start_dir, &pattern, &content))
 	    break;
 
 	dirname = filename = NULL;
-	v = find_file (start_dir, pattern, &dirname, &filename);
+	is_start = 0;
+	v = find_file (start_dir, pattern, content, &dirname, &filename);
 	free (start_dir);
 	free (pattern);
+	if (content)
+	    free (content);
 
 	if (v == B_ENTER){
 	    if (dirname || filename){
@@ -640,6 +818,10 @@ void do_find (void)
 		paint_panel (current_panel);
 		select_item (cpanel);
 	    }
+	    if (dirname)  
+		free (dirname);
+	    if (filename) 
+		free (filename);
 	    break;
 	}
 	dir_and_file_set = dirname && filename;
@@ -647,6 +829,7 @@ void do_find (void)
 	if (filename) free (filename);
 	if (v == B_CANCEL)
 	    break;
+	
 	if (v == B_PANELIZE){
 	    if (dir_and_file_set){
 	        try_to_select (cpanel, NULL);
