@@ -579,7 +579,7 @@ static Widget *mousedispatch_new (int y, int x, int yl, int xl)
     Widget *w = xmalloc (sizeof (Widget), "disp_new");
 
     init_widget (w, y, x, yl, xl,
-		 (callback_fn) md_callback, 0, (mouse_h)  help_event);
+		 (callback_fn) md_callback, 0, (mouse_h)  help_event, NULL);
 
     return w;
 }
@@ -744,6 +744,7 @@ void interactive_display (char *filename, char *node)
     }
     if (!(main = search_string (data, node))){
 	message (1, " Error ", " Can't find node %s in help file ", node);
+	interactive_display_finish ();
 	return;
     }
 
