@@ -29,7 +29,7 @@
 #endif
 #include <sys/types.h>		/* FD_ZERO et al */
 #include <sys/time.h>		/* struct timeval */
-#if IS_AIX
+#if HAVE_SYS_SELECT_H
 #   include <sys/select.h>
 #endif
 #include "tty.h"
@@ -148,6 +148,7 @@ void delete_select_channel (int fd)
 	    free (p);
 	}
 	prev = p;
+	p = p->next;
     }
 }
 
@@ -668,7 +669,7 @@ int get_event (Gpm_Event *event, int redo_event)
     static int dirty = 3;
 
     if ((dirty == 3) || is_idle ()){
-	refresh ();
+	mc_refresh ();
 	doupdate ();
 	dirty = 1;
     } else
@@ -920,7 +921,8 @@ int is_idle (void)
     return ! FD_ISSET (0, &select_set);
 }
 
-int ctrl_pressed ()
+
+int get_modifier ()
 {
 #ifdef __linux__
     unsigned char modifiers;
@@ -930,7 +932,16 @@ int ctrl_pressed ()
     if (ioctl (0, TIOCLINUX, &modifiers) < 0)
 	return 0;
 
-    if (modifiers & 4)
+    return (int) modifiers;
+#else
+    return 0;
+#endif
+}
+
+int ctrl_pressed ()
+{
+#ifdef __linux__
+    if (get_modifier () & CONTROL_PRESSED)
 	return 1;
 #endif
     return 0;
