@@ -7,6 +7,14 @@ int get_event (Gpm_Event *event, int redo_event);
 int is_idle (void);
 int ctrl_pressed ();
 
+/* Used to get the modifier information          */
+/* Currently, it just works on the Linux console */
+#define SHIFT_PRESSED 1
+#define ALTR_PRESSED 2
+#define CONTROL_PRESSED 4
+#define ALTL_PRESSED 8
+int get_modifier ();
+
 extern int double_click_speed;
 extern int old_esc_mode;
 extern int irix_fn_keys;
@@ -50,5 +58,5 @@ typedef struct {
 } key_code_name_t;
 
 extern key_code_name_t key_name_conv_tab [];
-
+extern int we_are_background;
 #endif	/* __KEY_H */
