@@ -109,7 +109,7 @@ int keybar_visible = 1;
 int message_visible = 1;
 
 /* Set if you want the message bar shown in xterm title bar to save space */
-int xterm_hintbar = 1;
+int xterm_hintbar = 0;
 
 /* The starting line for the output of the subprogram */
 int output_start_y = 0;
@@ -160,14 +160,15 @@ static struct {
     WCheck *widget;
     int    hkey;
     int    hpos;
+    char   *tkname;
 } check_options [] = {
-    { "Xterm hintbar",	  &xterm_hintbar,0, 'X', 0 },
-    { "hIntbar visible",  &message_visible,0, 'I', 1 },
-    { "Keybar visible",   &keybar_visible, 0, 'K', 0 },
-    { "command Prompt",   &command_prompt, 0, 'P', 8 },
-    { "show Mini status", &show_mini_info, 0, 'M', 5 },
-    { "menuBar visible",  &menubar_visible, 0, 'B',4 },
-    { "Equal split",      &equal_split, 0, 'E', 0 },
+    { "Xterm hintbar",	  &xterm_hintbar,0,    'X', 0, "h" },
+    { "hIntbar visible",  &message_visible,0,  'I', 1, "v" },
+    { "Keybar visible",   &keybar_visible, 0,  'K', 0, "k" },
+    { "command Prompt",   &command_prompt, 0,  'P', 8, "p" },
+    { "show Mini status", &show_mini_info, 0,  'M', 5, "m" },
+    { "menuBar visible",  &menubar_visible, 0, 'B', 4, "me" },
+    { "Equal split",      &equal_split, 0,     'E', 0, "eq" },
     { 0, 0, 0, 0, 0 }
 };
 
@@ -260,6 +261,7 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 {
     switch (Msg){
     case DLG_DRAW:
+#ifndef HAVE_X
     	/*When repainting the whole dialog (e.g. with C-l) we have to
     	  update everything*/
    	old_first_panel_size = -1;
@@ -290,10 +292,13 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 		addstr ("lines of output");
 	    }
 	}
+#endif
 	break;
 
     case DLG_POST_KEY:
+#ifndef HAVE_X
 	_equal_split = check_options [6].widget->state & C_BOOL;
+#endif
 	_menubar_visible = check_options [5].widget->state & C_BOOL;
 	_command_prompt = check_options [4].widget->state & C_BOOL;
 	_keybar_visible = check_options [2].widget->state & C_BOOL;
@@ -344,38 +349,38 @@ static void init_layout (void)
     int i;
 
     layout_dlg = create_dlg (0, 0, 14, 60, dialog_colors, layout_callback,
-			     "[Layout]", "layout", DLG_CENTER);
+			     "[Layout]", "layout", DLG_CENTER | DLG_GRID);
 			     
     x_set_dialog_title (layout_dlg, "Layout");
 
     add_widgetl (layout_dlg,
-		button_new (BY, BX+26, B_CANCEL, "[ Cancel ]",'c',2, 0, 0), 
+		button_new (BY, BX+26, B_CANCEL, "[ Cancel ]",'c',2, 0, 0, "c"), 
 		XV_WLAY_RIGHTOF);
     add_widgetl (layout_dlg,
-		button_new (BY, BX+12, B_EXIT,   "[ Save ]",'s',2, 0, 0),
+		button_new (BY, BX+12, B_EXIT,   "[ Save ]",'s',2, 0, 0, "s"),
 		XV_WLAY_RIGHTOF);
     add_widgetl (layout_dlg,
-		button_new (BY, BX,    B_ENTER,  "[[ Ok ]]",'o',3, 0, 0),
+		button_new (BY, BX,    B_ENTER,  "[[ Ok ]]",'o',3, 0, 0, "o"),
 		XV_WLAY_CENTERROW);
 #ifndef HAVE_X
     if (console_flag){
 	add_widget (layout_dlg,
-		    button_new (9, 36, B_MINUS, "-", '-', 0, bminus_cback, 0));
+		    button_new (9, 36, B_MINUS, "-", '-', 0, bminus_cback, 0, NULL));
 	add_widget (layout_dlg,
-		    button_new (9, 33, B_PLUS, "+", '+', 0, bplus_cback, 0));
+		    button_new (9, 33, B_PLUS, "+", '+', 0, bplus_cback, 0, NULL));
     }
 #endif    
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text,                             check_options[i].hkey, check_options[i].hpos
+#define XTRACT(i) *check_options[i].variable, check_options[i].text,                             check_options[i].hkey, check_options[i].hpos,  check_options[i].tkname
 
     for (i = 0; i < 6; i++){
 	check_options [i].widget = check_new (8 - i, 33, XTRACT(i));
 	add_widgetl (layout_dlg, check_options [i].widget, XV_WLAY_BELOWCLOSE);
     }
-#ifdef HAVE_X
-    add_widgetl (layout_dlg, label_new (2, 32, " Other options "), 
+#ifdef HAVE_XVIEW
+    add_widgetl (layout_dlg, label_new (2, 32, " Other options ", "oo"), 
         XV_WLAY_NEXTCOLUMN);
-    add_widgetl (layout_dlg, label_new (2, 6, " Panel split "),
+    add_widgetl (layout_dlg, label_new (2, 6, " Panel split ", "ps"),
         XV_WLAY_NEXTCOLUMN);
 #endif        
     _equal_split = equal_split;
@@ -384,22 +389,25 @@ static void init_layout (void)
     _keybar_visible = keybar_visible;
     _message_visible = message_visible;
     _xterm_hintbar = xterm_hintbar;
-    bright_widget = button_new(6, 18, B_2RIGHT, ">", '>', 0, b2right_cback, 0);
+#ifndef HAVE_X
+    bright_widget = button_new(6, 18, B_2RIGHT, ">", '>', 0, b2right_cback, 0, ">");
     add_widgetl (layout_dlg, bright_widget, XV_WLAY_RIGHTOF);
-    bleft_widget = button_new (6, 15, B_2LEFT, "<", '<', 0, b2left_cback, 0);
+    bleft_widget = button_new (6, 15, B_2LEFT, "<", '<', 0, b2left_cback, 0, "<");
     add_widgetl (layout_dlg, bleft_widget, XV_WLAY_RIGHTOF);
     check_options [6].widget = check_new (5, 8, XTRACT(6));
-
+#endif
     old_first_panel_size = -1;
     old_horizontal_split = -1;
     old_output_lines     = -1;
     
     _first_panel_size = first_panel_size;
     _output_lines = output_lines;
+#ifndef HAVE_X
     add_widget (layout_dlg, check_options [6].widget);
-    radio_widget = radio_new (3, 8, 2, s_split_direction, 1);
+    radio_widget = radio_new (3, 8, 2, s_split_direction, 1, "r");
     add_widget (layout_dlg, radio_widget);
     radio_widget->sel = horizontal_split;
+#endif
 }
 
 void layout_change (void)
@@ -601,6 +609,8 @@ void flag_winch (int dummy)
     winch_flag = 1;
 }
 
+void edit_adjust_size (Dlg_head * h);
+
 void change_screen_size (void)
 {
 #ifndef HAVE_X
@@ -608,6 +618,7 @@ void change_screen_size (void)
 #if defined TIOCGWINSZ && !defined SCO_FLAVOR
     struct winsize winsz;
     extern Dlg_head *view_dlg;
+    extern Dlg_head *edit_dlg;
     
     mc_noraw_mode ();
     endwin ();
@@ -629,7 +640,11 @@ void change_screen_size (void)
     setup_panels ();
     if (current_dlg == view_dlg)
 	view_adjust_size (view_dlg);
-	
+#ifdef USE_INTERNAL_EDIT
+    if (current_dlg == edit_dlg)
+	edit_adjust_size (edit_dlg);
+#endif
+    
     /* Now, force the redraw */
     do_refresh ();
     touchwin (stdscr);
@@ -680,13 +695,13 @@ void print_vfs_message(char *msg, ...)
     }
 }
 
-#ifdef BACKGROUND
+#ifdef BACKGROUND_XXXX
 void set_proc_stat_msg (char *msg)
 {
     int l = strlen (msg);
 
     process_status->widget.cols = l;
-    process_status->x = COLS - l;
+    process_status->widget.x = COLS - l;
     label_set_text (process_status, msg);
 
     /* since the message has changed, and we are called from one of the 
@@ -695,7 +710,7 @@ void set_proc_stat_msg (char *msg)
      */
     if (current_dlg == midnight_dlg){
 	update_cursor (midnight_dlg);
-	refresh ();
+	mc_refresh ();
     }
 }
 #endif
@@ -713,7 +728,7 @@ void rotate_dash (void)
 	pos = 0;
     move (0, COLS-1);
     addch (rotating_dash [pos]);
-    refresh ();
+    mc_refresh ();
     pos++;
 #endif
 }
@@ -835,7 +850,7 @@ void set_display_type (int num, int type)
 	else
 	    file_name = "";
 	
-	view_init ((WView *) new_widget, 0, file_name);
+	view_init ((WView *) new_widget, 0, file_name, 0);
 	break;
     }
     panels [num].type = type;
@@ -866,6 +881,19 @@ void set_display_type (int num, int type)
 
     if (type == view_tree)
 	the_tree = (WTree *) new_widget;
+
+    /* Prevent current_panel's value from becoming invalid.
+     * It's just a quick hack to prevent segfaults. Comment out and
+     * try following:
+     * - select left panel
+     * - invoke menue left/tree
+     * - as long as you stay in the left panel almost everything that uses
+     *   cpanel causes segfault, e.g. C-Enter, C-x c, ...
+     */
+
+    if (type != view_listing)
+	if (current_panel == (WPanel *) old_widget)
+	    current_panel = num == 0 ? right_panel : left_panel;
 }
 
 #ifndef HAVE_XVIEW
