@@ -166,7 +166,7 @@ static void init_listmode (char *oldlistformat)
 			      DLG_CENTER);
     x_set_dialog_title (listmode_dlg, "Listing format edit");	    
 
-#define XTRACT(i) BY+listmode_but[i].y, BX+listmode_but[i].x, listmode_but[i].ret_cmd, listmode_but[i].text, listmode_but[i].hkey, listmode_but[i].hpos, 0, 0
+#define XTRACT(i) BY+listmode_but[i].y, BX+listmode_but[i].x, listmode_but[i].ret_cmd, listmode_but[i].text, listmode_but[i].hkey, listmode_but[i].hpos, 0, 0, NULL
 
     for (i = 0; i < BUTTONS; i++)
 	add_widgetl (listmode_dlg, button_new (XTRACT (i)), (i == BUTTONS - 1) ?
@@ -175,23 +175,23 @@ static void init_listmode (char *oldlistformat)
     /* We add the labels. */
     for (i = 0; i < LABELS; i++){
 	pname = label_new (listmode_text [i].y,
-			   listmode_text [i].x, listmode_text [i].text);
+			   listmode_text [i].x, listmode_text [i].text, NULL);
 	add_widget (listmode_dlg, pname);
     }
 
     add_widget (listmode_dlg, button_new (UY+13, UX+37, B_MINUS,
-					  "-", '-', 0, bminus_cback, 0));
+					  "-", '-', 0, bminus_cback, 0, NULL));
     add_widget (listmode_dlg, button_new (UY+13, UX+34, B_PLUS,
-					  "+", '+', 0, bplus_cback, 0));
-    radio_itemwidth = radio_new (UY+9, UX+22, 3, s_itemwidth, 1);
+					  "+", '+', 0, bplus_cback, 0, NULL));
+    radio_itemwidth = radio_new (UY+9, UX+22, 3, s_itemwidth, 1, NULL);
     add_widget (listmode_dlg, radio_itemwidth);
     radio_itemwidth = 0;
-    radio_justify = radio_new (UY+5, UX+22, 3, s_justify, 1);
+    radio_justify = radio_new (UY+5, UX+22, 3, s_justify, 1, NULL);
     add_widget (listmode_dlg, radio_justify);
     radio_justify->sel = 1;
 
     /* get new listbox */
-    l_listmode = listbox_new (UY + 5, UX + 1, 16, 9, 0, l_call);
+    l_listmode = listbox_new (UY + 5, UX + 1, 16, 9, 0, l_call, NULL);
 
     if (strncmp (oldlistformat, "full ", 5) == 0){
 	format_width = 1;
@@ -217,10 +217,10 @@ static void init_listmode (char *oldlistformat)
     /* add listbox to the dialogs */
     add_widgetl (listmode_dlg, l_listmode, XV_WLAY_EXTENDWIDTH);
 
-    radio_columns = radio_new (UY+1, UX+32, 2, s_columns, 1);
+    radio_columns = radio_new (UY+1, UX+32, 2, s_columns, 1, NULL);
     add_widget (listmode_dlg, radio_columns);
     radio_columns->sel = format_columns;
-    radio_genwidth = radio_new (UY+1, UX+2, 2, s_genwidth, 1);
+    radio_genwidth = radio_new (UY+1, UX+2, 2, s_genwidth, 1, NULL);
     add_widget (listmode_dlg, radio_genwidth);
     radio_genwidth->sel = format_width;
 }
@@ -314,7 +314,7 @@ char *listmode_edit (char *oldlistformat)
 	    break;
 
 	case B_REMOVE:
-	    listbox_remove_current (l_listmode);
+	    listbox_remove_current (l_listmode, 0);
 	    break;
 
 	case B_ENTER:
