@@ -131,7 +131,10 @@ void *mad_realloc (void *ptr, int newsize, char *file, int line)
 {
     int i;
     char *area;
-    
+
+    if (!ptr)
+        return (mad_alloc (newsize, file, line));
+
     mad_check (file, line);
 
     for (i = 0; i < MAD_MAX_AREAS; i++){
