@@ -1,7 +1,8 @@
+
 /* Main program for the Midnight Commander
-   Copyright (C) 1994, 1995 The Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1997 The Free Software Foundation
    
-   Written by: 1994, 1995 Miguel de Icaza
+   Written by: 1994, 1995, 1996, 1997 Miguel de Icaza
                1994, 1995 Janne Kukonlehto
    
    This program is free software; you can redistribute it and/or modify
@@ -97,6 +98,7 @@
 #include "panelize.h"
 #include "learn.h"
 #include "listmode.h"
+#include "background.h"
 
 /* Listbox for the command history feature */
 #include "widget.h"
@@ -214,11 +216,6 @@ WLabel *the_prompt;
 /* The hint bar */
 WLabel *the_hint;
 
-#ifdef BACKGROUND
-/* The foreground/background message */
-WLabel *process_status;
-#endif
-
 /* The button bar */
 WButtonBar *the_bar;
 #ifdef HAVE_X
@@ -288,6 +285,9 @@ char *command_line_colors;
 /* File name to view if argument was supplied */
 char *view_one_file = 0;
 
+/* File name to view if argument was supplied */
+char *edit_one_file = 0;
+
 /* Used so that widgets know if they are being destroyed or
    shut down */
 int midnight_shutdown = 0;
@@ -298,6 +298,9 @@ static int last_paused = 0;
 /* Only used at program boot */
 int boot_current_is_left = 1;
 
+/* Used for keeping track of the original stdout */
+int stdout_fd = 0;
+
 /* Ugly hack in order to distinguish between left and right panel in menubar */
 int is_right;
 #define MENU_PANEL_IDX  (is_right ? 1 : 0)
@@ -618,7 +621,7 @@ void clr_scr (void)
 {
     standend ();
     dlg_erase (midnight_dlg);
-    refresh ();
+    mc_refresh ();
     doupdate ();
 }
 
@@ -773,9 +776,9 @@ void do_execute (const char *shell, const char *command, int internal_command)
 	    handle_console (CONSOLE_SAVE);
 	}
 
-	if (pause_after_run == pause_always ||
+	if ((pause_after_run == pause_always ||
 	    (pause_after_run == pause_on_dumb_terminals &&
-	     !xterm_flag && !console_flag)){
+	     !xterm_flag && !console_flag)) && !quit){
 	    printf ("Press any key to continue...");
 	    last_paused = 1;
 	    fflush (stdout);
@@ -789,14 +792,14 @@ void do_execute (const char *shell, const char *command, int internal_command)
     post_exec ();
 
 #ifdef HAVE_SUBSHELL_SUPPORT
-	if (new_dir){
+	if (new_dir)
 	    do_possible_cd (new_dir);
-#    ifdef USE_VFS
-	    if (old_vfs_dir){
-		mc_chdir (old_vfs_dir);
-		free (old_vfs_dir);
-	    }
-#    endif
+#endif
+	
+#ifdef USE_VFS
+	if (old_vfs_dir){
+	    mc_chdir (old_vfs_dir);
+	    free (old_vfs_dir);
 	}
 #endif
 
@@ -842,23 +845,27 @@ void change_panel (void)
 
 static int quit_cmd_internal (int quiet)
 {
+    int q = quit;
+
     if (quiet || !confirm_exit){
-	quit = 1;
+	q = 1;
     } else  {
 	if (query_dialog (" The Midnight Commander ",
 		     " Do you really want to quit the Midnight Commander? ",
 			  0, 2, " Yes ", " No ") == 0)
-	    quit = 1;
+	    q = 1;
     }
-    if (quit){
+    if (q){
 #ifdef HAVE_SUBSHELL_SUPPORT
 	if (!use_subshell)
 	    midnight_dlg->running = 0;
 	else
-	    if ((quit = exit_subshell ()))
+	    if ((q = exit_subshell ()))
 #endif
 		midnight_dlg->running = 0;
     }
+    if (q)
+        quit |= 1;
     return quit;
 }
 
@@ -881,33 +888,29 @@ int quiet_quit_cmd (void)
 
 /* This routine untouches the first line on both panels in order */
 /* to avoid the refreshing the menu bar */
+
+#ifdef HAVE_X
 void untouch_bar (void)
 {
-#ifndef HAVE_X
-    do_refresh ();
-#endif
 }
 
 void repaint_screen (void)
 {
-#ifndef HAVE_X
-    /* Are there pieces of code unpainted, this may be the offending line,
-     * uncomment it: */
-/*    touchwin (stdscr); */
     do_refresh ();
-    refresh ();
-#endif
 }
 
-#if 0
-void refresh_screen (void *unused)
+#else /* HAVE_X */
+void untouch_bar (void)
 {
-#ifndef HAVE_X
-    touchwin (stdscr);
-    refresh ();
-#endif
+    do_refresh ();
 }
-#endif
+
+void repaint_screen (void)
+{
+    do_refresh ();
+    mc_refresh ();
+}
+#endif /* HAVE_X */
 
 /* Wrapper for do_subshell_chdir, check for availability of subshell */
 void subshell_chdir (char *directory)
@@ -948,13 +951,14 @@ int do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
     }
 
     /* Success: save previous directory, shutdown status of previous dir */
+    directory_history_add (olddir);
     strcpy (panel->lwd, olddir);
     free (olddir);
     free_completions (input_w (cmdline));
     
     mc_get_current_wd (panel->cwd, sizeof (panel->cwd) - 2);
 
-    subshell_chdir (directory);
+    subshell_chdir (panel->cwd);
 
     /* Reload current panel */
     clean_dir (&panel->dir, panel->count);
@@ -1002,7 +1006,7 @@ int load_prompt (int fd, void *unused)
 	 */
 	if (current_dlg == midnight_dlg){
 	    update_cursor (midnight_dlg);
-	    refresh ();
+	    mc_refresh ();
 	}
     }
     update_prompt = 1;
@@ -1096,6 +1100,7 @@ static void listing_cmd (void)
     p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;    
     free (p->user_format);
     p->user_format = user;
+    
     free (p->mini_status_format);
     p->mini_status_format = status;
      
@@ -1104,7 +1109,7 @@ static void listing_cmd (void)
     
     if (set_panel_format (p, panel_format (p))){
 	free (p->user_format);
-	p->user_format = strdup (DEFAULT_USER_FORMAT);
+	p->user_format  = strdup (DEFAULT_USER_FORMAT);
 	set_panel_format (p, DEFAULT_USER_FORMAT);
     }
     do_refresh ();
@@ -1161,6 +1166,22 @@ void toggle_listing_cmd ()
     set_basic_panel_listing_to (current, list_full);
 }
 
+void set_sort_to (WPanel *p, sortfn *sort_order)
+{
+    p->sort_type = sort_order;
+    
+    /* The directory is already sorted, we have to load the unsorted stuff */
+    if (sort_order == (sortfn *) unsorted){
+	char *current_file;
+	
+	current_file = strdup (cpanel->dir.list [cpanel->selected].fname);
+	panel_reload (cpanel);
+	try_to_select (cpanel, current_file);
+	free (current_file);
+    }
+    do_re_sort (p);
+}
+
 static void sort_cmd (void)
 {
     WPanel  *p;
@@ -1503,16 +1524,16 @@ static void create_panels (void)
 
     /* Create the nice widgets */
     cmdline     = command_new (0, 0, 0);
-    the_prompt  = label_new (0, 0, prompt);
+    the_prompt  = label_new (0, 0, prompt, NULL);
     the_prompt->transparent = 1;
     the_bar     = buttonbar_new (keybar_visible);
-    the_hint    = label_new (0, 0, 0);
+    the_hint    = label_new (0, 0, 0, NULL);
     the_hint->transparent = 1;
     the_hint->auto_adjust_cols = 0;
     the_hint->widget.cols = COLS;
 
 #ifdef BACKGROUND
-    process_status = label_new (0, COLS-12, 0);
+    process_status = label_new (0, COLS-12, 0, NULL);
     process_status->transparent = 0;
     process_status->auto_adjust_cols = 0;
     process_status->widget.cols = 0;
@@ -1828,7 +1849,7 @@ static void setup_post ()
     midnight_colors [3] = NORMAL_COLOR;	     /* HOT_FOCUSC */
 }
 
-static void setup_mc ()
+static void setup_mc (void)
 {
     setup_pre ();
     init_menu ();
@@ -1843,7 +1864,7 @@ static void setup_mc ()
     setup_post ();
 }
 
-static void setup_mc_viewer ()
+static void setup_dummy_mc (const char *file)
 {
     setup_pre ();
     setup_post ();
@@ -1854,14 +1875,15 @@ static void setup_mc_viewer ()
     cpanel = xmalloc (sizeof (WPanel), "fake_panel");
     strcpy (cpanel->cwd, ".");
     cpanel->dir.list = (file_entry *) xmalloc (sizeof (file_entry), "fake_p");
+    cpanel->selected = 0;
     cpanel->count = 1;
-    cpanel->dir.list [0].fname = view_one_file;
+    cpanel->dir.list[0].fname = (char *) file;
 }
 
 static void done_screen ()
 {
 /* #ifndef HAVE_X */
-    if (quit != SUBSHELL_EXIT)
+    if (!(quit & SUBSHELL_EXIT))
 	clr_scr ();
     reset_shell_mode ();
 #ifndef HAVE_X
@@ -1882,7 +1904,9 @@ static void done_mc ()
      * we only change the setup data if we have the auto save feature set
      */
     if (auto_save_setup)
-	save_setup ();
+	save_setup ();   /* does also call save_hotlist */
+    else
+	save_hotlist();
     done_screen ();
     vfs_add_current_stamps ();
     if (xterm_flag && xterm_hintbar)
@@ -2070,30 +2094,15 @@ void load_hint ()
 	free (hint);
     } else {
 	set_hintbar ("The Midnight Commander " VERSION
-			" (C) 1995 the Free Software Foundation");
+			" (C) 1995-1997 the Free Software Foundation");
     }
 }
 
-static void do_nc (void)
+static void
+setup_panels_and_run_mc ()
 {
     int first, second;
 
-    midnight_dlg = create_dlg (0, 0, LINES, COLS,
-        midnight_colors, midnight_callback, "[main]", "midnight", 0);
-    midnight_dlg->has_menubar = 1;
-    
-    if (view_one_file){
-	/* Invoke the internal view routine with:
-	 * the default processing and forcing the internal viewer
-	 */
-	setup_mc_viewer ();
-	view_file (view_one_file, 0, 1);
-	midnight_shutdown = 1;
-	done_screen ();
-	return;
-    }
-    setup_mc ();
-
     xtoolkit_panel_setup ();
     tk_new_frame (midnight_dlg, "p.");
 #ifndef HAVE_X
@@ -2126,22 +2135,77 @@ static void do_nc (void)
 
     /* Run the Midnight Commander if no file was specified in the command line */
     run_dlg (midnight_dlg);
-    midnight_shutdown = 1;
-    
-    /* destroy_dlg destroys even cpanel->cwd, so we have to save a copy :) */
-    if (print_last_wd && !view_one_file){
-	if (vfs_current_is_local ())
-	    last_wd_string = strdup (cpanel->cwd);
-	else
-	    last_wd_string = strdup (".");
+}
+
+/* result must be free'd (I think this should go in util.c) */
+char *prepend_cwd (const char *filename)
+{
+    char *d;
+    int l;
+    if (*filename == PATH_SEP)	/* an absolute pathname */
+	return strdup (filename);
+    d = malloc (MC_MAXPATHLEN + strlen (filename) + 2);
+    mc_get_current_wd (d, MC_MAXPATHLEN);
+    l = strlen(d);
+    d[l++] = PATH_SEP;
+    strcpy (d + l, filename);
+    return canonicalize_pathname (d);
+}
+
+#ifdef USE_INTERNAL_EDIT
+void edit (const char *file_name);
+#endif
+
+static void do_nc (void)
+{
+    midnight_dlg = create_dlg (0, 0, LINES, COLS,
+	    midnight_colors, midnight_callback, "[main]", "midnight", 0);
+    midnight_dlg->has_menubar = 1;
+
+    if (view_one_file || edit_one_file) {
+	/* Invoke the internal view/edit routine with:
+	 * the default processing and forcing the internal viewer/editor
+	 */
+	char *path;
+	if (view_one_file) {
+	    path = prepend_cwd (view_one_file);
+	    setup_dummy_mc (path);
+	    view_file (path, 0, 1);
+	}
+#ifdef USE_INTERNAL_EDIT
+	else {
+	    path = prepend_cwd ("");
+	    setup_dummy_mc (path);
+	    edit (edit_one_file);
+	}
+#endif
+	free (path);
+	midnight_shutdown = 1;
+	done_screen ();
+	return;
     }
-    
-    done_mc  ();
-    
+    setup_mc ();
+
+    setup_panels_and_run_mc ();
+
+    /* Program end */
+    midnight_shutdown = 1;
+
+    /* destroy_dlg destroys even cpanel->cwd, so we have to save a copy :) */
+    if (print_last_wd) {
+	if (!vfs_current_is_local ())
+	    last_wd_string = strdup (".");
+	else
+	    last_wd_string = strdup (cpanel->cwd);
+    }
+    done_mc ();
+
     destroy_dlg (midnight_dlg);
     done_mc_profile ();
 }
 
+
+
 #if OLD
 static int do_nc (void)
 {
@@ -2178,7 +2242,7 @@ static int do_nc (void)
 	
 	if (was_searching != searching){
 	    display_mini_info (cpanel);
-	    refresh ();
+	    mc_refresh ();
 	}
 
 	/* This is needed for ncurses 1.8.7, ugh :-( */
@@ -2212,46 +2276,23 @@ static int do_nc (void)
 }
 #endif /* EVENT mc */
 
-#ifndef VERSION
-#    define VERSION "undefined"
-#endif
+#include "features.inc"
 
-int stdout_fd = 0;
-	
-static void version (void)
+static void version (int verbose)
 {
-#ifdef HAVE_X
-    char *version;
+    fprintf (stderr, "Midnight Commander %s\n", VERSION);
+    if (!verbose)
+	return;
     
-#ifdef HAVE_XVIEW
-    version = "XView";
-#else
-    version = "Tk";
-#endif
-    
-    fprintf (stderr, "%s Midnight Commander %s\n%s", version, VERSION,
-#else /* HAVE_X */
+#ifndef HAVE_X
     fprintf (stderr,
-	    "Midnight Commander %s, with mouse support on xterm%s,\n%s%s.\n",
-	    VERSION, status_mouse_support ? " and the Linux console" : "",
-	    status_using_ncurses ? "using the ncurses library"
-#ifdef HAVE_SLANG
-	     : "using the S-Lang library "
-#ifdef SLANG_TERMINFO
-	     "with terminfo database",
-#else
-#ifdef USE_TERMCAP
-	     "with termcap database",
-#else
-	     "with unknown terminal database",
-#endif /* USE_TERMCAP */
-#endif /* SLANG_TERMINFO */
-#else
-	    : "using some unknown curses library",
-#endif /* HAVE_SLANG */
+	    "with mouse support on xterm%s%s%s.\n",
+	     status_mouse_support ? " and the Linux console" : "",
+	     status_using_ncurses ? "\nusing the ncurses library" : "\n",
+	     status_using_old_tools ? " and compiled with old tools" : "");
 #endif /* HAVE_X */
-	    status_using_old_tools ? " and compiled with old tools" : "");
 
+    fprintf (stderr, features);
     if (print_last_wd)
 	write (stdout_fd, ".", 1);
 }
@@ -2270,26 +2311,32 @@ static void handle_args (int argc, char *argv [])
     extern char *optarg;
     int    c;
     char   *termvalue;
+    int    finish_program = 0;
 
 #ifdef HAVE_SLANG
-#    ifdef _OS_NT
-       static
-#    else
-       extern
-#    endif
-int SLtt_try_termcap;
+#ifdef _OS_NT
+    static
+#else
+    extern
+#endif
+    int SLtt_try_termcap;
 #endif
 
 #ifdef USE_TERMCAP
     SLtt_try_termcap = 1;
 #endif
-    
-    while ((c = getopt (argc, argv, "VbsdPcfC:hmxtuUv:l:NX")) != -1){
-	switch (c){
+
+    while ((c = getopt (argc, argv, "VbsdPcfC:hmxtuUe:v:l:NXB")) != -1) {
+	switch (c) {
 	case 'V':
-	    version ();
-	    exit (1);
-	    
+	    version (1);
+	    finish_program = 1;
+	    break;
+
+	case 'B':
+	    background_wait = 1;
+	    break;
+
 	case 'b':
 	    disable_colors = 1;
 	    break;
@@ -2303,13 +2350,17 @@ int SLtt_try_termcap;
 
 	case 'f':
 	    fprintf (stderr, "Library directory for the Midnight Commander: " LIBDIR "\n");
-	    exit (1);
-	    
+	    finish_program = 1;
+	    break;
+
 	case 'm':
 	    use_8th_bit_as_meta = 0;
 	    break;
-	    
-	    
+
+	case 'e':
+	    edit_one_file = optarg;
+	    break;
+
 	case 'v':
 	    view_one_file = optarg;
 	    break;
@@ -2322,7 +2373,7 @@ int SLtt_try_termcap;
 	case 'P':
 	    print_last_wd = 1;
 	    break;
-	    
+
 	case 's':
 	    slow_terminal = 1;
 	    break;
@@ -2348,9 +2399,9 @@ int SLtt_try_termcap;
 	case 'N':
 	    SLtt_want_unicode ();
 	    break;
-#endif /* _OS_NT */
-#endif /* HAVE_SLANG */
-	    
+#endif				/* _OS_NT */
+#endif				/* HAVE_SLANG */
+
 	case 'u':
 #ifdef HAVE_SUBSHELL_SUPPORT
 		use_subshell = 0;
@@ -2362,55 +2413,59 @@ int SLtt_try_termcap;
 	    debug_subshell = 1;
 #endif
 	    break;
-	    
+
 	case 'U':
 #ifdef HAVE_SUBSHELL_SUPPORT
 	    use_subshell = 1;
 #endif
 	    break;
-	    
+
 	case '?':
 	case 'h':
 	default:
-	    version ();
+	    version (0);
 	    fprintf (stderr, "Usage is:\n"
-		    "mc [-bCcdfhPsuUlVv?] [other_panel_dir]\n"
-		    "-b Force black and white\n"
-		    "-C <keyword>=<fore>,<back> Color usage\n"
-		    "   keywords: normal, selected, marked, markselect, "
-                        "errors, reverse,\n"
-		    "   dnormal, dfocus, dhotnormal, dhotfocus,\n"
-		    "   menu, menuhot, menusel, menuhotsel\n"
-		    "   colors: black, red, green, brightgreen, brown,\n"
-                    "   yellow, blue, brightblue, magenta, brightmagenta,\n"
-                    "   cyan, brightcyan, lightgray and white\n"
-		    "-c Force color, only available if terminfo allows it\n"
-		    "-d Disable mouse support\n"
-		    "-f Print configured paths\n"
-		    "-m Don't use the highest character bit as meta flag\n"
-		    "-P At exit, print last working directory\n"
-		    "-s Disables verbose operation (for slow terminals)\n"
+		"mc [-bCcdfhlmNPstuUVvx?] [other_panel_dir] [this-dir]\n"
+		     "-b Force black and white\n"
+		     "-C <keyword>=<fore>,<back> Color usage keywords:\n"
+	     "   normal, selected, marked, markselect, errors, reverse\n"
+	     "   dnormal, dfocus, dhotnormal, dhotfocus, menu, menuhot\n"
+		     "   menusel, menuhotsel."
+		     "   colors: black, red, green, brightgreen, brown,\n"
+		 "   yellow, blue, brightblue, magenta, brightmagenta,\n"
+		     "   cyan, brightcyan, lightgray and white\n"
+		 "-c Force color, only available if terminfo allows it\n"
+		     "-d Disable mouse support\n"
+		     "-f Print configured paths\n"
+		  "-m Don't use the highest character bit as meta flag\n"
+		     "-P At exit, print last working directory\n"
+		   "-s Disables verbose operation (for slow terminals)\n"
 #ifdef HAVE_SLANG
-		    "-t Activate support for the TERMCAP variable\n"
-		    "-N Use Unicode if available\n"
+		     "-t Activate support for the TERMCAP variable\n"
+		     "-N Use Unicode if available\n"
 #endif
 #ifdef USE_NETCODE
-		    "-l file Log ftpfs commands to the file\n"
+		     "-l file Log ftpfs commands to the file\n"
 #endif
-		    "-u Disable the concurrent subshell mode\n"
-		    "-U Force the concurrent subshell mode\n"
-		    "-v [file] start up into the viewer mode\n"
-                    "-x Force xterm mouse support and screen save/restore\n"
-		    );
-	    exit (1);
+		     "-u Disable the concurrent subshell mode\n"
+		     "-U Force the concurrent subshell mode\n"
+		     "-v [file] start up into the viewer mode\n"
+		 "-x Force xterm mouse support and screen save/restore\n"
+		);
+	    finish_program = 1;
 	}
     }
+    if (finish_program){
+	if (print_last_wd)
+	    printf (".");
+	exit (1);
+    }
 #ifdef _OS_NT
     SetConsoleTitle ("GNU Midnight Commander");
     shell = getenv ("COMSPEC");
     if (!shell || !*shell)
 	shell = "cmd.exe";
-     /* Default opening mode for files is binary, not text (CR/LF translation) */
+    /* Default opening mode for files is binary, not text (CR/LF translation) */
     _fmode = _O_BINARY;
 #else
     termvalue = getenv ("TERM");
@@ -2427,27 +2482,34 @@ int SLtt_try_termcap;
     }
     shell = getenv ("SHELL");
     if (!shell || !*shell)
-	shell = strdup (getpwuid (geteuid ()) -> pw_shell);
+	shell = strdup (getpwuid (geteuid ())->pw_shell);
     if (!shell || !*shell)
 	shell = "/bin/sh";
-#endif /* _OS_NT */
-    
+#endif				/* _OS_NT */
+
     sprintf (control_file, CONTROL_FILE, getpid ());
     my_putenv ("MC_CONTROL_FILE", control_file);
 
     /* sets the current dir and the other dir */
-    for (; optind < argc; optind++){
-	if (this_dir){
-	    other_dir = strdup (argv [optind]);
-	} else {
-	    char buffer [MC_MAXPATHLEN+2];
-	    
-	    this_dir = argv [optind];
-	    mc_get_current_wd (buffer, sizeof (buffer)-2);
+    if (strstr (argv[0], "mcedit") == argv[0] + strlen(argv[0]) - 6) {
+	edit_one_file = "";
+	for (; optind < argc; optind++) {
+	    edit_one_file = strdup (argv[optind]);
+	}
+    } else {
+	for (; optind < argc; optind++) {
+	    if (this_dir) {
+		other_dir = strdup (argv[optind]);
+	    } else {
+		char buffer[MC_MAXPATHLEN + 2];
+		this_dir = argv[optind];
+		mc_get_current_wd (buffer, sizeof (buffer) - 2);
+	    }
 	}
     }
 }
 
+
 #ifdef _OS_NT
 static void sigchld_handler_no_subshell (int sig)
 {
@@ -2612,6 +2674,10 @@ int main (int argc, char *argv [])
     load_setup ();
 
     init_curses ();
+
+    /* The directory hot list */
+    load_hotlist ();
+
 #   ifndef HAVE_X
     /* Removing this from the X code let's us type C-c */
     load_key_defs ();
@@ -2639,6 +2705,8 @@ int main (int argc, char *argv [])
     /* Virtual File System shutdown */
     vfs_shut ();
 
+    flush_extension_file (); /* does only free memory */
+
 #   ifndef HAVE_X
     /* Miguel, maybe the fix in slang is not required and
      * it could be done by removing the slang_done_screen.
@@ -2647,7 +2715,7 @@ int main (int argc, char *argv [])
     endwin ();
     slang_shutdown ();
 
-    if (console_flag && quit != SUBSHELL_EXIT)
+    if (console_flag && !(quit & SUBSHELL_EXIT))
 	restore_console ();
     if (alternate_plus_minus && (console_flag || xterm_flag)) {
         fprintf (stderr, "\033>"); fflush (stderr);
@@ -2669,7 +2737,7 @@ int main (int argc, char *argv [])
     free (home_dir);
 #endif
     if (print_last_wd) {
-        if (print_last_revert)
+        if (print_last_revert || edit_one_file || view_one_file)
             write (stdout_fd, ".", 1);
         else
 	    write (stdout_fd, last_wd_string, strlen (last_wd_string));
