@@ -355,7 +355,8 @@ static int menubar_callback (Dlg_head *h, WMenu *menubar, int msg, int par)
     return default_proc (h, msg, par);
 }
 
-static int menubar_event    (Gpm_Event *event, WMenu *menubar)
+int
+menubar_event    (Gpm_Event *event, WMenu *menubar)
 {
     int was_active;
     int new_selection;
@@ -442,7 +443,7 @@ WMenu *menubar_new (int y, int x, int cols, Menu menu [], int items)
     init_widget (&menubar->widget, y, x, 1, cols,
                  (callback_fn) menubar_callback,
 		 (destroy_fn)  menubar_destroy,
-		 (mouse_h)     menubar_event);
+		 (mouse_h)     menubar_event, NULL);
     menubar->menu = menu;
     menubar->active = 0;
     menubar->dropped = 0;
