@@ -41,7 +41,6 @@ int xmouse_flag = 0;
 
 #ifdef HAVE_LIBGPM
 static int mouse_d;		/* Handle to the mouse server */
-static int console_fd;		/* Handle to the console */
 #endif
 
 #ifdef DEBUGMOUSE
@@ -56,7 +55,7 @@ void show_mouse_pointer (int x, int y)
 {
 #ifdef HAVE_LIBGPM
     if (use_mouse_p == GPM_MOUSE){
-	Gpm_DrawPointer (x, y, console_fd);
+	Gpm_DrawPointer (x, y, gpm_consolefd);
     }
 #endif
 }
@@ -118,9 +117,6 @@ void init_mouse (void)
 	    conn.minMod      = 0;
 	    conn.maxMod      = 0;
 
-	    if ((console_fd = open("/dev/console", O_WRONLY)) == -1)
-	        return;
-
 	    if ((mouse_d = Gpm_Open (&conn, 0)) == -1)
 	        return;
 
@@ -156,7 +152,6 @@ void shut_mouse (void)
     switch (use_mouse_p){
 #ifdef HAVE_LIBGPM
       case GPM_MOUSE:
-	close (console_fd);
 	Gpm_Close ();
 	break;
 #endif
