@@ -46,13 +46,13 @@
 #define PY	2
 
 #define RX      5
-#define RY      10
+#define RY      11
 
 #define CX	5
 #define CY	2
 
 #define BX	15
-#define BY	15
+#define BY	16
 
 #define OX	34
 #define OY	2
@@ -67,6 +67,8 @@ static int r_but;
 
 #define TOGGLE_VARIABLE 0
 
+extern int use_internal_edit;
+
 static struct {
     char   *text;
     int    *variable;
@@ -74,24 +76,26 @@ static struct {
     WCheck *widget;
     int    hkey;
     int    hpos;
+    char   *tk;
 } check_options [] = {
-   {"safe deLete",       &know_what_am_i_doing, TOGGLE_VARIABLE,    0,'l', 7 },
-   {"cd follows linKs",  &cd_symlinks,       TOGGLE_VARIABLE,       0,'k',14 },
-   {"advanced choWn",    &advanced_chfns,    TOGGLE_VARIABLE,       0,'w',12 },
-   {"lYnx-like motion",  &navigate_with_arrows,TOGGLE_VARIABLE,     0,'y', 1 },
-   {"Rotating dash",     &nice_rotating_dash,TOGGLE_VARIABLE,       0,'r', 0 },
-   {"Complete: show all",&show_all_if_ambiguous,TOGGLE_VARIABLE,    0,'c', 0 },
-   {"Use internal view", &use_internal_view, TOGGLE_VARIABLE,       0,'u', 0 },
-   {"auto mEnus",        &auto_menu,         TOGGLE_VARIABLE,       0,'e', 6 },
-   {"Auto save setup",   &auto_save_setup,   TOGGLE_VARIABLE,       0,'a', 0 },
-   {"shell Patterns",    &easy_patterns,     TOGGLE_VARIABLE,       0,'p', 6 },
-   {"Verbose operation", &verbose,           TOGGLE_VARIABLE,       0,'v', 0 },
-   {"Fast dir reload",   &fast_reload,       toggle_fast_reload,    0,'f', 0 },
-   {"miX all files",     &mix_all_files,     toggle_mix_all_files,  0,'x', 2 },
-   {"Drop down menus",   &drop_menus,        TOGGLE_VARIABLE,       0,'d', 0 },
-   {"Mark moves down",   &mark_moves_down,   TOGGLE_VARIABLE,       0,'m', 0 },
-   {"show Hidden files", &show_dot_files,    toggle_show_hidden,    0,'h', 5 },
-   {"show Backup files", &show_backups,      toggle_show_backup,    0,'b', 5 },
+   {"safe deLete",       &know_what_am_i_doing, TOGGLE_VARIABLE,    0,'l', 7, "safe-del" },
+   {"cd follows linKs",  &cd_symlinks,       TOGGLE_VARIABLE,       0,'k',14, "cd-follow" },
+   {"advanced choWn",    &advanced_chfns,    TOGGLE_VARIABLE,       0,'w',12, "achown" },
+   {"lYnx-like motion",  &navigate_with_arrows,TOGGLE_VARIABLE,     0,'y', 1, "lynx" },
+   {"Rotating dash",     &nice_rotating_dash,TOGGLE_VARIABLE,       0,'r', 0, "rotating" },
+   {"Complete: show all",&show_all_if_ambiguous,TOGGLE_VARIABLE,    0,'c', 0, "completion" },
+   {"Use internal view", &use_internal_view, TOGGLE_VARIABLE,       0,'u', 0, "view-int" },
+   {"use internal edIt", &use_internal_edit, TOGGLE_VARIABLE,       0,'i',15, "edit-int" },
+   {"auto mEnus",        &auto_menu,         TOGGLE_VARIABLE,       0,'e', 6, "auto-menus" },
+   {"Auto save setup",   &auto_save_setup,   TOGGLE_VARIABLE,       0,'a', 0, "auto-save" },
+   {"shell Patterns",    &easy_patterns,     TOGGLE_VARIABLE,       0,'p', 6, "shell-patt" },
+   {"Verbose operation", &verbose,           TOGGLE_VARIABLE,       0,'v', 0, "verbose" },
+   {"Fast dir reload",   &fast_reload,       toggle_fast_reload,    0,'f', 0, "fast-reload" },
+   {"miX all files",     &mix_all_files,     toggle_mix_all_files,  0,'x', 2, "mix-files" },
+   {"Drop down menus",   &drop_menus,        TOGGLE_VARIABLE,       0,'d', 0, "drop-menus" },
+   {"Mark moves down",   &mark_moves_down,   TOGGLE_VARIABLE,       0,'m', 0, "mark-moves" },
+   {"show Hidden files", &show_dot_files,    toggle_show_hidden,    0,'h', 5, "show-hidden" },
+   {"show Backup files", &show_backups,      toggle_show_backup,    0,'b', 5, "show-backup" },
    { 0, 0, 0, 0 }
 };
 
@@ -111,7 +115,7 @@ static int configure_callback (struct Dlg_head *h, int Id, int Msg)
 	
 	draw_box (h, 1, 2, 16, 62);
 	draw_box (h, PY, PX, 8, 27);
-	draw_box (h, OY, OX, 13, 27);
+	draw_box (h, OY, OX, 14, 27);
 	draw_box (h, RY, RX, 5, 27);
 	break;
 
@@ -126,57 +130,50 @@ static void init_configure (void)
 {
     int i;
 
-    conf_dlg = create_dlg (0, 0, 18, 66, dialog_colors,
+    conf_dlg = create_dlg (0, 0, 19, 66, dialog_colors,
 			   configure_callback, "[Options Menu]",
-			   "option", DLG_CENTER);
+			   "option", DLG_CENTER | DLG_GRID);
     x_set_dialog_title (conf_dlg, "Configure options");
 
-    tk_new_frame (conf_dlg, "b.");
     add_widgetl (conf_dlg,
-	button_new (BY, BX+26, B_CANCEL, "[ Cancel ]",'c',2, 0, 0),
+	button_new (BY, BX+26, B_CANCEL, "[ Cancel ]",'c',2, 0, 0, "button-cancel"),
 	XV_WLAY_RIGHTOF);
 
     add_widgetl (conf_dlg,
-	button_new (BY, BX+12, B_EXIT,   "[ Save ]",'s',2, 0, 0),
+	button_new (BY, BX+12, B_EXIT,   "[ Save ]",'s',2, 0, 0, "button-save"),
 	XV_WLAY_RIGHTOF);
     
     add_widgetl (conf_dlg,
-        button_new (BY, BX,    B_ENTER,  "[[ Ok ]]",'o',3, 0, 0),
+        button_new (BY, BX,    B_ENTER,  "[[ Ok ]]",'o',3, 0, 0, "button-ok"),
         XV_WLAY_CENTERROW);
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].hkey, check_options[i].hpos
+#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].hkey, check_options[i].hpos, check_options [i].tk
 
-    tk_new_frame (conf_dlg, "c.");
     /* Add all the checkboxes */
-    for (i = 0; i < 11; i++){
-	check_options [i].widget = check_new (OY + (11-i), OX+2, XTRACT(i));
+    for (i = 0; i < 12; i++){
+	check_options [i].widget = check_new (OY + (12-i), OX+2, XTRACT(i));
 	add_widgetl (conf_dlg, check_options [i].widget,
 	    XV_WLAY_BELOWCLOSE);
     }
 
-    tk_new_frame (conf_dlg, "o.");
-    add_widgetl (conf_dlg, label_new (OY, OX + 1, "Other options"),
+    add_widgetl (conf_dlg, label_new (OY, OX + 1, "Other options", "label-other"),
         XV_WLAY_NEXTCOLUMN);
 
-    tk_new_frame (conf_dlg, "p.");
-    pause_radio = radio_new (RY+1, RX+1, 3, pause_options, 1);
+    pause_radio = radio_new (RY+1, RX+1, 3, pause_options, 1, "pause-radio");
     pause_radio->sel = pause_after_run;
     add_widgetl (conf_dlg, pause_radio, XV_WLAY_BELOWCLOSE);
 
-    add_widgetl (conf_dlg, label_new (RY, RX + 1, "Pause after run..."),
+    add_widgetl (conf_dlg, label_new (RY, RX + 1, "Pause after run...", "label-pause"),
         XV_WLAY_BELOWOF);
 
-    tk_frame (conf_dlg, "o.");
     for (i = 0; i < 6; i++){
-	check_options [i+11].widget = check_new (PY + (6-i), PX+2,
-						  XTRACT(i+11));
-	add_widgetl (conf_dlg, check_options [i+11].widget,
+	check_options [i+12].widget = check_new (PY + (6-i), PX+2,
+						  XTRACT(i+12));
+	add_widgetl (conf_dlg, check_options [i+12].widget,
 	    XV_WLAY_BELOWCLOSE);
     }
-    tk_frame (conf_dlg, "c.");
-    add_widgetl (conf_dlg, label_new (PY, PX + 1, "Panel options"),
+    add_widgetl (conf_dlg, label_new (PY, PX + 1, "Panel options", "label-panel"),
         XV_WLAY_NEXTCOLUMN);
-    tk_end_frame ();
 }
 
 
