@@ -32,7 +32,7 @@
 #include "global.h"
 #include "profile.h"
 
-#define INIFILE "win.ini"
+#define INIFILE "xxx.ini"
 #define STRSIZE 4096
 #define overflow (next == &CharBuffer [STRSIZE-1])
 
@@ -264,12 +264,14 @@ char *get_profile_string (char *AppName, char *KeyName, char *Default,
     return GetSetProfileChar (0, AppName, KeyName, Default, FileName);
 }
 
+#if 0
 int GetProfileString (char * AppName, char * KeyName, char * Default, 
 		      char * ReturnedString, int Size)
 {
     return GetPrivateProfileString (AppName, KeyName, Default,
 				    ReturnedString, Size, INIFILE);
 }
+#endif
 
 int GetPrivateProfileInt (char * AppName, char * KeyName, int Default,
 			   char * File)
@@ -288,10 +290,12 @@ int GetPrivateProfileInt (char * AppName, char * KeyName, int Default,
     return atoi (IntBuf);
 }
 
+#if 0
 int GetProfileInt (char * AppName, char * KeyName, int Default)
 {
     return GetPrivateProfileInt (AppName, KeyName, Default, INIFILE);
 }
+#endif
 
 int WritePrivateProfileString (char * AppName, char * KeyName, char * String,
 				char * FileName)
@@ -299,10 +303,12 @@ int WritePrivateProfileString (char * AppName, char * KeyName, char * String,
     return GetSetProfile (1, AppName, KeyName, String, "", 0, FileName);
 }
 
+#if 0
 int WriteProfileString (char * AppName, char * KeyName, char * String)
 {
     return (WritePrivateProfileString (AppName, KeyName, String, INIFILE));
 }
+#endif
 
 static void dump_keys (FILE *profile, TKeys *p)
 {
@@ -470,9 +476,9 @@ int profile_has_section (char *section_name, char *profile)
 
 void profile_forget_profile (char *file)
 {
-    TProfile *p = Base;
+    TProfile *p;
 
-    for (p = Base; Base; Base = Base->link){
+    for (p = Base; p; p = p->link){
 	if (strcasecmp (file, p->FileName))
 	    continue;
 	p->FileName [0] = 0;
