@@ -72,8 +72,12 @@ WPanel *the_info_panel = 0;
 /* The hook list for the select file function */
 Hook *select_file_hook = 0;
 
+int panel_callback (Dlg_head *h, WPanel *p, int Msg, int Par);
+int panel_event (Gpm_Event *event, WPanel *panel);
+
 #ifndef HAVE_TK
 #   define x_adjust_top_file(p)
+#   define x_reset_sort_labels(x) 
 #endif
 
 #ifdef HAVE_X
@@ -81,15 +85,18 @@ Hook *select_file_hook = 0;
 #else
 #   define x_create_panel(x,y,z) 1;
 #   define x_panel_load_index(p,x)
+#   define x_panel_select_item(a,b,c)
 
-void set_colors (WPanel *panel)
+void
+set_colors (WPanel *panel)
 {
     standend ();
     if (hascolors)
 	attrset (NORMAL_COLOR);
 }
 
-void set_attr (int hilight, int marked)
+void
+set_attr (int hilight, int marked)
 {
     int color;
 
@@ -110,7 +117,8 @@ void set_attr (int hilight, int marked)
 #endif
 
 /* This functions return a string representation of a file entry */
-char *string_file_type (file_entry *fe, int len)
+char *
+string_file_type (file_entry *fe, int len)
 {
     static char buffer [2];
 
@@ -139,12 +147,14 @@ char *string_file_type (file_entry *fe, int len)
     return buffer;
 }
 
-char *string_file_permission (file_entry *fe, int len)
+char *
+string_file_permission (file_entry *fe, int len)
 {
     return string_perm (fe->buf.st_mode);
 }
 
-char *string_file_nlinks (file_entry *fe, int len)
+char *
+string_file_nlinks (file_entry *fe, int len)
 {
     static char buffer [16];
 
@@ -152,17 +162,20 @@ char *string_file_nlinks (file_entry *fe, int len)
     return buffer;
 }
 
-char *string_file_owner (file_entry *fe, int len)
+char *
+string_file_owner (file_entry *fe, int len)
 {
     return get_owner (fe->buf.st_uid);
 }
 
-char *string_file_group (file_entry *fe, int len)
+char *
+string_file_group (file_entry *fe, int len)
 {
     return get_group (fe->buf.st_gid);
 }
 
-char *string_file_size (file_entry *fe, int len)
+char *
+string_file_size (file_entry *fe, int len)
 {
     static char buffer [16];
     int i;
@@ -188,22 +201,26 @@ char *string_file_size (file_entry *fe, int len)
     return buffer;
 }
 
-char *string_file_mtime (file_entry *fe, int len)
+char *
+string_file_mtime (file_entry *fe, int len)
 {
     return file_date (fe->buf.st_mtime);
 }
 
-char *string_file_atime (file_entry *fe, int len)
+char *
+string_file_atime (file_entry *fe, int len)
 {
     return file_date (fe->buf.st_atime);
 }
 
-char *string_file_ctime (file_entry *fe, int len)
+char *
+string_file_ctime (file_entry *fe, int len)
 {
     return file_date (fe->buf.st_ctime);
 }
 
-char *string_file_name (file_entry *fe, int len)
+char *
+string_file_name (file_entry *fe, int len)
 {
     if (len)
         return name_trunc (fe->fname, len);
@@ -211,17 +228,20 @@ char *string_file_name (file_entry *fe, int len)
 	return fe->fname;
 }
 
-char *string_space (file_entry *fe, int len)
+char *
+string_space (file_entry *fe, int len)
 {
     return " ";
 }
 
-char *string_marked (file_entry *fe, int len)
+char *
+string_marked (file_entry *fe, int len)
 {
     return fe->f.marked ? "*" : " ";
 }
 
-char *string_file_perm_octal (file_entry *fe, int len)
+char *
+string_file_perm_octal (file_entry *fe, int len)
 {
     static char buffer [9];
 
@@ -229,7 +249,8 @@ char *string_file_perm_octal (file_entry *fe, int len)
     return buffer;
 }
 
-char *string_inode (file_entry *fe, int len)
+char *
+string_inode (file_entry *fe, int len)
 {
     static char buffer [9];
 
@@ -237,7 +258,8 @@ char *string_inode (file_entry *fe, int len)
     return buffer;
 }
 
-char *string_file_ngid (file_entry *fe, int len)
+char *
+string_file_ngid (file_entry *fe, int len)
 {
     static char buffer [9];
 
@@ -245,7 +267,8 @@ char *string_file_ngid (file_entry *fe, int len)
     return buffer;
 }
 
-char *string_file_nuid (file_entry *fe, int len)
+char *
+string_file_nuid (file_entry *fe, int len)
 {
     static char buffer [9];
 
@@ -260,24 +283,25 @@ static struct {
     int  default_just;
     char *title;
     char *(*string_fn)(file_entry *, int);
+    sortfn *sort_routine;
 } formats [] = {
-{ "name",  12, 1, J_LEFT,  "Name",       string_file_name },
-{ "size",  7,  0, J_RIGHT, "Size",       string_file_size },
-{ "type",  1,  0, J_LEFT,  "",           string_file_type },
-{ "mtime", 12, 0, J_RIGHT, "MTime",      string_file_mtime },
-{ "perm",  10, 0, J_RIGHT, "Permission", string_file_permission },
-{ "mode",  7,  0, J_RIGHT, "OctM",       string_file_perm_octal },
-{ "|",     1,  0, J_RIGHT, "|",          0 },
-{ "nlink", 2,  0, J_RIGHT, "Nl",         string_file_nlinks },
-{ "ngid",  5,  0, J_RIGHT, "GID",        string_file_ngid },
-{ "nuid",  5,  0, J_RIGHT, "UID",        string_file_nuid },
-{ "owner", 8,  0, J_LEFT,  "Owner",      string_file_owner },
-{ "group", 8,  0, J_LEFT,  "Group",      string_file_group },
-{ "atime", 12, 0, J_RIGHT, "ATime",      string_file_atime },
-{ "ctime", 12, 0, J_RIGHT, "CTime",      string_file_ctime },
-{ "space", 1,  0, J_RIGHT, " ",          string_space },
-{ "mark",  1,  0, J_RIGHT, " ",          string_marked },
-{ "inode", 5,  0, J_RIGHT, "Inode",      string_inode },
+{ "name",  12, 1, J_LEFT,  "Name",       string_file_name,       (sortfn *) sort_name },
+{ "size",  7,  0, J_RIGHT, "Size",       string_file_size,       (sortfn *) sort_size },
+{ "type",  1,  0, J_LEFT,  "",           string_file_type,       (sortfn *) sort_type },
+{ "mtime", 12, 0, J_RIGHT, "MTime",      string_file_mtime,      (sortfn *) sort_time },
+{ "perm",  10, 0, J_RIGHT, "Permission", string_file_permission, NULL},
+{ "mode",  7,  0, J_RIGHT, "OctM",       string_file_perm_octal, NULL },
+{ "|",     1,  0, J_RIGHT, "|",          0,                      NULL },
+{ "nlink", 2,  0, J_RIGHT, "Nl",         string_file_nlinks,     (sortfn *) sort_links },
+{ "ngid",  5,  0, J_RIGHT, "GID",        string_file_ngid,       (sortfn *) sort_ngid },
+{ "nuid",  5,  0, J_RIGHT, "UID",        string_file_nuid,       (sortfn *) sort_nuid },
+{ "owner", 8,  0, J_LEFT,  "Owner",      string_file_owner,      (sortfn *) sort_owner },
+{ "group", 8,  0, J_LEFT,  "Group",      string_file_group,      (sortfn *) sort_group },
+{ "atime", 12, 0, J_RIGHT, "ATime",      string_file_atime,      (sortfn *) sort_atime },
+{ "ctime", 12, 0, J_RIGHT, "CTime",      string_file_ctime,      (sortfn *) sort_ctime },
+{ "space", 1,  0, J_RIGHT, " ",          string_space,           NULL },
+{ "mark",  1,  0, J_RIGHT, " ",          string_marked,          NULL },
+{ "inode", 5,  0, J_RIGHT, "Inode",      string_inode,           (sortfn *) sort_inode },
 };
 
 static char *
@@ -308,7 +332,8 @@ to_buffer (char *dest, int just_mode, int len, char *txt)
 }
 
 /* Formats the file number file_index of panel in the buffer dest */
-void format_file (char *dest, WPanel *panel, int file_index, int width)
+void
+format_file (char *dest, WPanel *panel, int file_index, int width)
 {
     int   i, length = 0;
     int   top = panel->fmt_count;
@@ -356,7 +381,8 @@ void format_file (char *dest, WPanel *panel, int file_index, int width)
 }
 
 #ifndef HAVE_X
-void repaint_file (WPanel *panel, int file_index, int mv)
+void
+repaint_file (WPanel *panel, int file_index, int mv)
 {
     int    second_column = 0;
     int	   width, offset;
@@ -400,7 +426,8 @@ void repaint_file (WPanel *panel, int file_index, int mv)
 
 /* FIXME: This function uses hard coded constants */
 /* panel->cols - 22 is not a wise idea */
-void mini_info_brief (WPanel *panel)
+void
+mini_info_brief (WPanel *panel)
 {
     int   isdir = 0;
     char  *sdir_txt = "";
@@ -425,7 +452,8 @@ void mini_info_brief (WPanel *panel)
     printw ("%10s", string_perm (entry.st_mode));
 }
 
-void display_mini_info (WPanel *panel)
+void
+display_mini_info (WPanel *panel)
 {
 #ifndef HAVE_X
     if (!show_mini_info)
@@ -440,15 +468,6 @@ void display_mini_info (WPanel *panel)
 	return;
     }
 
-#if 0
-    if (panel->view_type == view_tree){
-	attrset (NORMAL_COLOR);
-	printw  ("%-*s", panel->widget.cols-2,
-		 name_trunc (panel->cwd, panel->widget.cols));
-	return;
-    }
-#endif
-    
     if (panel->marked){
 	char buffer [100];
 	char *p;
@@ -534,7 +553,8 @@ void display_mini_info (WPanel *panel)
 }
 
 #ifndef HAVE_XVIEW
-void paint_dir (WPanel *panel)
+void
+paint_dir (WPanel *panel)
 {
     int i;
     int hilight;		/* Color used for hilighting */
@@ -561,7 +581,8 @@ void paint_dir (WPanel *panel)
 #ifdef HAVE_X
 #define mini_info_separator(x)
 #else
-static void mini_info_separator (WPanel *panel)
+static void
+mini_info_separator (WPanel *panel)
 {
     if (!show_mini_info)
 	return;
@@ -577,7 +598,8 @@ static void mini_info_separator (WPanel *panel)
 #endif 
 }
 
-void show_dir (WPanel *panel)
+void
+show_dir (WPanel *panel)
 {
     char tmp [200];
 
@@ -619,7 +641,8 @@ void show_dir (WPanel *panel)
 #endif
 
 /* To be used only by long_frame and full_frame to adjust top_file */
-static void adjust_top_file (WPanel *panel)
+static void
+adjust_top_file (WPanel *panel)
 {
     int old_top = panel->top_file;
     
@@ -637,7 +660,8 @@ static void adjust_top_file (WPanel *panel)
 extern void paint_info_panel (WPanel *panel);
 
 /* Repaints the information that changes after a command */
-void panel_update_contents (WPanel *panel)
+void
+panel_update_contents (WPanel *panel)
 {
     show_dir (panel);
 #ifdef HAVE_X
@@ -648,14 +672,16 @@ void panel_update_contents (WPanel *panel)
     display_mini_info (panel);
 }
 
-void paint_panel (WPanel *panel)
+void
+paint_panel (WPanel *panel)
 {
     paint_frame (panel);
     panel_update_contents (panel);
     mini_info_separator (panel);
 }
 
-void Xtry_to_select (WPanel *panel, char *name)
+void
+Xtry_to_select (WPanel *panel, char *name)
 {
     int i;
     char *subdir;
@@ -703,10 +729,17 @@ void Xtry_to_select (WPanel *panel, char *name)
 	return;
 }
 
+#ifdef HAVE_X
+void panel_update_cols (Widget *widget, int frame_size)
+{
+    /* X version gets this information from a different source */
+}
+#else
 /* Sets the cols variable in the widget structure according to the
  * settings selected by the user
  */
-void panel_update_cols (Widget *widget, int frame_size)
+void
+panel_update_cols (Widget *widget, int frame_size)
 {
     int cols, origin;
     
@@ -731,11 +764,10 @@ void panel_update_cols (Widget *widget, int frame_size)
     widget->cols = cols;
     widget->x = origin;
 }
+#endif
 
-int panel_callback (Dlg_head *h, WPanel *p, int Msg, int Par);
-int panel_event    (Gpm_Event *event, WPanel *panel);
-
-static char *panel_save_name (WPanel *panel)
+static char *
+panel_save_name (WPanel *panel)
 {
     extern int saving_setup;
     
@@ -746,7 +778,8 @@ static char *panel_save_name (WPanel *panel)
 	return copy_strings ("Temporal:", panel->panel_name, 0);
 }
 
-static void panel_destroy (WPanel *p)
+static void
+panel_destroy (WPanel *p)
 {
     char *name = panel_save_name (p);
 
@@ -761,12 +794,21 @@ static void panel_destroy (WPanel *p)
     free (name);
 }
 
-void panel_update_format (WPanel *panel)
+static void
+panel_format_modified (WPanel *panel)
+{
+    panel->format_modified = 1;
+    x_reset_sort_labels (panel);
+}
+
+void
+panel_update_format (WPanel *panel)
 {
     char *err;
     
     panel->format = parse_display_format (panel, panel_format (panel),
 					  &err, 0);
+    panel_format_modified (panel);
     if (err)
 	free (err);
     panel_update_cols (&(panel->widget), panel->frame_size);
@@ -774,7 +816,8 @@ void panel_update_format (WPanel *panel)
 
 /* Panel creation */
 /* The parameter specifies the name of the panel for setup retieving */
-WPanel *panel_new (char *panel_name)
+WPanel *
+panel_new (char *panel_name)
 {
     WPanel *panel;
     char *section;
@@ -784,7 +827,7 @@ WPanel *panel_new (char *panel_name)
     /* No know sizes of the panel at startup */
     init_widget (&panel->widget, 0, 0, 0, 0, (callback_fn)
 		 panel_callback, (destroy_fn) panel_destroy,
-		 (mouse_h) panel_event);
+		 (mouse_h) panel_event, NULL);
 
     /* We do not want the cursor */
     widget_want_cursor (panel->widget, 0);
@@ -805,13 +848,16 @@ WPanel *panel_new (char *panel_name)
     panel->format    = 0;
     panel->fmt_count = 0;
     panel->dirty     = 1;
-    panel->has_dir_sizes = 0;
+    panel->searching = 0;
     panel->dirs_marked = 0;
     panel->is_panelized = 0;
-    panel->searching = 0;
+    panel->has_dir_sizes = 0;
+    panel->format_modified = 1;
+    
     panel->panel_name = strdup (panel_name);
     panel->user_format = strdup (DEFAULT_USER_FORMAT);
     panel->mini_status_format = strdup (DEFAULT_USER_FORMAT);
+    
     panel->search_buffer [0] = 0;
     panel->frame_size = frame_half;
     section = copy_strings ("Temporal:", panel->panel_name, 0);
@@ -829,22 +875,12 @@ WPanel *panel_new (char *panel_name)
     return panel;
 }
 
-void panel_reload (WPanel *panel)
+void
+panel_reload (WPanel *panel)
 {
     int i;
     struct stat current_stat;
 
-#if 0
-    if (!PANEL_ISVIEW (panel))
-	return;
-    
-    if (panel->view_type == view_tree){
-	tree_rescan_cmd ();
-	paint_panel (panel);
-	return;
-    }
-#endif
-    
     if (fast_reload
 	&& !stat (panel->cwd, &current_stat)
 	&& current_stat.st_ctime == panel->dir_stat.st_ctime
@@ -855,6 +891,7 @@ void panel_reload (WPanel *panel)
 	char *last_slash;
 
 	if (panel->cwd [0] == PATH_SEP && panel->cwd [1] == 0){
+	    clean_dir (&panel->dir, panel->count);
 	    panel->count = set_zero_dir (&panel->dir);
 	    return;
 	}
@@ -880,14 +917,51 @@ void panel_reload (WPanel *panel)
 	}
 }
 
-void paint_frame (WPanel *panel)
+#ifdef HAVE_X
+void
+paint_frame (WPanel *panel)
 {
-#ifndef HAVE_X
+    const int top = panel->fmt_count;
+    char *txt, buffer [30], new_buffer [80];
+    int header_len, spaces, extra;
+    int i = 0, j;
+    
+    show_dir (panel);
+    x_sort_label_start (panel);
+    for (i = 0; i < top; i++){ 
+	txt = panel->format [i].title;
+	header_len = strlen (txt);
+	if (header_len > panel->format [i].field_len){
+	    strcpy (buffer, txt);
+	    txt = buffer;
+	    txt [panel->format [i].field_len] = 0;
+	    header_len = strlen (txt);
+	}
+	
+	spaces = (panel->format [i].field_len - header_len) / 2;
+	extra  = (panel->format [i].field_len - header_len) % 2;
+	sprintf (new_buffer, "%*s%-s%*s", spaces, "",
+		 txt, spaces+extra, "");
+
+	for (j = 0; j < ELEMENTS(formats); j++){
+	    if (strcmp (panel->format [i].title, formats [j].title))
+		continue;
+	    x_add_sort_label (panel, i, new_buffer, panel->format [i].title, formats [j].sort_routine);
+	}
+    }
+    panel->format_modified = 0;
+}
+
+#else
+void
+paint_frame (WPanel *panel)
+{
+    const int top = panel->fmt_count;
     int  header_len;
     int  spaces, extra;
-    int  i, top, side, width;
+    int  i, side, width;
     char *txt, buffer[30]; /*Hope that this is enough ;-) */
-
+	
     if (!panel->split)
 	adjust_top_file (panel);
     
@@ -906,7 +980,7 @@ void paint_frame (WPanel *panel)
 	else
 	    width = panel->widget.cols - 2;
 	
-	for (i = 0, top = panel->fmt_count; i < top; i++){
+	for (i = 0; i < top; i++){
 
 	    if (panel->format [i].string_fn){
 		txt = panel->format [i].title;
@@ -934,12 +1008,11 @@ void paint_frame (WPanel *panel)
 	if (width > 0)
 	    printw ("%*s", width, "");
     }
-#else
-    show_dir (panel);
-#endif
 }
+#endif
 
-static char *parse_panel_size (WPanel *panel, char *format)
+static char *
+parse_panel_size (WPanel *panel, char *format)
 {
     format = skip_separators (format);
 
@@ -983,8 +1056,8 @@ static char *parse_panel_size (WPanel *panel, char *format)
    
 */
 
-format_e *parse_display_format (WPanel *panel, char *format, 
-				char **error, int isstatus)
+format_e *
+parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 {
 #define MAX_EXPAND 4
     
@@ -1139,7 +1212,8 @@ format_e *parse_display_format (WPanel *panel, char *format,
 }
 
 /* Switches the panel to the mode specified in the format */
-char *set_panel_format (WPanel *p, char *format)
+char *
+set_panel_format (WPanel *p, char *format)
 {
     char *err;
 
@@ -1150,6 +1224,7 @@ char *set_panel_format (WPanel *p, char *format)
     if (err)
 	return err;
 
+    panel_format_modified (p);
     panel_update_cols (&(p->widget), p->frame_size);
     paint_frame (p);
     paint_panel (p);
@@ -1158,7 +1233,8 @@ char *set_panel_format (WPanel *p, char *format)
 }
 
 /* Given the panel->view_type returns the format string to be parsed */
-char *panel_format (WPanel *panel)
+char *
+panel_format (WPanel *panel)
 {
     switch (panel->list_type){
 
@@ -1183,7 +1259,8 @@ char *panel_format (WPanel *panel)
 /*                          */
 
 /* Returns the number of items in the given panel */
-int ITEMS (WPanel *p)
+int
+ITEMS (WPanel *p)
 {
 #ifdef HAVE_TK
     return p->widget.lines;
@@ -1196,7 +1273,8 @@ int ITEMS (WPanel *p)
 }
 
 /* This function sets redisplays the selection */
-void select_item (WPanel *panel)
+void
+select_item (WPanel *panel)
 {
     int repaint = 0;
     int items = ITEMS (panel);
@@ -1261,7 +1339,8 @@ void select_item (WPanel *panel)
 }
 
 /* Clears all files in the panel, used only when one file was marked */
-void unmark_files (WPanel *panel)
+void
+unmark_files (WPanel *panel)
 {
     int i;
 
@@ -1275,15 +1354,20 @@ void unmark_files (WPanel *panel)
     panel->total = 0;
 }
 
-void unselect_item (WPanel *panel)
+#ifdef HAVE_X
+void
+unselect_item (WPanel *panel)
+{
+    x_unselect_item (panel);
+}
+#else
+void
+unselect_item (WPanel *panel)
 {
-#ifndef HAVE_X
     set_attr (0, selection (panel)->f.marked);
     repaint_file (panel, panel->selected, 1);
-#else
-    x_unselect_item (panel);
-#endif
 }
+#endif
 
 /*                           */
 /* Panel key binded commands */
@@ -1310,7 +1394,8 @@ static void move_down (WPanel *panel)
     select_item (panel);
 }
 
-static void move_up (WPanel *panel)
+static void
+move_up (WPanel *panel)
 {
     if (panel->selected == 0)
 	return;
@@ -1330,7 +1415,8 @@ static void move_up (WPanel *panel)
 
 
 /* Changes the selection by lines (may be negative) */
-static void move_selection (WPanel *panel, int lines)
+static void
+move_selection (WPanel *panel, int lines)
 {
     int new_pos;
     int adjust = 0;
@@ -1367,7 +1453,8 @@ static void move_selection (WPanel *panel, int lines)
     select_item (panel);
 }
 
-static int move_left (WPanel *panel, int c_code)
+static int
+move_left (WPanel *panel, int c_code)
 {
     if (panel->split){
 	move_selection (panel, -llines (panel));
@@ -1376,7 +1463,8 @@ static int move_left (WPanel *panel, int c_code)
 	return maybe_cd (c_code, 0);
 }
 
-static int move_right (WPanel *panel, int c_code)
+static int
+move_right (WPanel *panel, int c_code)
 {
     if (panel->split){
 	move_selection (panel, llines (panel));
@@ -1385,7 +1473,8 @@ static int move_right (WPanel *panel, int c_code)
 	return maybe_cd (c_code, 1);
 }
 
-static void prev_page (WPanel *panel)
+static void
+prev_page (WPanel *panel)
 {
     int items;
 
@@ -1413,7 +1502,8 @@ static void prev_page (WPanel *panel)
 #endif
 }
 
-static void prev_page_key (WPanel *panel)
+static void
+prev_page_key (WPanel *panel)
 {
     if (console_flag && ctrl_pressed ()){
 	do_cd ("..", cd_exact);
@@ -1421,7 +1511,8 @@ static void prev_page_key (WPanel *panel)
 	prev_page (panel);
 }
 
-static void next_page (WPanel *panel)
+static void
+next_page (WPanel *panel)
 {
     int items;
 
@@ -1461,14 +1552,16 @@ static void next_page_key (WPanel *panel)
 	next_page (panel);
 }
 
-static void goto_top_file (WPanel *panel)
+static void
+goto_top_file (WPanel *panel)
 {
     unselect_item (panel);
     panel->selected = panel->top_file;
     select_item (panel);
 }
 
-static void goto_middle_file (WPanel *panel)
+static void
+goto_middle_file (WPanel *panel)
 {
     unselect_item (panel);
     panel->selected = panel->top_file + (ITEMS (panel)/2);
@@ -1477,7 +1570,8 @@ static void goto_middle_file (WPanel *panel)
     select_item (panel);
 }
 
-static void goto_bottom_file (WPanel *panel)
+static void
+goto_bottom_file (WPanel *panel)
 {
     unselect_item (panel);
     panel->selected = panel->top_file + ITEMS (panel)-1;
@@ -1486,7 +1580,8 @@ static void goto_bottom_file (WPanel *panel)
     select_item (panel);
 }
 
-static void move_home (WPanel *panel)
+static void
+move_home (WPanel *panel)
 {
     if (panel->selected == 0)
 	return;
@@ -1514,7 +1609,8 @@ static void move_home (WPanel *panel)
     select_item (panel);
 }
 
-static void move_end (WPanel *panel)
+static void
+move_end (WPanel *panel)
 {
     if (panel->selected == panel->count-1)
 	return;
@@ -1540,7 +1636,8 @@ static void move_end (WPanel *panel)
 }
 
 /* This routine marks a file or a directory */
-void do_file_mark (WPanel *panel, int idx, int mark)
+void
+do_file_mark (WPanel *panel, int idx, int mark)
 {
     if (panel->dir.list [idx].f.marked == mark)
         return;
@@ -1576,7 +1673,8 @@ void do_file_mark (WPanel *panel, int idx, int mark)
     }
 }
 
-static void do_mark_file (WPanel *panel, int do_move)
+static void
+do_mark_file (WPanel *panel, int do_move)
 {
     do_file_mark (panel, panel->selected, selection (panel)->f.marked ? 0 : 1);
     if (mark_moves_down && do_move)
@@ -1584,13 +1682,15 @@ static void do_mark_file (WPanel *panel, int do_move)
     display_mini_info (panel);
 }
 
-static void mark_file (WPanel *panel)
+static void
+mark_file (WPanel *panel)
 {
     do_mark_file (panel, 1);
 }
 
 /* Incremental search of a file name in the panel */
-static void do_search (WPanel *panel, int c_code)
+static void
+do_search (WPanel *panel, int c_code)
 {
     int l, i;
     int wrapped = 0;
@@ -1616,10 +1716,8 @@ static void do_search (WPanel *panel, int c_code)
 	    wrapped = 1;
 	}
 	if (strncmp (panel->dir.list [i].fname, panel->search_buffer, l) == 0){
+	    unselect_item (panel);
 	    panel->selected = i;
-#if 0
-		panel->top_file = panel->selected - panel->lines/2; 
-#endif
 	    select_item (panel);
 	    found = 1;
 	    break;
@@ -1632,7 +1730,8 @@ static void do_search (WPanel *panel, int c_code)
 #endif
 }
 
-static void start_search (WPanel *panel)
+static void
+start_search (WPanel *panel)
 {
     if (panel->searching){
 	move_down (panel);
@@ -1641,13 +1740,12 @@ static void start_search (WPanel *panel)
 	panel->searching = 1;
 	panel->search_buffer [0] = 0;
 	display_mini_info (panel);
-#ifndef HAVE_X
-	refresh ();
-#endif
+	mc_refresh ();
     }
 }
 
-void do_enter (WPanel *panel)
+void
+do_enter (WPanel *panel)
 {
     if (S_ISDIR (selection (panel)->buf.st_mode)
 	|| link_isdir (selection (panel))){
@@ -1656,18 +1754,15 @@ void do_enter (WPanel *panel)
 	if (is_exe (selection (panel)->buf.st_mode) &&
 	    if_link_is_exe (selection (panel))) {
 	    if (vfs_current_is_local ()) {
-#ifdef _OS_NT
-	        char *tmp = copy_strings (".\\", selection (panel)->fname, 0);
-#else
-	        char *tmp = copy_strings ("./", selection (panel)->fname, 0);
-#endif
+	        char *tmp = copy_strings (".", PATH_SEP_STR, selection (panel)->fname, 0);
+
 	        execute (tmp);
 	        free (tmp);
 	    } else if (vfs_current_is_extfs ()) {
 	        char *tmp = vfs_get_current_dir();
 	        char *tmp2;
-	        if (tmp [strlen (tmp) - 1] != '/')
-	            tmp2 = copy_strings (tmp, "/", selection (panel)->fname, 0);
+	        if (tmp [strlen (tmp) - 1] != PATH_SEP)
+	            tmp2 = copy_strings (tmp, PATH_SEP_STR, selection (panel)->fname, 0);
 	        else
 	            tmp2 = copy_strings (tmp, selection (panel)->fname, 0);
 	        extfs_run(tmp2);
@@ -1679,7 +1774,8 @@ void do_enter (WPanel *panel)
     }
 }
 
-static void chdir_other_panel (WPanel *panel)
+static void
+chdir_other_panel (WPanel *panel)
 {
     char *new_dir;
 
@@ -1701,7 +1797,8 @@ static void chdir_other_panel (WPanel *panel)
     free (new_dir);
 }
 
-static void chdir_to_readlink (WPanel *panel)
+static void
+chdir_to_readlink (WPanel *panel)
 {
     char *new_dir;
 
@@ -1782,6 +1879,7 @@ static key_map panel_keymap [] = {
     { KEY_F(3),   view_cmd },
     { KEY_F(13),  view_simple_cmd },
     { KEY_F(4),   edit_cmd },
+    { KEY_F(14),  edit_cmd_new },
     { KEY_F(5),   copy_cmd },
     { KEY_F(6),   ren_cmd },
     { KEY_F(7),   mkdir_cmd },
@@ -1793,13 +1891,15 @@ static key_map panel_keymap [] = {
     { 0, 0 }
 };
     
-static inline int panel_key (WPanel *panel, int key)
+static inline int
+panel_key (WPanel *panel, int key)
 {
     int i;
 
     for (i = 0; panel_keymap [i].key_code; i++){
 	if (key == panel_keymap [i].key_code){
-	    panel->searching = 0;
+	    if (panel_keymap [i].fn != start_search)
+               panel->searching = 0;
 	    (*panel_keymap [i].fn)(panel);
 	    return 1;
 	}
@@ -1812,6 +1912,11 @@ static inline int panel_key (WPanel *panel, int key)
     if (key == KEY_RIGHT)
 	move_right (panel, key);
 
+    if (is_abort_char (key)) {
+	panel->searching = 0;
+	display_mini_info (panel);
+    }
+
     if (panel->searching){
 	do_search (panel, key);
 	return 1;
@@ -1831,7 +1936,8 @@ static inline int panel_key (WPanel *panel, int key)
     return 1;
 }
 
-int panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
+int
+panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 {
     switch (msg){
     case WIDGET_INIT:
@@ -1880,13 +1986,13 @@ int panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 	
     case WIDGET_UNFOCUS:
 	/* Janne: look at this for the multiple panel options */
-	panel->active = 0;
 	if (panel->searching){
 	    panel->searching = 0;
 	    display_mini_info (panel);
 	}
 	show_dir (panel);
 	unselect_item (panel);
+	panel->active = 0;
 	return 1;
 	
     case WIDGET_KEY:
@@ -1901,13 +2007,15 @@ int panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 /*                                     */
 static int mouse_marking = 0;
 
-static void mouse_toggle_mark (WPanel *panel)
+static void
+mouse_toggle_mark (WPanel *panel)
 {
     do_mark_file (panel, 0);
     mouse_marking = selection (panel)->f.marked;
 }
 
-static void mouse_set_mark (WPanel *panel)
+static void
+mouse_set_mark (WPanel *panel)
 {
     if (mouse_marking && !(selection (panel)->f.marked))
 	do_mark_file (panel, 0);
@@ -1915,7 +2023,8 @@ static void mouse_set_mark (WPanel *panel)
 	do_mark_file (panel, 0);
 }
 
-static inline int mark_if_marking (WPanel *panel, Gpm_Event *event)
+static inline int
+mark_if_marking (WPanel *panel, Gpm_Event *event)
 {
     if (event->buttons & GPM_B_RIGHT){
 	if (event->type & GPM_DOWN)
@@ -1927,14 +2036,50 @@ static inline int mark_if_marking (WPanel *panel, Gpm_Event *event)
     return 0;
 }
 
+WPanel *
+get_other_panel (int line)
+{
+    if (get_other_type () != view_listing){
+	fprintf (stderr, "Fatal: used opanel with view_type != listing\n\r");
+	fprintf (stderr, "Report this error to mc-devel@nuclecu.unam.mx\n\r");
+	fprintf (stderr, "Error line number: %d\n\r", line);
+	exit (1);
+    }
+    return (WPanel *) get_panel_widget (get_other_index ());
+}
+
+void
+file_mark (WPanel *panel, int index, int val)
+{
+    panel->dir.list [index].f.marked = val;
+    x_panel_select_item (panel, index, val);
+}
+
+#ifdef HAVE_TK
+
+sortfn *
+get_sort_fn (char *name)
+{
+    int i;
+
+    /* First, try the long name options, from dir.c */
+    for (i = 0; i < SORT_TYPES_TOTAL; i++)
+	if (strcmp (name, sort_orders [i].sort_name) == 0)
+	    return sort_orders [i].sort_fn;
+
+    /* Then try the short name options, from our local table */
+    for (i = 0; i < ELEMENTS (formats); i++){
+	if (strcmp (name, formats [i].title) == 0 && formats [i].sort_routine)
+	    return formats [i].sort_routine;
+    }
+    return NULL;
+}
+
 /* not static because it's called from Tk's version */
-int panel_event (Gpm_Event *event, WPanel *panel)
+int
+panel_event (Gpm_Event *event, WPanel *panel)
 {
-#ifdef HAVE_X
     const int lines = panel->count;
-#else
-    const int lines = llines (panel);
-#endif
     
     int my_index;
     extern void change_panel (void);
@@ -1947,24 +2092,12 @@ int panel_event (Gpm_Event *event, WPanel *panel)
 
 	if (event->y <= 0){
 	    mark_if_marking (panel, event);
-#ifndef HAVE_X
-	    if (mouse_move_pages)
-		prev_page (panel);
-	    else
-		move_up (panel);
-#endif
 	    return MOU_REPEAT;
 	}
 
 	if (!((panel->top_file + event->y <= panel->count) &&
 	      event->y <= lines)){
 	    mark_if_marking (panel, event);
-#ifndef HAVE_X
-	    if (mouse_move_pages)
-		next_page (panel);
-	    else
-		move_down (panel);
-#endif
 	    return MOU_REPEAT;
 	}
 	my_index = panel->top_file + event->y - 1;
@@ -1992,21 +2125,63 @@ int panel_event (Gpm_Event *event, WPanel *panel)
     return MOU_NORMAL;
 }
 
-WPanel *get_other_panel (int line)
-{
-    if (get_other_type () != view_listing){
-	fprintf (stderr, "Fatal: used opanel with view_type != listing\n\r");
-	fprintf (stderr, "Report this error to mc-devel@nuclecu.unam.mx\n\r");
-	fprintf (stderr, "Error line number: %d\n\r", line);
-	exit (1);
-    }
-    return (WPanel *) get_panel_widget (get_other_index ());
-}
+#else
 
-void file_mark (WPanel *panel, int index, int val)
+int
+panel_event (Gpm_Event *event, WPanel *panel)
 {
-    panel->dir.list [index].f.marked = val;
-#ifdef HAVE_X
-    x_panel_select_item (panel, index, val);
-#endif
+    const int lines = llines (panel);
+    
+    int my_index;
+    extern void change_panel (void);
+
+    event->y -= 2;
+    if ((event->type & (GPM_DOWN|GPM_DRAG))){
+
+	if (panel != (WPanel *) current_dlg->current->widget)
+	    change_panel ();
+
+	if (event->y <= 0){
+	    mark_if_marking (panel, event);
+	    if (mouse_move_pages)
+		prev_page (panel);
+	    else
+		move_up (panel);
+	    return MOU_REPEAT;
+	}
+
+	if (!((panel->top_file + event->y <= panel->count) &&
+	      event->y <= lines)){
+	    mark_if_marking (panel, event);
+	    if (mouse_move_pages)
+		next_page (panel);
+	    else
+		move_down (panel);
+	    return MOU_REPEAT;
+	}
+	my_index = panel->top_file + event->y - 1;
+	if (panel->split){
+	    if (event->x > ((panel->widget.cols-2)/2))
+		my_index += llines (panel);
+	}
+
+	if (my_index >= panel->count)
+	    my_index = panel->count - 1;
+	
+	if (my_index != panel->selected){
+	    unselect_item (panel);
+	    panel->selected = my_index;
+	    select_item (panel);
+	}
+
+	/* This one is new */
+	mark_if_marking (panel, event);
+	
+    } else if ((event->type & (GPM_UP|GPM_DOUBLE)) == (GPM_UP|GPM_DOUBLE)){
+            if (event->y > 0 && event->y <= lines)
+		do_enter (panel);
+    }
+    return MOU_NORMAL;
 }
+#endif
+
