@@ -42,11 +42,15 @@
 #include "layout.h"
 #include "menu.h"		/* menubar_visible declaration */
 #include "win.h"		/* lookup_key */
+#include "cmd.h"
+#include "dirhist.h"		/* Directory history routines */
+
 #include "../vfs/vfs.h"
 #ifdef USE_NETCODE
 #   include "../vfs/ftpfs.h"
 extern int use_netrc;
 extern int ftpfs_retry_seconds;
+extern int ftpfs_use_passive_connections;
 #endif
 
 /* "$Id: setup.c,v 1.9 1995/02/21 19:07:13 miguel Exp $" */
@@ -144,6 +148,7 @@ static struct {
     { "align_extensions", &align_extensions },
     { "auto_menu", &auto_menu },
     { "use_internal_view", &use_internal_view },
+    { "use_internal_edit", &use_internal_edit },
     { "clear_before_exec", &clear_before_exec },
     { "mix_all_files", &mix_all_files },
     { "fast_reload", &fast_reload },
@@ -187,6 +192,8 @@ static struct {
     { "ftpfs_directory_timeout", &ftpfs_directory_timeout },
     { "use_netrc", &use_netrc },
     { "ftpfs_retry_seconds", &ftpfs_retry_seconds },
+    { "ftpfs_always_use_proxy", &ftpfs_always_use_proxy },
+    { "ftpfs_use_passive_connections", &ftpfs_use_passive_connections },
 #endif
 #endif    
     { 0, 0 }
@@ -303,6 +310,8 @@ void save_setup (void)
     save_hotlist ();
     save_panelize ();
     save_panel_types ();
+    directory_history_save ();
+    
 #ifdef USE_VFS
 #ifdef USE_NETCODE
     WritePrivateProfileString ("Misc", "ftpfs_password",
@@ -429,8 +438,6 @@ void load_setup (void)
 
     load_layout (profile);
 
-    /* The directory hot list */
-    load_hotlist ();
     load_panelize ();
 
     startup_left_mode = load_mode ("New Left Panel");
@@ -466,6 +473,8 @@ void load_setup (void)
     load_string ("Colors", getenv ("TERM"), "",
 			     term_color_string, sizeof (term_color_string));
 
+    /* Load the directory history */
+    directory_history_load ();
     /* Remove the temporal entries */
     profile_clean_section ("Temporal:New Left Panel", profile_name);
     profile_clean_section ("Temporal:New Right Panel", profile_name);
@@ -496,6 +505,7 @@ void done_setup (void)
     free (profile_name);
     done_hotlist ();
     done_panelize ();
+    directory_history_free ();
 }
 
 void load_keys_from_section (char *terminal, char *profile_name)
