@@ -42,7 +42,7 @@
 #   include <unistd.h>	/* For pipe, fork, setsid, access etc */
 #endif
 
-#ifdef IS_AIX
+#ifdef HAVE_SYS_SELECT_H
 #   include <sys/select.h>
 #endif
 
@@ -442,7 +442,7 @@ void init_subshell (void)
 	    break;
 
 	    case ZSH:
-	    execl (shell, "zsh", NULL);
+	    execl (shell, "zsh", "+Z", NULL);
 	    break;
 	}
 
@@ -844,7 +844,7 @@ void sigchld_handler (int sig)
 	{
 	    subshell_alive = FALSE;
 	    if (WIFEXITED (status) && WEXITSTATUS (status) != FORK_FAILURE)
-		quit = SUBSHELL_EXIT;  /* Exited normally */
+		quit |= SUBSHELL_EXIT;  /* Exited normally */
 	}
 
 	/* }}} */
@@ -1116,27 +1116,30 @@ static int pty_open_slave (const char *pty_name)
     }
 
 #if !defined(__osf__)
-    if (ioctl (pty_slave, I_PUSH, "ptem") == -1)
-    {
-	perror ("ioctl (pty_slave, I_PUSH, \"ptem\")");
-	close (pty_slave);
-	return -1;
-    }
+    if (!ioctl (pty_slave, I_FIND, "ptem"))
+	if (ioctl (pty_slave, I_PUSH, "ptem") == -1)
+	{
+	    perror ("ioctl (pty_slave, I_PUSH, \"ptem\")");
+	    close (pty_slave);
+	    return -1;
+	}
 	
-    if (ioctl (pty_slave, I_PUSH, "ldterm") == -1)
-    {
-	perror ("ioctl (pty_slave, I_PUSH, \"ldterm\")");
-	close (pty_slave);
-	return -1;
-    }
+    if (!ioctl (pty_slave, I_FIND, "ldterm"))
+        if (ioctl (pty_slave, I_PUSH, "ldterm") == -1)
+	{
+	    perror ("ioctl (pty_slave, I_PUSH, \"ldterm\")");
+	    close (pty_slave);
+	    return -1;
+	}
 
 #if !defined(sgi) && !defined(__sgi)
-    if (ioctl (pty_slave, I_PUSH, "ttcompat") == -1)
-    {
-	perror ("ioctl (pty_slave, I_PUSH, \"ttcompat\")");
-	close (pty_slave);
-	return -1;
-    }
+    if (!ioctl (pty_slave, I_FIND, "ttcompat"))
+        if (ioctl (pty_slave, I_PUSH, "ttcompat") == -1)
+	{
+	    perror ("ioctl (pty_slave, I_PUSH, \"ttcompat\")");
+	    close (pty_slave);
+	    return -1;
+	}
 #endif /* sgi || __sgi */
 #endif /* __osf__ */
 
