@@ -376,6 +376,10 @@ void tree_destroy (WTree *tree)
 	free (old->name);
 	free (old);
     }
+    if (tree->tree_shown){
+	free (tree->tree_shown);
+	tree->tree_shown = 0;
+    }
     tree->selected_ptr = tree->tree_first = tree->tree_last = NULL;
 }
 
@@ -423,7 +427,7 @@ void do_tree_check (WTree *tree, const char *subname)
 {
     char *name;
     tree_entry *current, *base;
-    int flag = 0, len;
+    int flag = 1, len;
 
     /* Calculate the full name of the subdirectory */
     if (subname [0] == '.' &&
@@ -898,7 +902,7 @@ void tree_event (WTree *tree, int y)
 
 static void chdir_sel (WTree *tree);
 
-static maybe_chdir (WTree *tree)
+static void maybe_chdir (WTree *tree)
 {
     if (!(xtree_mode && tree->is_panel))
 	return;
@@ -1258,7 +1262,24 @@ static void chdir_sel (WTree *tree)
 
 static void start_search (WTree *tree)
 {
-    tree->searching = 1;
+    int i;
+
+    if (tree->searching){
+	/* set navigation mode temporarily to 'Static' because in 
+	 * dynamic navigation mode tree_move_forward will not move
+	 * to a lower sublevel if necessary (sequent searches must
+	 * start with the directory followed the last found directory)
+         */	 
+	i = tree_navigation_flag;
+	tree_navigation_flag = 0;
+	tree_move_forward (tree, 1);
+	tree_navigation_flag = i;
+	tree_do_search (tree, 0);
+    }
+    else {
+	tree->searching = 1;
+	tree->search_buffer[0] = 0;
+    }
 }
 
 static key_map tree_keymap [] = {
@@ -1280,6 +1301,7 @@ static key_map tree_keymap [] = {
     { XCTRL('p'), move_down    },
     { XCTRL('s'), start_search },
     { ALT('s'),   start_search },
+    { 0, 0 }
     };
 
 static inline int tree_key (WTree *tree, int key)
@@ -1288,7 +1310,8 @@ static inline int tree_key (WTree *tree, int key)
 
     for (i = 0; tree_keymap [i].key_code; i++){
 	if (key == tree_keymap [i].key_code){
-	    tree->search_buffer [0] = 0;
+	    if (tree_keymap [i].fn != start_search)
+	        tree->searching = 0;
 	    (*tree_keymap [i].fn)(tree);
 	    show_tree (tree);
 	    break;
@@ -1302,6 +1325,10 @@ static inline int tree_key (WTree *tree, int key)
     if (key == KEY_RIGHT)
 	move_right (tree);
 
+    if (is_abort_char (key)) {
+	tree->searching = 0;
+    }
+
     if (tree->searching){
 	tree_do_search (tree, key);
 	return 1;
@@ -1385,7 +1412,7 @@ WTree *tree_new (int is_panel, int y, int x, int lines, int cols)
     WTree *tree = xmalloc (sizeof (WTree), "tree_new");
 
     init_widget (&tree->widget, y, x, lines, cols, tcallback,
-		 (destroy_fn) tree_destroy, (mouse_h) event_callback);
+		 (destroy_fn) tree_destroy, (mouse_h) event_callback, NULL);
     tree->is_panel = is_panel;
     tree->selected_ptr = 0;
     tree->tree_shown = 0;
