@@ -42,7 +42,7 @@
 #include <grp.h>
 #include <string.h>
 #include <ctype.h>
-#ifdef IS_AIX
+#ifdef HAVE_SYS_SELECT_H
 #  include <sys/select.h>
 #endif
 
@@ -275,7 +275,9 @@ char *strip_home(char *dir)
             }
         }
     } else if (!strncmp (dir, "mc:", 3)) {
-        if ((p = strchr (dir + 3, PATH_SEP)) != NULL) {
+        char  *pth = (dir[3] == '/' && dir[4] == '/') ? dir + 5 : dir + 3;
+
+        if ((p = strchr (pth, PATH_SEP)) != NULL) {
             *p = 0;
             if ((q = mcfs_gethome (dir)) == NULL)
 		return dir;
@@ -485,8 +487,10 @@ char *load_file (char *filename)
 
     if (read_size > 0)
 	return data;
-    else
+    else {
+	free (data);
 	return 0;
+    }
 }
 
 char *file_date (time_t when)
