@@ -43,7 +43,7 @@
 #include <grp.h>
 #include <string.h>
 #include <ctype.h>
-#ifdef IS_AIX
+#ifdef HAVE_SYS_SELECT_H
 #  include <sys/select.h>
 #endif
 #ifdef SCO_FLAVOR
@@ -248,6 +248,16 @@ char *tilde_expand (char *directory)
     return directory;
 }
 
+int
+set_nonblocking (int fd)
+{
+    int val;
+
+    val = fcntl (fd, F_GETFL, 0);
+    val |= O_NONBLOCK;
+    return fcntl (fd, F_SETFL, val) != -1;
+}
+
 /* Pipes are guaranteed to be able to hold at least 4096 bytes */
 /* More than that would be unportable */
 #define MAX_PIPE_SIZE 4096
@@ -400,15 +410,15 @@ int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
 		      }
 	    	default:
 	    	    {
-	    	    	char buffer [512];
+	    	    	char buffer [8192];
 	    	    	int i;
 
 	    	    	close (pipe0 [0]);
 	    	    	close (pipe1 [0]);
 	    	    	close (pipe1 [1]);
 	    	    	while ((i = mc_read (inhandle, buffer,
-                                             (inlen == -1 || inlen > 512) 
-                                             ? 512 : inlen)) > 0) {
+                                             (inlen == -1 || inlen > 8192) 
+                                             ? 8192 : inlen)) > 0) {
 	    	    	    write (pipe0 [1], buffer, i);
 	    	    	    if (inlen != -1) {
 			        inlen -= i;
@@ -650,3 +660,12 @@ void my_statfs (struct my_statfs *myfs_stats, char *path)
 	myfs_stats->nodes = 0;
     }
 }
+
+#ifdef HAVE_GET_PROCESS_STATS
+#    include <sys/procstats.h>
+
+int gettimeofday (struct timeval *tp, void *tzp)
+{
+  return get_process_stats(tp, PS_SELF, 0, 0);
+}
+#endif
