@@ -1,10 +1,11 @@
 /* Widgets for the Midnight Commander
 
-   Copyright (C) 1994, 1995 the Free Software Foundation
+   Copyright (C) 1994, 1995, 1996 the Free Software Foundation
    
    Authors: 1994, 1995 Radek Doulik
             1994, 1995 Miguel de Icaza
             1995 Jakub Jelinek
+	    1996 Andrej Borsenkow
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -68,7 +69,8 @@ static int button_event (Gpm_Event *event, WButton *b);
 
 int quote = 0;
 
-static int button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
+static int
+button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 {
     int stop = 0;
 
@@ -96,7 +98,18 @@ static int button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 	}
 	return 1;
 
-#ifndef HAVE_X
+#ifdef HAVE_X
+    case WIDGET_FOCUS:
+    case WIDGET_CURSOR:
+    {
+	char *s = b->action == B_ENTER ? ".button" : "";
+	
+	tk_evalf ("focus %s%s", (char *)(b->widget.wdata)+1, s);
+	/* Do not call default_proc: we did the tk focus command */
+	return 1;
+    }
+#else
+	
     case WIDGET_CURSOR:
 	widget_move (&b->widget, 0, b->hotpos);
 	return 1;
@@ -119,7 +132,8 @@ static int button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
     return default_proc (h, Msg, Par);
 }
 
-static int button_event (Gpm_Event *event, WButton *b)
+static int
+button_event (Gpm_Event *event, WButton *b)
 {
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h=b->widget.parent;
@@ -133,14 +147,16 @@ static int button_event (Gpm_Event *event, WButton *b)
     return MOU_NORMAL;
 }
 
-static void button_destroy (WButton *b)
+static void
+button_destroy (WButton *b)
 {
     x_destroy_cmd (b);
     free (b->text);
 }
 
-WButton *button_new (int y, int x, int action, char *text, int hkey, int hpos,
-		     int (*callback)(int, void *), void *callback_data)
+WButton *
+button_new (int y, int x, int action, char *text, int hkey, int hpos,
+	    int (*callback)(int, void *), void *callback_data, char *tkname)
 {
     WButton *b = xmalloc (sizeof (WButton), "new_button");
 
@@ -149,7 +165,7 @@ WButton *button_new (int y, int x, int action, char *text, int hkey, int hpos,
 
     init_widget (&b->widget, y, x, 1, strlen (text),
 		 (callback_fn) button_callback,
-		 (destroy_fn) button_destroy, (mouse_h)button_event);
+		 (destroy_fn) button_destroy, (mouse_h)button_event, tkname);
     b->action = action;
     b->selected = 0;
     b->text   = strdup (text);
@@ -162,7 +178,8 @@ WButton *button_new (int y, int x, int action, char *text, int hkey, int hpos,
     return b;
 }
 
-void button_set_text (WButton *b, char *text)
+void
+button_set_text (WButton *b, char *text)
 {
     free (b->text);
     b->text = strdup (text);
@@ -170,15 +187,16 @@ void button_set_text (WButton *b, char *text)
     x_button_set (b, text);
 #else
     button_callback (b->widget.parent, b, WIDGET_DRAW, 0);
-    refresh ();
 #endif
+    mc_refresh ();
 }
 
 
 /* Radio button widget */
 static int radio_event (Gpm_Event *event, WRadio *r);
 
-static int radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
+static int
+radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
 {
     int i;
     
@@ -288,7 +306,8 @@ static int radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
 #   define radio_destroy 0
 #endif
 
-static int radio_event (Gpm_Event *event, WRadio *r)
+static int
+radio_event (Gpm_Event *event, WRadio *r)
 {
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h = r->widget.parent;
@@ -305,7 +324,8 @@ static int radio_event (Gpm_Event *event, WRadio *r)
     return MOU_NORMAL;
 }
 
-WRadio *radio_new (int y, int x, int count, char **texts, int use_hotkey)
+WRadio *
+radio_new (int y, int x, int count, char **texts, int use_hotkey, char *tkname)
 {
     WRadio *r = xmalloc (sizeof (WRadio), "radio_new");
     int i, max, m;
@@ -319,7 +339,7 @@ WRadio *radio_new (int y, int x, int count, char **texts, int use_hotkey)
     }
 
     init_widget (&r->widget, y, x, count, max, (callback_fn) radio_callback,
-		 radio_destroy, (mouse_h) radio_event);
+		 radio_destroy, (mouse_h) radio_event, tkname);
     r->state = 1;
     r->pos = 0;
     r->sel = 0;
@@ -336,7 +356,8 @@ WRadio *radio_new (int y, int x, int count, char **texts, int use_hotkey)
 
 static int check_event (Gpm_Event *event, WCheck *b);
 
-static int check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
+static int
+check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
 {
     switch (Msg) {
     case WIDGET_INIT:
@@ -382,7 +403,8 @@ static int check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
     return default_proc (h, Msg, Par);
 }
 
-static int check_event (Gpm_Event *event, WCheck *c)
+static int
+check_event (Gpm_Event *event, WCheck *c)
 {
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h = c->widget.parent;
@@ -408,7 +430,8 @@ static int check_event (Gpm_Event *event, WCheck *c)
 #   define check_destroy 0
 #endif
 
-WCheck *check_new (int y, int x, int state, char *text, int hkey, int hpos)
+WCheck *
+check_new (int y, int x, int state, char *text, int hkey, int hpos, char *tkname)
 {
     WCheck *c =  xmalloc (sizeof (WRadio), "check_new");
 
@@ -417,7 +440,7 @@ WCheck *check_new (int y, int x, int state, char *text, int hkey, int hpos)
 
     init_widget (&c->widget, y, x, 1, strlen (text),
 		 (callback_fn)check_callback,
-		 check_destroy, (mouse_h) check_event);
+		 check_destroy, (mouse_h) check_event, tkname);
     c->state = state ? C_BOOL : 0;
     c->text = text;
     c->hotkey = hkey;
@@ -430,7 +453,8 @@ WCheck *check_new (int y, int x, int state, char *text, int hkey, int hpos)
 
 /* Label widget */
 
-static int label_callback (Dlg_head *h, WLabel *l, int Msg, int Par)
+static int
+label_callback (Dlg_head *h, WLabel *l, int Msg, int Par)
 {
     if (Msg == WIDGET_INIT)
 	return x_create_label (h, h->wdata, l);
@@ -471,7 +495,8 @@ static int label_callback (Dlg_head *h, WLabel *l, int Msg, int Par)
     return default_proc (h, Msg, Par);
 }
 
-void label_set_text (WLabel *label, char *text)
+void
+label_set_text (WLabel *label, char *text)
 {
     int newcols = label->widget.cols;
     
@@ -501,20 +526,22 @@ void label_set_text (WLabel *label, char *text)
         label->widget.cols = newcols;
 }
 
-static void label_destroy (WLabel *l)
+static void
+label_destroy (WLabel *l)
 {
     x_destroy_cmd (l);
     if (l->text)
 	free (l->text);
 }
 
-WLabel *label_new (int y, int x, char *text)
+WLabel *
+label_new (int y, int x, char *text, char *tkname)
 {
     WLabel *l = xmalloc (sizeof (WLabel), "label_new");
 
     init_widget (&l->widget, y, x, 1, 1,
 		 (callback_fn) label_callback,
-		 (destroy_fn) label_destroy, NULL);
+		 (destroy_fn) label_destroy, NULL, tkname);
     l->text = text ? strdup (text) : 0;
     l->auto_adjust_cols = 1;
     l->transparent = 0;
@@ -527,7 +554,8 @@ WLabel *label_new (int y, int x, char *text)
 /* Currently width is hardcoded here for text mode */
 #define gauge_len 47
 
-static int gauge_callback (Dlg_head *h, WGauge *g, int Msg, int Par)
+static int
+gauge_callback (Dlg_head *h, WGauge *g, int Msg, int Par)
 {
 
     if (Msg == WIDGET_INIT)
@@ -571,7 +599,8 @@ static int gauge_callback (Dlg_head *h, WGauge *g, int Msg, int Par)
     return default_proc (h, Msg, Par);
 }
 
-void gauge_set_value (WGauge *g, int max, int current)
+void
+gauge_set_value (WGauge *g, int max, int current)
 {
     if (g->current == current && g->max == max)
     	return; /* Do not flicker */
@@ -591,7 +620,8 @@ void gauge_set_value (WGauge *g, int max, int current)
 #endif
 }
 
-void gauge_show (WGauge *g, int shown)
+void
+gauge_show (WGauge *g, int shown)
 {
     if (g->shown == shown)
         return;
@@ -603,17 +633,20 @@ void gauge_show (WGauge *g, int shown)
 #endif    
 }
 
-static void gauge_destroy (WGauge *g)
+static void
+gauge_destroy (WGauge *g)
 {
+    /* nothing */
 }
 
-WGauge *gauge_new (int y, int x, int shown, int max, int current)
+WGauge *
+gauge_new (int y, int x, int shown, int max, int current, char *tkname)
 {
     WGauge *g = xmalloc (sizeof (WGauge), "gauge_new");
 
     init_widget (&g->widget, y, x, 1, gauge_len,
 		 (callback_fn) gauge_callback,
-		 (destroy_fn) gauge_destroy, NULL);
+		 (destroy_fn) gauge_destroy, NULL, tkname);
     g->shown = shown;
     if (max == 0)
         max = 1; /* I do not like division by zero :) */
@@ -631,7 +664,8 @@ WGauge *gauge_new (int y, int x, int shown, int max, int current)
 /* Pointer to killed data */
 static char *kill_buffer = 0;
 
-void update_input (WInput *in)
+void
+update_input (WInput *in)
 {
 #ifndef HAVE_XVIEW
     int    i, j;
@@ -676,14 +710,16 @@ void update_input (WInput *in)
 #endif
 }
 
-void winput_set_origin (WInput *in, int x, int field_len)
+void
+winput_set_origin (WInput *in, int x, int field_len)
 {
     in->widget.x    = x;
     in->field_len = in->widget.cols = field_len;
     update_input (in);
 }
 
-static void input_destroy (WInput *in)
+static void
+input_destroy (WInput *in)
 {
     if (!in){
 	fprintf (stderr, "Internal error: null Input *\n");
@@ -710,18 +746,21 @@ static void input_destroy (WInput *in)
 
 static char disable_update = 0;
 
-void input_disable_update (WInput *in)
+void
+input_disable_update (WInput *in)
 {
     in->disable_update++;
 }
 
-void input_enable_update (WInput *in)
+void
+input_enable_update (WInput *in)
 {
     in->disable_update--;
     update_input (in);
 }
 
-int push_history (WInput *in, char *text)
+int
+push_history (WInput *in, char *text)
 {
     Hist *new;
     char *p;
@@ -747,7 +786,8 @@ int push_history (WInput *in, char *text)
 }
 
 /* Cleans the input line and adds the current text to the history */
-void new_input (WInput *in)
+void
+new_input (WInput *in)
 {
     if (in->buffer)
 	push_history (in, in->buffer);
@@ -759,7 +799,8 @@ void new_input (WInput *in)
     update_input (in);
 }
 
-static int insert_char (WInput *in, int c_code)
+static int
+insert_char (WInput *in, int c_code)
 {
     int i;
 
@@ -781,7 +822,7 @@ static int insert_char (WInput *in, int c_code)
     }
     if (strlen (in->buffer)+1 < in->current_max_len){
 	int l = strlen (&in->buffer [in->point]);
-	for (i = l+2; i > 0; i--)
+	for (i = l+1; i > 0; i--)
 	    in->buffer [in->point+i] = in->buffer [in->point+i-1];
 	in->buffer [in->point] = c_code;
 	in->point++;
@@ -789,29 +830,34 @@ static int insert_char (WInput *in, int c_code)
     return 1;
 }
 
-static void beginning_of_line (WInput *in)
+static void
+beginning_of_line (WInput *in)
 {
     in->point = 0;
 }
 
-static void end_of_line (WInput *in)
+static void
+end_of_line (WInput *in)
 {
     in->point = strlen (in->buffer);
 }
 
-static void backward_char (WInput *in)
+static void
+backward_char (WInput *in)
 {
     if (in->point)
 	in->point--;
 }
 
-static void forward_char (WInput *in)
+static void
+forward_char (WInput *in)
 {
     if (in->buffer [in->point])
 	in->point++;
 }
 
-static void forward_word (WInput *in)
+static void
+forward_word (WInput *in)
 {
     char *p = in->buffer+in->point;
 
@@ -822,7 +868,8 @@ static void forward_word (WInput *in)
     in->point = p - in->buffer;
 }
 
-static void backward_word (WInput *in)
+static void
+backward_word (WInput *in)
 {
     char *p = in->buffer+in->point;
 
@@ -833,7 +880,8 @@ static void backward_word (WInput *in)
     in->point = p - in->buffer;
 }
 
-static void backward_delete (WInput *in)
+static void
+backward_delete (WInput *in)
 {
     int i;
     
@@ -845,7 +893,8 @@ static void backward_delete (WInput *in)
     in->point--;
 }
 
-static void delete_char (WInput *in)
+static void
+delete_char (WInput *in)
 {
     int i;
 
@@ -854,7 +903,8 @@ static void delete_char (WInput *in)
     in->need_push = 1;
 }
 
-static void copy_region (WInput *in, int x_first, int x_last)
+static void
+copy_region (WInput *in, int x_first, int x_last)
 {
     int first = min (x_first, x_last);
     int last  = max (x_first, x_last);
@@ -870,7 +920,8 @@ static void copy_region (WInput *in, int x_first, int x_last)
     kill_buffer [last-first] = 0;
 }
 
-static void delete_region (WInput *in, int x_first, int x_last)
+static void
+delete_region (WInput *in, int x_first, int x_last)
 {
    int first = min (x_first, x_last);
    int last  = max (x_first, x_last);
@@ -881,7 +932,8 @@ static void delete_region (WInput *in, int x_first, int x_last)
    in->need_push = 1;
 }
 
-static void kill_word (WInput *in)
+static void
+kill_word (WInput *in)
 {
     int old_point = in->point;
     int new_point;
@@ -895,7 +947,8 @@ static void kill_word (WInput *in)
     in->need_push = 1;
 }
 
-static void back_kill_word (WInput *in)
+static void
+back_kill_word (WInput *in)
 {
     int old_point = in->point;
     int new_point;
@@ -909,23 +962,27 @@ static void back_kill_word (WInput *in)
     in->need_push = 1;
 }
 
-static void set_mark (WInput *in)
+static void
+set_mark (WInput *in)
 {
     in->mark = in->point;
 }
 
-static void kill_save (WInput *in)
+static void
+kill_save (WInput *in)
 {
     copy_region (in, in->mark, in->point);
 }
 
-static void kill_region (WInput *in)
+static void
+kill_region (WInput *in)
 {
     kill_save (in);
     delete_region (in, in->point, in->mark);
 }
 
-static void yank (WInput *in)
+static void
+yank (WInput *in)
 {
     char *p;
     
@@ -935,7 +992,8 @@ static void yank (WInput *in)
 	insert_char (in, *p);
 }
 
-static void kill_line (WInput *in)
+static void
+kill_line (WInput *in)
 {
     if (kill_buffer)
 	free (kill_buffer);
@@ -943,9 +1001,11 @@ static void kill_line (WInput *in)
     in->buffer [in->point] = 0;
 }
 
-void assign_text (WInput *in, char *text)
+void
+assign_text (WInput *in, char *text)
 {
     free_completions (in);
+    free (in->buffer);
     in->buffer = strdup (text);	/* was in->buffer->text */
     in->current_max_len = strlen (in->buffer) + 1;
     in->point = strlen (in->buffer);
@@ -953,7 +1013,8 @@ void assign_text (WInput *in, char *text)
     in->need_push = 1;
 }
 
-static void hist_prev (WInput *in)
+static void
+hist_prev (WInput *in)
 {
     if (!in->history)
 	return;
@@ -972,7 +1033,8 @@ static void hist_prev (WInput *in)
     in->need_push = 0;
 }
 
-static void hist_next (WInput *in)
+static void
+hist_next (WInput *in)
 {
     if (in->need_push) {
         switch (push_history (in, in->buffer)) {
@@ -1046,7 +1108,8 @@ static struct {
 /* This function is a test for a special input key used in complete.c */
 /* Returns 0 if it is not a special key, 1 if it is a non-complete key
    and 2 if it is a complete key */
-int is_in_input_map (WInput *in, int c_code)
+int
+is_in_input_map (WInput *in, int c_code)
 {
     int i;
     
@@ -1059,7 +1122,8 @@ int is_in_input_map (WInput *in, int c_code)
     return 0;
 }
 
-int handle_char (WInput *in, int c_code)
+int
+handle_char (WInput *in, int c_code)
 {
     int    i;
     int    v;
@@ -1103,7 +1167,8 @@ int handle_char (WInput *in, int c_code)
 }
 
 /* Inserts text in input line */
-void stuff (WInput *in, char *text, int insert_extra_space)
+void
+stuff (WInput *in, char *text, int insert_extra_space)
 {
     input_disable_update (in);
     while (*text)
@@ -1114,7 +1179,8 @@ void stuff (WInput *in, char *text, int insert_extra_space)
     update_input (in);
 }
 
-void input_set_point (WInput *in, int pos)
+void
+input_set_point (WInput *in, int pos)
 {
     if (pos > in->current_max_len)
 	pos = in->current_max_len;
@@ -1126,7 +1192,8 @@ void input_set_point (WInput *in, int pos)
 
 int input_event (Gpm_Event *event, WInput *b);
 
-static int input_callback (Dlg_head *h, WInput *in, int Msg, int Par)
+static int
+input_callback (Dlg_head *h, WInput *in, int Msg, int Par)
 {
     int    t;
 
@@ -1176,7 +1243,8 @@ static int input_callback (Dlg_head *h, WInput *in, int Msg, int Par)
 
 /* Not declared static, since we check against this value in dlg.c */
 /* FIXME: Declare static again and provide an identification mechanism */
-int input_event (Gpm_Event *event, WInput *in)
+int
+input_event (Gpm_Event *event, WInput *in)
 {
     if (event->type & (GPM_DOWN|GPM_DRAG)){
 	dlg_select_widget (in->widget.parent, in);
@@ -1192,14 +1260,15 @@ int input_event (Gpm_Event *event, WInput *in)
     return MOU_NORMAL;
 }
 
-WInput *input_new (int y, int x, int color, int len, char *def_text)
+WInput *
+input_new (int y, int x, int color, int len, char *def_text, char *tkname)
 {
     WInput *in = xmalloc (sizeof (WInput), "input_new");
     int   initial_buffer_len;
 
     init_widget (&in->widget, y, x, 1, len,
 		 (callback_fn) input_callback,
-		 (destroy_fn) input_destroy, (mouse_h)input_event);
+		 (destroy_fn) input_destroy, (mouse_h)input_event, tkname);
 
 
     initial_buffer_len = 1 + max (len, strlen (def_text));
@@ -1234,7 +1303,8 @@ WInput *input_new (int y, int x, int color, int len, char *def_text)
  */
 static int listbox_cdiff (WLEntry *s, WLEntry *e);
 
-static void listbox_drawscroll (WListbox *l)
+static void
+listbox_drawscroll (WListbox *l)
 {
     extern int slow_terminal;
     int line;
@@ -1271,7 +1341,8 @@ static void listbox_drawscroll (WListbox *l)
     }
 }
     
-static void listbox_draw (WListbox *l, Dlg_head *h, int focused)
+static void
+listbox_draw (WListbox *l, Dlg_head *h, int focused)
 {
     WLEntry *e;
     int i;
@@ -1316,7 +1387,8 @@ static void listbox_draw (WListbox *l, Dlg_head *h, int focused)
 
 /* Returns the number of items between s and e,
    must be on the same linked list */
-static int listbox_cdiff (WLEntry *s, WLEntry *e)
+static int
+listbox_cdiff (WLEntry *s, WLEntry *e)
 {
     int count;
 
@@ -1325,7 +1397,8 @@ static int listbox_cdiff (WLEntry *s, WLEntry *e)
     return count;
 }
 
-static WLEntry *listbox_check_hotkey (WListbox *l, int key)
+static WLEntry *
+listbox_check_hotkey (WListbox *l, int key)
 {
     int i;
     WLEntry *e;
@@ -1350,7 +1423,8 @@ static WLEntry *listbox_check_hotkey (WListbox *l, int key)
 }
 
 /* Used only for display updating, for avoiding line at a time scroll */
-void listbox_select_last (WListbox *l, int set_top)
+void
+listbox_select_last (WListbox *l, int set_top)
 {
     if (l->list){
 	l->current = l->list->prev;
@@ -1361,43 +1435,85 @@ void listbox_select_last (WListbox *l, int set_top)
     }
 }
 
-void listbox_remove_current (WListbox *l)
+void
+listbox_remove_list (WListbox *l)
+{
+    WLEntry *p, *q;
+    int      i;
+
+    if (!l->count)
+	return;
+
+#ifdef HAVE_X
+    if (l->widget.wdata != (widget_data) NULL)
+	for (i = 0; i < l->count; i++)
+	    x_listbox_delete_nth (l, i);
+#endif
+    p = l->list;
+    
+    while (l->count--) {
+	q = p->next;
+	free (p->text);
+	free (p);
+	p = q;
+    }
+    l->pos = l->count = 0;
+    l->list = l->top = l->current = 0;
+}
+
+/*
+ * bor 30.10.96: added force flag to remove *last* entry as well
+ * bor 30.10.96: corrected selection bug if last entry was removed
+ */
+
+void
+listbox_remove_current (WListbox *l, int force)
 {
     WLEntry *p;
     
     /* Ok, note: this won't allow for emtpy lists */
-    if (!l->count || l->count == 1)
+    if (!force && (!l->count || l->count == 1))
 	return;
     
 #ifdef HAVE_X
     if (l->widget.wdata != (widget_data) NULL) {
         x_listbox_delete_nth (l, l->pos);
-        if (l->current->next != l->list)
-            x_listbox_select_nth (l, l->pos);
-        else if (l->current != l->list)
-            x_listbox_select_nth (l, 0);
+	if (l->count > 1)
+	    if (l->current->next != l->list)
+		x_listbox_select_nth (l, l->pos);
+	    else if (l->current != l->list)
+		x_listbox_select_nth (l, l->pos - 1);
+	else
+	    x_listbox_select_nth (l, 0);
     }
 #endif
     l->count--;
-    l->current->next->prev = l->current->prev;
-    l->current->prev->next = l->current->next;
     p = l->current;
-    if (p->next == l->list) {
-	l->current = p->prev;
-	l->pos--;
-    }	
-    else 
-	l->current = p->next;
-    
-    if (p == l->list)
-        l->list = l->top = p->next;
+
+    if (l->count) {
+	l->current->next->prev = l->current->prev;
+	l->current->prev->next = l->current->next;
+	if (p->next == l->list) {
+	    l->current = p->prev;
+	    l->pos--;
+	}	
+	else 
+	    l->current = p->next;
+	
+	if (p == l->list)
+	    l->list = l->top = p->next;
+    } else {
+	l->pos = 0;
+	l->list = l->top = l->current = 0;
+    }
 
     free (p->text);
     free (p);
 }
 
 /* Makes *e the selected entry (sets current and pos) */
-void listbox_select_entry (WListbox *l, WLEntry *dest)
+void
+listbox_select_entry (WListbox *l, WLEntry *dest)
 {
     WLEntry *e;
     int pos;
@@ -1431,7 +1547,8 @@ void listbox_select_entry (WListbox *l, WLEntry *dest)
 }
 
 /* Selects from base the pos element */
-static WLEntry *listbox_select_pos (WListbox *l, WLEntry *base, int pos)
+static WLEntry *
+listbox_select_pos (WListbox *l, WLEntry *base, int pos)
 {
     WLEntry *last = l->list->prev;
 
@@ -1445,7 +1562,8 @@ static WLEntry *listbox_select_pos (WListbox *l, WLEntry *base, int pos)
     return base;
 }
 
-static inline int listbox_back (WListbox *l)
+static inline int
+listbox_back (WListbox *l)
 {
     if (l->pos){
 	listbox_select_entry (l, listbox_select_pos (l, l->list, l->pos-1));
@@ -1454,7 +1572,8 @@ static inline int listbox_back (WListbox *l)
     return 0;
 }
 
-static inline int listbox_fwd (WListbox *l)
+static inline int
+listbox_fwd (WListbox *l)
 {
     if (l->current != l->list->prev){
 	listbox_select_entry (l, listbox_select_pos (l, l->list, l->pos+1));
@@ -1464,10 +1583,14 @@ static inline int listbox_fwd (WListbox *l)
 }
 
 /* Returns 1 if we want a redraw */
-static int listbox_key (WListbox *l, int key)
+static int
+listbox_key (WListbox *l, int key)
 {
     int i;
     int j = 0;
+
+    if (!l->list)
+	return 0;
     
     switch (key){
     case KEY_HOME:
@@ -1510,7 +1633,8 @@ static int listbox_key (WListbox *l, int key)
 }
 
 static int listbox_event (Gpm_Event *event, WListbox *l);
-static int listbox_callback (Dlg_head *h, WListbox *l, int msg, int par)
+static int
+listbox_callback (Dlg_head *h, WListbox *l, int msg, int par)
 {
     WLEntry  *e;
     /* int selected_color; Never used */
@@ -1558,7 +1682,8 @@ static int listbox_callback (Dlg_head *h, WListbox *l, int msg, int par)
     return default_proc (h, msg, par);
 }
 
-static int listbox_event (Gpm_Event *event, WListbox *l)
+static int
+listbox_event (Gpm_Event *event, WListbox *l)
 {
 #ifndef HAVE_X
     int i;
@@ -1584,7 +1709,7 @@ static int listbox_event (Gpm_Event *event, WListbox *l)
 	/* We need to refresh ourselves since the dialog manager doesn't */
 	/* know about this event */
 	listbox_callback (h, l, WIDGET_DRAW, 0);
-	refresh ();
+	mc_refresh ();
 	return MOU_REPEAT;
     }
 
@@ -1616,7 +1741,8 @@ static int listbox_event (Gpm_Event *event, WListbox *l)
     return MOU_NORMAL;
 }
 
-static void listbox_destroy (WListbox *l)
+static void
+listbox_destroy (WListbox *l)
 {
     WLEntry *n, *p = l->list;
     int i;
@@ -1630,15 +1756,16 @@ static void listbox_destroy (WListbox *l)
     }
 }
 
-WListbox *listbox_new (int y, int x, int width, int height,
-		       int action, lcback callback)
+WListbox *
+listbox_new (int y, int x, int width, int height,
+	     int action, lcback callback, char *tkname)
 {
     WListbox *l = xmalloc (sizeof (WListbox), "listbox_new");
     extern int slow_terminal;
     
     init_widget (&l->widget, y, x, height, width,
 		 (callback_fn)listbox_callback,
-		 (destroy_fn) listbox_destroy, (mouse_h)listbox_event);
+		 (destroy_fn) listbox_destroy, (mouse_h)listbox_event, tkname);
 
     l->list   = l->top = l->current = 0;
     l->pos    = 0;
@@ -1658,7 +1785,9 @@ WListbox *listbox_new (int y, int x, int width, int height,
 
 /* Listbox item adding function.  They still lack a lot of functionality */
 /* any takers? */
-static void listbox_append_item (WListbox *l, WLEntry *e)
+/* 1.11.96 bor: added pos argument to control placement of new entry */
+static void
+listbox_append_item (WListbox *l, WLEntry *e, enum append_pos pos)
 {
     if (!l->list){
 	l->list = e;
@@ -1666,17 +1795,32 @@ static void listbox_append_item (WListbox *l, WLEntry *e)
 	l->current = e;
 	e->next = l->list;
 	e->prev = l->list;
-    } else {
+    } else if (pos == LISTBOX_APPEND_AT_END) {
 	e->next = l->list;
 	e->prev = l->list->prev;
 	l->list->prev->next = e;
 	l->list->prev = e;
+    } else if (pos == LISTBOX_APPEND_BEFORE){
+	e->next = l->current;
+	e->prev = l->current->prev;
+	l->current->prev->next = e;
+	l->current->prev = e;
+	if (l->list == l->current) {	/* move list one position down */
+	    l->list = e;
+	    l->top =  e;
+	}
+    } else if (pos == LISTBOX_APPEND_AFTER) {
+	e->prev = l->current;
+	e->next = l->current->next;
+	l->current->next->prev = e;
+	l->current->next = e;
     }
     x_list_insert (l, l->list, e);
     l->count++;
 }
 
-char *listbox_add_item (WListbox *l, int pos, int hotkey, char *text,
+char *
+listbox_add_item (WListbox *l, enum append_pos pos, int hotkey, char *text,
 			void *data)
 {
     WLEntry *entry;
@@ -1693,18 +1837,20 @@ char *listbox_add_item (WListbox *l, int pos, int hotkey, char *text,
     entry->data = data;
     entry->hotkey = hotkey;
 
-    listbox_append_item (l, entry);
+    listbox_append_item (l, entry, pos);
     
     return entry->text;
 }
 
 /* Selects the nth entry in the listbox */
-void listbox_select_by_number (WListbox *l, int n)
+void
+listbox_select_by_number (WListbox *l, int n)
 {
     listbox_select_entry (l, listbox_select_pos (l, l->list, n));
 }
 
-WLEntry *listbox_search_text (WListbox *l, char *text)
+WLEntry *
+listbox_search_text (WListbox *l, char *text)
 {
     WLEntry *e;
 
@@ -1722,7 +1868,8 @@ WLEntry *listbox_search_text (WListbox *l, char *text)
 }
 
 /* Returns the current string text as well as the associated extra data */
-void listbox_get_current (WListbox *l, char **string, char **extra)
+void
+listbox_get_current (WListbox *l, char **string, char **extra)
 {
     if (!l->current){
 	*string = 0;
@@ -1734,7 +1881,8 @@ void listbox_get_current (WListbox *l, char **string, char **extra)
 	*extra = l->current->data;
 }
 
-int buttonbar_callback (Dlg_head *h, WButtonBar *bb, int msg, int par)
+int
+buttonbar_callback (Dlg_head *h, WButtonBar *bb, int msg, int par)
 {
     int i;
     
@@ -1777,7 +1925,8 @@ int buttonbar_callback (Dlg_head *h, WButtonBar *bb, int msg, int par)
     return default_proc (h, msg, par);
 }
 
-static void buttonbar_destroy (WButtonBar *bb)
+static void
+buttonbar_destroy (WButtonBar *bb)
 {
     int i;
 
@@ -1787,7 +1936,8 @@ static void buttonbar_destroy (WButtonBar *bb)
     }
 }
 
-static int buttonbar_event (Gpm_Event *event, WButtonBar *bb)
+static int
+buttonbar_event (Gpm_Event *event, WButtonBar *bb)
 {
 #ifndef HAVE_X
     int button;
@@ -1803,14 +1953,15 @@ static int buttonbar_event (Gpm_Event *event, WButtonBar *bb)
     return MOU_NORMAL;
 }
 
-WButtonBar *buttonbar_new (int visible)
+WButtonBar *
+buttonbar_new (int visible)
 {
     int i;
     WButtonBar *bb = xmalloc (sizeof (WButtonBar), "buttonbar_new");
 
     init_widget (&bb->widget, LINES-1, 0, 1, COLS,
 		 (callback_fn) buttonbar_callback,
-		 (destroy_fn) buttonbar_destroy, (mouse_h) buttonbar_event);
+		 (destroy_fn) buttonbar_destroy, (mouse_h) buttonbar_event, NULL);
     
     bb->visible = visible;
     for (i = 0; i < 10; i++){
@@ -1823,7 +1974,8 @@ WButtonBar *buttonbar_new (int visible)
     return bb;
 }
 
-void set_label_text (WButtonBar *bb, int index, char *text)
+void
+set_label_text (WButtonBar *bb, int index, char *text)
 {
     if (bb->labels [index-1].text)
 	free (bb->labels [index-1].text);
@@ -1832,7 +1984,8 @@ void set_label_text (WButtonBar *bb, int index, char *text)
 }
 
 /* paneletc is either the panel widget, or info or view or tree widget */
-WButtonBar *find_buttonbar (Dlg_head *h, Widget *paneletc)
+WButtonBar *
+find_buttonbar (Dlg_head *h, Widget *paneletc)
 {
     WButtonBar *bb;
     Widget_Item *item;
@@ -1856,8 +2009,10 @@ WButtonBar *find_buttonbar (Dlg_head *h, Widget *paneletc)
     }
     return bb;
 }
-void define_label_data (Dlg_head *h, Widget *paneletc, int idx, char *text,
-			buttonbarfn cback, void *data)
+
+void
+define_label_data (Dlg_head *h, Widget *paneletc, int idx, char *text,
+		   buttonbarfn cback, void *data)
 {
     WButtonBar *bb = find_buttonbar (h, paneletc);
     if (!bb)
@@ -1869,15 +2024,21 @@ void define_label_data (Dlg_head *h, Widget *paneletc, int idx, char *text,
     x_redefine_label (bb, idx);
 }
 
-void define_label (Dlg_head *h, Widget *paneletc, 
-    int idx, char *text, void (*cback)(void))
+void
+define_label (Dlg_head *h, Widget *paneletc, int idx, char *text, void (*cback)(void))
 {
     define_label_data (h, paneletc, idx, text, (void (*)(void *)) cback, 0);
 }
 
+#ifdef HAVE_X
 void redraw_labels (Dlg_head *h, Widget *paneletc)
 {
-#ifndef HAVE_X
+}
+
+#else
+void
+redraw_labels (Dlg_head *h, Widget *paneletc)
+{
     Widget_Item *item;
     int i;
 
@@ -1887,8 +2048,8 @@ void redraw_labels (Dlg_head *h, Widget *paneletc)
 	    return;
 	}
     }
-#endif
 }
+#endif
 
 
 
