@@ -6,9 +6,10 @@
 /* Widget based utility functions.
    Copyright (C) 1994, 1995 the Free Software Foundation
    
-   Authors: 1994, 1995 Miguel de Icaza
+   Authors: 1994, 1995, 1996 Miguel de Icaza
             1994, 1995 Radek Doulik
 	    1995  Jakub Jelinek
+	    1995  Andrej Borsenkow
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,17 +53,46 @@
 
 /* }}} */
 
-/* {{{ Listbox utility functions */
-
-void listbox_refresh (Dlg_head *h)
+/* {{{ Common dialog callback */
+#ifdef HAVE_X
+void dialog_repaint (struct Dlg_head *h, int back, int title_fore)
 {
-    /* Draw the screen */
-    attrset (COLOR_NORMAL);
+}
+#else
+void
+dialog_repaint (struct Dlg_head *h, int back, int title_fore)
+{
+    attrset (back);
     dlg_erase (h);
     draw_box (h, 1, 1, h->lines - 2, h->cols - 2);
-    attrset (COLOR_HOT_NORMAL);
-    dlg_move (h, 1, (h->cols-strlen (h->title))/2);
-    printw ("%s", h->title);
+    attrset (title_fore);
+    if (h->title){
+	dlg_move (h, 1, (h->cols-strlen (h->title))/2);
+	printw ("%s", h->title);
+    }
+}
+#endif
+void
+common_dialog_repaint (struct Dlg_head *h)
+{
+    dialog_repaint (h, REVERSE_COLOR, COLOR_HOT_NORMAL);
+}
+
+int
+common_dialog_callback (struct Dlg_head *h, int id, int msg)
+{
+    if (msg == DLG_DRAW)
+	common_dialog_repaint (h);
+    return 0;
+}
+
+/* }}} */
+/* {{{ Listbox utility functions */
+
+void
+listbox_refresh (Dlg_head *h)
+{
+    dialog_repaint (h, COLOR_NORMAL, COLOR_HOT_NORMAL);
 }
 
 static int listbox_callback (Dlg_head *h, int id, int msg)
@@ -92,13 +122,13 @@ Listbox *create_listbox_window (int cols, int lines, char *title, char *help)
 
     /* Create components */
     listbox->dlg = create_dlg (ypos, xpos, lines+6, cols+4, dialog_colors,
-			       listbox_callback, help, "listbox", DLG_CENTER);
+			       listbox_callback, help, "listbox", DLG_CENTER|DLG_GRID);
     x_set_dialog_title (listbox->dlg, title);	       
     
-    listbox->list = listbox_new (2, 2, cols, lines, listbox_finish, 0);
+    listbox->list = listbox_new (2, 2, cols, lines, listbox_finish, 0, "li");
 
     add_widget (listbox->dlg,
-	button_new (lines+3, (cols/2)-5, B_CANCEL, "[ Cancel ]",'c',2,0, 0));
+	button_new (lines+3, (cols/2)-5, B_CANCEL, "[ Cancel ]",'c',2,0, 0, "c"));
     add_widget (listbox->dlg, listbox->list);
 #ifndef HAVE_X
     listbox_refresh(listbox->dlg);
@@ -216,7 +246,7 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
     if (flags & WITH_HOTKEYS)
 	add_widget (query_dlg, button_new
 		    (lines-3, cols, B_USER+i, cur_name+1,
-		     cur_name [0], -1, 0, 0));
+		     cur_name [0], -1, 0, 0, NULL));
     else {
 	char *s = cur_name;
 
@@ -225,7 +255,7 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 
 	add_widget (query_dlg, button_new
 		    (lines-3, cols, B_USER+i, cur_name,
-		     *s, -1, 0, 0));
+		     *s, -1, 0, 0, NULL));
     }
 #else			 
 	    buttonnames [i] = cur_name;
@@ -241,16 +271,16 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	    if (flags & WITH_HOTKEYS)
 		add_widgetl (query_dlg, button_new
 			    (0, 0, B_USER+i, buttonnames [i] + 1,
-			     *(buttonnames [i]), -1, 0, 0),
+			     *(buttonnames [i]), -1, 0, 0, NULL),
 			     i ? XV_WLAY_RIGHTOF : XV_WLAY_CENTERROW);
 	    else
 		add_widgetl (query_dlg, button_new
 			    (0, 0, B_USER+i, buttonnames [i],
-			     *(buttonnames [i]), -1, 0, 0), 
+			     *(buttonnames [i]), -1, 0, 0, NULL), 
 			     i ? XV_WLAY_RIGHTOF : XV_WLAY_CENTERROW);
 	}  
 #endif	
-	add_widget (query_dlg, label_new (2, 3, text));
+	add_widget (query_dlg, label_new (2, 3, text, NULL));
 	
 	/* run dialog and make result */
 	run_dlg (query_dlg);
@@ -265,10 +295,10 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	destroy_dlg (query_dlg);
     } else {
 #ifdef HAVE_X
-	add_widgetl (query_dlg, button_new(0, 0, B_EXIT, "OK", 0, -1, 0, 0),
+	add_widgetl (query_dlg, button_new(0, 0, B_EXIT, "OK", 0, -1, 0, 0, NULL),
 	    XV_WLAY_CENTERROW);
 
-	add_widget (query_dlg, label_new (2, 3, text));
+	add_widget (query_dlg, label_new (2, 3, text, NULL));
 #ifdef HAVE_TK
 	if (flags & D_INSERT){
 	} else
@@ -278,8 +308,8 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	    destroy_dlg (query_dlg);
 	}
 #else
-	add_widget (query_dlg, label_new (2, 3, text));
-	add_widget (query_dlg, button_new(0, 0, 0, "", 0, -1, 0, 0));
+	add_widget (query_dlg, label_new (2, 3, text, NULL));
+	add_widget (query_dlg, button_new(0, 0, 0, "", 0, -1, 0, 0, NULL));
 #endif /* HAVE_X */
 	last_query_dlg = query_dlg;
     }
@@ -340,7 +370,7 @@ static int  remove_callback (int i, void *data)
 {
     Chooser *c = (Chooser *) data;
     
-    listbox_remove_current (c->listbox);
+    listbox_remove_current (c->listbox, 0);
 
     dlg_select_widget (c->dialog, c->listbox);
     dlg_select_nth_widget (c->dialog, 0);
@@ -349,28 +379,14 @@ static int  remove_callback (int i, void *data)
     return 0;
 }
 
-static int choose_callback (struct Dlg_head *h, int id, int Msg)
-{
-    switch (Msg){
-#ifndef HAVE_X    
-    case DLG_DRAW:
-	attrset (REVERSE_COLOR);
-	dlg_erase (h);
-	draw_box (h, 0, 0, h->lines, h->cols);
-	break;
-#endif	
-    }
-    return 0;
-}
-
 Chooser *new_chooser (int lines, int cols, char *help, int flags)
 {
     Chooser  *c;
     int      button_lines;
 
     c = (Chooser *) xmalloc (sizeof (Chooser), "new_chooser");
-    c->dialog = create_dlg (0, 0, lines, cols, dialog_colors, choose_callback,
-			    help, "chooser", DLG_CENTER);
+    c->dialog = create_dlg (0, 0, lines, cols, dialog_colors, common_dialog_callback,
+			    help, "chooser", DLG_CENTER | DLG_GRID);
     
     c->dialog->lines = lines;
     c->dialog->cols  = cols;
@@ -378,14 +394,14 @@ Chooser *new_chooser (int lines, int cols, char *help, int flags)
     button_lines = flags & CHOOSE_EDITABLE ? 3 : 0;
     
     c->listbox = listbox_new (1, 1, cols-2, lines-button_lines,
-			   listbox_finish, 0);
+			   listbox_finish, 0, "listbox");
     
     if (button_lines){
 	add_widget (c->dialog, button_new (lines-button_lines+1,
 				    20, B_ENTER, "[ Remove ]", 'r', 2,
-				    remove_callback, c));
+				    remove_callback, c, "button-remove"));
 	add_widget (c->dialog, button_new (lines-button_lines+1,
-				    4, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0));
+				    4, B_CANCEL, "[ Cancel ]", 'c', 2, 0, 0, "button-cancel"));
     }		    
     add_widget (c->dialog, c->listbox);
     return c;
@@ -444,11 +460,11 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
     
     if (qd->xpos == -1)
         dd = create_dlg (0, 0, qd->ylen, qd->xlen, dialog_colors, quick_callback,
-		         qd->help, qd->class, DLG_CENTER | DLG_TRYUP);
+		         qd->help, qd->class, DLG_CENTER | DLG_TRYUP | DLG_GRID);
     else
         dd = create_dlg (qd->ypos, qd->xpos, qd->ylen, qd->xlen, dialog_colors, 
                          quick_callback,
-		         qd->help, qd->class, 0);
+		         qd->help, qd->class, DLG_GRID);
 
     x_set_dialog_title (dd, qd->title);
     
@@ -463,32 +479,27 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
 	ypos = (qd->ylen * qw->relative_y)/qw->y_divisions;
 #endif
 	
-	if (!qw->tk_frame)
-	    tk_end_frame ();
-	else
-	    tk_new_frame (dd, qw->tk_frame);
-	
 	switch (qw->widget_type){
 	case quick_checkbox:
 	    widget = check_new (ypos, xpos, *qw->result, qw->text,
-				qw->hotkey, qw->hotkey_pos);
+				qw->hotkey, qw->hotkey_pos, qw->tkname);
 	    break;
 
 	case quick_radio:
-	    r = radio_new (ypos, xpos, qw->hotkey, qw->str_result, 1);
+	    r = radio_new (ypos, xpos, qw->hotkey, qw->str_result, 1, qw->tkname);
 	    r->pos = r->sel = qw->value;
 	    widget = r;
 	    break;
 	    
 	case quick_button:
 	    widget = button_new (ypos, xpos, qw->value, qw->text,
-				qw->hotkey, qw->hotkey_pos, 0, 0);
+				qw->hotkey, qw->hotkey_pos, 0, 0, qw->tkname);
 	    break;
 
 	    /* We use the hotkey pos as the field length */
 	case quick_input:
 	    input = input_new (ypos, xpos, INPUT_COLOR,
-			       qw->hotkey_pos, qw->text);
+			       qw->hotkey_pos, qw->text, qw->tkname);
 	    input->is_password = qw->value == 1;
 	    input->point = 0;
 	    if (qw->value & 2)
@@ -497,7 +508,7 @@ int quick_dialog_skip (QuickDialog *qd, int nskip)
 	    break;
 
 	case quick_label:
-	    widget = label_new (ypos, xpos, qw->text);
+	    widget = label_new (ypos, xpos, qw->text, qw->tkname);
 	    break;
 	    
 	default:
@@ -547,16 +558,16 @@ int quick_dialog (QuickDialog *qd)
 
 /* {{{ Input routines */
 #define INPUT_INDEX 2
-char *input_dialog_help (char *header, char *text, char *help, char *def_text)
+char *real_input_dialog_help (char *header, char *text, char *help, char *def_text)
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
     { quick_button, 6, 10, 1, 0, "[ Cancel ]", 'c', 2, B_CANCEL, 0, 0,
-	  XV_WLAY_DONTCARE, "b." },
+	  XV_WLAY_DONTCARE, "button-cancel" },
     { quick_button, 3, 10, 1, 0, "[ Ok ]", 'o', 2, B_ENTER, 0, 0,
-	  XV_WLAY_DONTCARE, "" },
-    { quick_input,  4, 80, 0, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, 0 },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, 0 },
+	  XV_WLAY_DONTCARE, "button-ok" },
+    { quick_input,  4, 80, 0, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input" },
+    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
     { 0 } };
     
     int len;
@@ -602,6 +613,63 @@ char *input_dialog (char *header, char *text, char *def_text)
     return input_dialog_help (header, text, "[Input Line Keys]", def_text);
 }
 
+int input_dialog_help_2 (char *header, char *text1, char *text2, char *help, char **r1, char **r2)
+{
+    QuickDialog Quick_input;
+    QuickWidget quick_widgets [] = {
+    { quick_button, 6, 10, 4, 0, "[ Cancel ]", 'c', 2, B_CANCEL, 0, 0,
+	  XV_WLAY_DONTCARE, "button-cancel" },
+    { quick_button, 3, 10, 4, 0, "[ Ok ]", 'o', 2, B_ENTER, 0, 0,
+	  XV_WLAY_DONTCARE, "button-ok" },
+    { quick_input,  4, 80, 4, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
+    { quick_label,  3, 80, 3, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
+    { quick_input,  4, 80, 3, 0, 0, 0, 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-lbl" },
+    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-lbl" },
+    { 0 } };
+    
+    int len;
+    int i;
+    int lines1, lines2;
+    char *my_str1, *my_str2;
+    
+    len = max (strlen (header), msglen (text1, &lines1));
+    len = max (len, msglen (text2, &lines2)) + 4;
+    len = max (len, 64);
+
+    Quick_input.xlen  = len;
+    Quick_input.xpos  = -1;
+    Quick_input.title = header;
+    Quick_input.help  = help;
+    Quick_input.class = "quick_input_2";
+    quick_widgets [5].text = text1;
+    quick_widgets [3].text = text2;
+
+    for (i = 0; i < 6; i++)
+	quick_widgets [i].y_divisions = lines1+lines2+7;
+    Quick_input.ylen  = lines1 + lines2 + 7;
+
+    quick_widgets [0].relative_y += (lines1 + lines2);
+    quick_widgets [1].relative_y += (lines1 + lines2);
+    quick_widgets [2].relative_y += (lines1);
+    quick_widgets [3].relative_y += (lines1);
+
+    quick_widgets [4].str_result = &my_str1;
+    quick_widgets [2].str_result = &my_str2;
+    
+    Quick_input.widgets = quick_widgets;
+    if (quick_dialog (&Quick_input) != B_CANCEL){
+	 *r1 = *(quick_widgets [4].str_result);
+	 *r2 = *(quick_widgets [2].str_result);
+	 return 1;
+    } else
+	 return 0;
+}
+
+int input_dialog_2 (char *header, char *text1, char *text2, char **r1, char **r2)
+{
+    return input_dialog_help_2 (header, text1, text2,  "[Input Line Keys]", r1, r2);
+}
+
 char *input_expand_dialog (char *header, char *text, char *def_text)
 {
     char *result;
