@@ -1,6 +1,12 @@
-# Midnight Commander Tk initialization.
-# Copyright (C) 1995 Miguel de Icaza
+# Midnight Commander Tk code.
+# Copyright (C) 1995, 1996, 1997 Miguel de Icaza
 #
+#
+#   Todo:
+#     Fix the internal viewer
+#     Missing commands: mc_file_info, mc_open_with
+#     The default buttons have a problem with the new frame around them: 
+#     they don't display the focus.
 #   
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -21,56 +27,24 @@
 #
 proc create_top_menu {} {
     global top_menus
+    global setup
     set top_menus ""
 
     frame .mbar -relief raised -bd 2
 
     pack .mbar -side top -fill x
-    menubutton .mbar.tk -text "Tk options" -underline 0 -menu .mbar.tk.menu
-    pack .mbar.tk -side right
-    menu .mbar.tk.menu 
- 
-    .mbar.tk.menu add command -label "Broken: Color model" -command color_model
-    .mbar.tk.menu add command -label "Broken: Font..." -command font_choose
 }
 
 #
-# This routine is completely broken
-#
-#
-proc setfont {scheme} {
-    set panelfont $scheme
-    if [string compare $scheme miguel] {
-	set panelfont lucidasanstypewriter-bold-14
-    }
-    .left.m.panel configure -font $panelfont
-    .right.m.panel configure -font $panelfont
-    .p.i0 configure -font $scheme
-    .p.l5 configure -font $scheme
-}
-
-#
-# This is, BTW, too.
-#
-#
-proc font_choose {} {
-    toplevel .font
-    button .font.1 -text "Fixed" -command {setfont fixed; destroy .font}
-    button .font.2 -text "Courier" -command {setfont courier; destroy .font}
-    button .font.3 -text "Miguel's" -command {setfont miguel; destroy .font}
-
-    pack .font.1 .font.2 .font.3 -side left -padx 4m -pady 4m
-}
-
-#
-# Widget: Menu
+# Widget: WMenu
 #
 proc create_menu {str topmenu} {
     global top_menus
+    global setup
 
     menubutton .mbar.$topmenu -text $str -underline 0 -menu .mbar.$topmenu.menu
     pack .mbar.$topmenu -side left
-    menu .mbar.$topmenu.menu
+    menu .mbar.$topmenu.menu -tearoff $setup(tearoff)
     set top_menus "$top_menus $topmenu"
 }
 
@@ -82,8 +56,18 @@ proc add_separator {topmenu} {
     .mbar.$topmenu.menu add separator
 }
 
-proc newbutton {name cmd text} {
+#
+# Widget: WButton
+#
+proc newbutton {name cmd text isdef} {
+    if $isdef {
+	frame $name -relief sunken -bd 1 
+	set name "$name.button"
+    }
     button $name -text "$text" -command $cmd -justify left
+    if $isdef {
+	pack $name -padx 1m -pady 1m
+    }
 }
 
 #
@@ -133,7 +117,7 @@ proc newview {is_panel container winname cmd} {
     frame $winname
     frame $winname.v
     frame $winname.v.status
-    eval text $winname.v.view -font $setup(panelfont) $setup(view_normal)
+    eval text $winname.v.view $setup(view_normal) -font $setup(panelfont) 
     
     # Create the tag names for the viewer attributes
     eval $winname.v.view tag configure bold $setup(view_bold)
@@ -247,7 +231,7 @@ proc bind_entry {win} {
 }
 
 proc newinput {name text} {
-    entry $name -relief sunken
+    entry $name -relief sunken -background white -foreground black
     $name insert 0 "$text"
     bindtags $name "all . $name "
     bind_entry $name
@@ -267,6 +251,15 @@ proc newcheck {name cmd text act} {
 # window is the container name and widget name: .left.o2
 # container is the container name: .left
 #
+    proc info_entry {win name text} {
+	frame $win.b.finfo.$name
+	label $win.b.finfo.$name.label -text "$text"
+	label $win.b.finfo.$name.info  
+	grid $win.b.finfo.$name.label -row 0 -column 0 -sticky we
+	grid $win.b.finfo.$name.info -row 0 -column 1 -sticky w
+	grid columnconfigure $win.b.finfo.$name 0 -minsize 100
+    }
+
 proc newinfo {container window version} {
     global setup
 
@@ -285,15 +278,16 @@ proc newinfo {container window version} {
 	-relief groove
     pack  $window.b.v.version -fill x
 
-    label $window.b.finfo.fname
-    label $window.b.finfo.location -text "test2"
-    label $window.b.finfo.mode
-    label $window.b.finfo.links
-    label $window.b.finfo.owner
-    label $window.b.finfo.size
-    label $window.b.finfo.created
-    label $window.b.finfo.modified
-    label $window.b.finfo.access
+    info_entry $window fname    "File:"
+    info_entry $window location "Location:"
+    info_entry $window mode     "Mode:"
+    info_entry $window links    "Links:"
+    info_entry $window owner    "Owner:"
+    info_entry $window size     "Size:"
+    info_entry $window created  "Created:"
+    info_entry $window modified "Modified:"
+    info_entry $window access   "Access:"
+
     pack  $window.b.finfo.fname \
 	$window.b.finfo.location $window.b.finfo.mode $window.b.finfo.links \
 	$window.b.finfo.owner $window.b.finfo.size $window.b.finfo.created \
@@ -360,6 +354,9 @@ proc infotext {win text} {
     $win configure -text $text
 }
 
+proc xinfotext {win text} {
+    $win.info configure -text $text
+}
     # w containes the window name *and* the .b frame (like: .left.o2.b)
     # FIXME: We should also display the rdev information
 proc info_update {w
@@ -377,19 +374,19 @@ proc info_update {w
     # 
     # Set all the text information
     # 
-    infotext $w.finfo.fname    "File:\t\t$fname"
-    infotext $w.finfo.location "Location:\t\t${dev}h:${ino}h"
-    infotext $w.finfo.mode     "Mode:\t\t$mode ($mode_oct)"
-    infotext $w.finfo.links    "Links:\t\t$links"
-    infotext $w.finfo.owner    "Owner:\t\t$owner/$group"
+    xinfotext $w.finfo.fname    "$fname"
+    xinfotext $w.finfo.location "${dev}h:${ino}h"
+    xinfotext $w.finfo.mode     "$mode ($mode_oct)"
+    xinfotext $w.finfo.links    "$links"
+    xinfotext $w.finfo.owner    "$owner/$group"
     if $have_blocks {
-	infotext $w.finfo.size     "Size:\t\t$size ($blocks blocks)"
+	xinfotext $w.finfo.size     "$size ($blocks blocks)"
     } else {
-	infotext $w.finfo.size     "Size:\t\t$size"
+	xinfotext $w.finfo.size     "$size"
     }
-    infotext $w.finfo.created  "Created:\t\t$create"
-    infotext $w.finfo.modified "Modified:\t\t$modify"
-    infotext $w.finfo.access   "Accessed:\t$access"
+    xinfotext $w.finfo.created  "$create"
+    xinfotext $w.finfo.modified "$modify"
+    xinfotext $w.finfo.access   "$access"
    
     infotext $w.fs.fsys "Filesystem:\t$fsys"
     infotext $w.fs.dev  "Device:\t\t$dev"
@@ -418,15 +415,16 @@ proc info_update {w
 proc listbox_sel {win item} {
     $win selection clear 0 end
     $win selection set $item
+    $win see $item
 }
 
 #
 # Widget: WPanel
 #
 proc panel_select {w pos cback} {
-    $w.m.panel tag add se $pos.0 "$pos.0 lineend"
-    $w.m.panel see $pos.0
-    $cback top [$w.m.panel index @0,0]
+    $w.m.p.panel tag add selected $pos.0 "$pos.0 lineend"
+    $w.m.p.panel see $pos.0
+    $cback top [$w.m.p.panel index @0,0]
 }
 
 proc panel_scroll {win cback args} {
@@ -434,31 +432,80 @@ proc panel_scroll {win cback args} {
     $cback top [$win index @0,0]
 }
 
-proc panel_setup {which} {
+proc cmd_sort_add {name menu cmd} {
+    $menu add command -label $name -command "$cmd sort $name"
+}
+
+proc panel_setup {which cmd} {
     global setup
 
     frame $which
+    set m $which.cwd.menu
+    menubutton $which.cwd -text "loading..." -bd 1 -relief raised \
+	-menu $m -indicatoron 0
+    menu $m -tearoff 0
+        menu [set mm $m.sort] -tearoff 0
+        $m add command -label "Reverse sort order" -command "$cmd reverse"
+        $m add cascade -label "Sort" -menu $mm
+            cmd_sort_add "Name"  $mm $cmd
+            cmd_sort_add "Extension" $mm $cmd
+            cmd_sort_add "Size"  $mm $cmd
+            cmd_sort_add "Modify Time" $mm $cmd
+            cmd_sort_add "Access Time" $mm $cmd
+            cmd_sort_add "Change Time" $mm $cmd
+            cmd_sort_add "Inode" $mm $cmd
+            cmd_sort_add "Type"  $mm $cmd
+            cmd_sort_add "Links" $mm $cmd
+            cmd_sort_add "NGID"  $mm $cmd
+            cmd_sort_add "NUID"  $mm $cmd
+            cmd_sort_add "Owner" $mm $cmd
+            cmd_sort_add "Group" $mm $cmd
+            cmd_sort_add "Unsorted" $mm $cmd
+
+	$m add command -label "Refresh" -command "$cmd refresh"
+	$m add separator
+    	$m add command -label "Set mask" -command "$cmd setmask"
+	$m add command -label "No mask"  -command "$cmd nomask"
 
-    label $which.cwd -text "wait..."
     frame $which.m
     label $which.mini
     scrollbar $which.m.scroll -width 3m
-    text  $which.m.panel -width $setup(cols) -yscroll "$which.m.scroll set" \
-	-font $setup(panelfont) -fore $setup(def_fore) -back $setup(def_back) \
-	-wrap none -height $setup(lines)
 
-    bindtags $which.m.panel "all . $which.m.panel"
-    
-    pack $which.m.panel -side left -fill both -expand 1
+    frame $which.m.p -relief sunken -borderwidth 2
+        # The sort bar
+        if $setup(with_sortbar) {
+	    canvas $which.m.p.types \
+		    -borderwidth 0\
+		    -back $setup(def_back) \
+		    -highlightthickness 0 -height 0
+	    pack $which.m.p.types  -side top -fill x
+	}
+
+        scrollbar $which.m.p.scroll -width 3m -orient horizontal
+
+        # The file listing panel
+        text  $which.m.p.panel -width $setup(cols) -yscroll "$which.m.scroll set" \
+	    -fore $setup(def_fore) -back $setup(def_back) \
+   	    -wrap none -height $setup(lines) -font $setup(panelfont)  \
+	    -relief flat -borderwidth 0 -highlightthickness 0 \
+	    -xscroll "$which.m.p.scroll set"
+
+        bindtags $which.m.p.panel "all . $which.m.p.panel"
+
+	proc x$which.m.p.panel {x} "$cmd \$x"
+	
+        pack $which.m.p.panel  -side top  -fill both -expand 1
+        pack $which.m.p.scroll -side top  -fill x 
+        pack $which.m.p        -side left -fill both -expand 1
+
     pack $which.m.scroll -side right -fill y
-
     pack $which.cwd  -side top -anchor w
     pack $which.m    -side top -fill both -expand 1
     pack $which.mini -side top -fill x
 
     pack $which -fill both -expand 1
-    # Tags: st (selected), sm (selected+marked), ma (marked), re (regular)
-    config_colors $which.m.panel
+
+    config_colors $which.m.p.panel
 }
 
     #
@@ -473,10 +520,6 @@ proc panel_cancel_repeat {} {
     set mc_repeat {}
 }
 
-    #
-    #  This routine passes the size of the text widget back to the C code
-    #
-
 proc panel_drag {w cmd n} {
     global mc_y
     global mc_x
@@ -494,13 +537,27 @@ proc panel_drag {w cmd n} {
     set mc_repeat [after 50 panel_drag $w $cmd $n]
 }
 
+    #
+    #  This routine passes the size of the text widget back to the C code
+    #
+
 proc panel_size {cmd panel w h} {
     global setup
 
-    set setup(height) $h
-    set setup(width)  $w
-    set setup(lines) [expr $h/$setup(heightc)]
-    set setup(cols)  [expr $w/$setup(widthc)]
+    set setup(real_height) $h
+    set setup(real_width)  $w
+
+    if $setup(with_icons) {
+	set setup(height) [expr $h-setup(iconheight)]
+	set setup(width)  [expr $w-$setup(iconwidth)]
+	set setup(lines)  [expr $setup(height)/$setup(heightc)]
+	set setup(cols)   [expr $setup(width)/$setup(widthc)]
+    } else {
+	set setup(height) $h
+	set setup(width)  $w
+	set setup(lines)  [expr $h/$setup(heightc)]
+	set setup(cols)   [expr $w/$setup(widthc)]
+    }
     $cmd resize $panel
 }
 
@@ -519,89 +576,216 @@ proc panel_conf {panel cmd} {
     bind $panel <Configure> "panel_size $cmd $panel %w %h"
 }
 
+#
+# Manage the bar that keeps the sort orders 
+#
+proc panel_reset_sort_labels {win} {
+#    $win.m.p.types delete all
+}
+
+proc panel_sort_label_start {win} {
+#    $win.m.p.types delete all
+}
+
+#
+# This right now uses a canvas, creates labels and places them 
+# on the canvas.  The button emulation is done with a manual 
+# bind.
+#
+
+proc panel_add_sort {win text_len text pos end_pos tag} {
+    global setup
+
+    if !$setup(with_sortbar) return
+
+    catch { destroy $win.m.p.types.$pos }
+    label $win.m.p.types.$pos -text $text -borderwidth 2 \
+	    -font $setup(panelfont) -relief raised
+
+    $win.m.p.types create window $pos 0 -window $win.m.p.types.$pos -anchor nw
+    $win.m.p.types configure -height [expr $setup(heightc)+8]
+    $win.m.p.types create line $end_pos 1 $end_pos [expr $setup(heightc)-1] -fill gray
+
+    # Simulate the button.
+    bind $win.m.p.types.$pos <Button-1> "
+        $win.m.p.types.$pos configure -relief sunken
+        x$win sort $tag
+        after 100 { $win.m.p.types.$pos configure -relief raised }
+     "
+}
+
+#
+# Called back from the action menu
+#
+proc popup_add_action {filename cmd idx} {
+    global setup
+    menu [set m .m.action] -tearoff 0
+        .m add cascade -foreground $setup(action_foreground) -label "$filename" -menu $m
+        $m add command -label "Info"            -command "mc_file_info \"$filename\""
+        $m add command -label "Open with..."    -command "mc_open_with \"$filename\""
+	$m add separator
+	$m add command -label "Copy..."         -command "$cmd invoke %d Copy"
+	$m add command -label "Rename, move..." -command "$cmd invoke %d Move"
+	$m add command -label "Delete..."       -command "$cmd invoke %d Delete"
+	$m add separator 
+	$m add command -label "Open"            -command "$cmd invoke %d Open"
+	$m add command -label "View"            -command "$cmd invoke %d View"
+}
+
+proc start_drag {mode panel_cmd W x y X Y} {
+    global drag_mode
+    global drag_text
+
+    set drag_mode $mode
+    set drag_text [$panel_cmd dragtext [$W index @$x,$y]]
+    set drag_text "$mode $drag_text"
+
+    catch {destroy .drag}
+    toplevel .drag
+    wm overrideredirect .drag 1
+    wm withdraw .drag
+    label .drag.text -text "$drag_text"
+    pack .drag.text
+    wm deiconify .drag
+    wm geometry .drag +$X+$Y
+}
+
+proc drag_test {token} {
+    if {[winfo children $token] == ""} {
+	label $token.value -text "Zonzo"
+	pack $token.value
+	puts "Configurado!"
+    }
+    $token.value configure -text Hla
+    return "caca";
+}
+
+proc mc_drag_target {} {
+    global DragDrop
+
+    set data $DragDrop(text)
+    puts "Recibimos esto: $data"
+}
+
+proc mc_drag_send {} {
+    puts "drag send"}
+
     #
     # Mouse bindings for the panels
     #
 proc panel_bind {the_panel panel_cmd} {
     global setup
 
-    bind $the_panel.m.panel <Button-1> "
-        $panel_cmd mdown 3  \[%W index @%x,%y]
+    set pn "$the_panel.m.p.panel"
+
+    bind $pn <Button-1> "
+        $panel_cmd mdown 2  \[%W index @%x,%y]
     "
  
-    bind $the_panel.m.panel <ButtonRelease-1> "
+    bind $pn <ButtonRelease-1> "
         panel_cancel_repeat
-        $panel_cmd mup 3    \[%W index @%x,%y]"
+        $panel_cmd mup 2    \[%W index @%x,%y]"
 
-    bind $the_panel.m.panel <Double-1> "
+    bind $pn <Double-1> "
         panel_cancel_repeat
-        $panel_cmd double 3 \[%W index @%x,%y]"
+        $panel_cmd double 2 \[%W index @%x,%y]"
 
-    bind $the_panel.m.panel <B1-Motion> "
+    bind $pn <B1-Motion> "
        set mc_x %x
        set mc_y %y
-       $panel_cmd motion 3 \[%W index @%x,%y]
+       $panel_cmd motion 2 \[%W index @%x,%y]
     "
 
-    bind $the_panel.m.panel <B1-Leave> "
+    bind $pn <B1-Leave> "
         set mc_x %x
         set mc_y %y
-	panel_drag %W $panel_cmd 3
+	panel_drag %W $panel_cmd 2
     "
 
-    bind $the_panel.m.panel <B1-Enter> panel_cancel_repeat
+    bind $pn <B1-Enter> panel_cancel_repeat
 
-    bind $the_panel.m.panel <Button-3> "
-        $panel_cmd mdown 1  \[%W index @%x,%y]"
+    if $setup(b2_marks) {
+	bind $pn <Button-2> "
+            $panel_cmd mdown 1  \[%W index @%x,%y]"
 
-    bind $the_panel.m.panel <ButtonRelease-3> "
-        $panel_cmd mup 1    \[%W index @%x,%y]
-        panel_cancel_repeat
+	bind $pn <ButtonRelease-2> "
+	    $panel_cmd mup 1    \[%W index @%x,%y]
+	    panel_cancel_repeat
+	"
+
+	bind $pn <B2-Motion> "
+	    set mc_x %x
+	    set mc_y %y
+	    $panel_cmd motion 1 \[%W index @%x,%y]
+	"
+
+	bind $pn <B2-Leave> "
+	    set mc_x %x
+	    set mc_y %y
+	    panel_drag %W $panel_cmd 1
+	"
+    } else {
+	# We have blt
+#	blt_drag&drop source $pn config -button 2 -packagecmd drag_test \
+#		-selftarget 1
+#	blt_drag&drop source $pn handler text dd_send_file
+    }
+
+    # Menu popup.
+    bind $pn <Button-3> "
+	$panel_cmd mdown 2 \[%W index @%x,%y]
+	catch {destroy .m}
+	menu .m -tearoff 0
+	$panel_cmd load \[%W index @%x,%y] %X %Y
+	tk_popup .m %X %Y 
     "
 
-    bind $the_panel.m.panel <B3-Motion> "
-        set mc_x %x
-        set mc_y %y
-        $panel_cmd motion 1 \[%W index @%x,%y]
-    "
+    bind $pn <B2-Enter> panel_cancel_repeat
 
-    bind $the_panel.m.panel <B3-Leave> "
-        set mc_x %x
-        set mc_y %y
-	panel_drag %W $panel_cmd 1
-    "
-    
-    bind $the_panel.m.panel <B3-Enter> panel_cancel_repeat
     $the_panel.m.scroll configure \
-        -command "panel_scroll $the_panel.m.panel $panel_cmd"
-
-    panel_conf $the_panel.m.panel $panel_cmd
+        -command "panel_scroll $pn $panel_cmd"
+    $the_panel.m.p.scroll configure \
+	-command "$pn xview"
+    panel_conf $pn $panel_cmd
 }
 
-proc panel_width {} {
+proc panel_info {item} {
     global setup
-    return $setup(cols)
-}
 
-proc panel_height {} {
-    global setup
-    return $setup(lines)
+    return $setup($item)
 }
 
 proc panel_mark {tag panel n} {
     config_colors $panel
-    puts stderr "TAG: $tag, LINEA=$n\n\r"
     $panel tag add $tag "${n}.0" "${n}.0 lineend"
 }
 
+# op is add or remove
+proc panel_mark_entry {win op line} {
+    $win.m.p.panel tag $op marked $line.0 "$line.0 lineend"
+}
+
+proc panel_unmark_entry {win line} {
+    $win.m.p.panel tag remove selected $line.0 "$line.0 lineend"
+}
+
 #
 # Misc routines
 #
+
+# Configure the panel tags
 proc config_colors {which} {
     global setup
 
-    $which tag configure se -back $setup(selected) -fore $setup(selected_fg)
-    $which tag configure ma -fore $setup(marked)
+    # se -- selected file
+    $which tag configure selected -back $setup(panelcolor,selected_back)
+
+    foreach v {marked directory executable regular selected} {
+	$which tag configure $v -fore $setup(panelcolor,$v)
+    }
+    $which tag configure directory -font $setup(paneldir)
+
+    $which tag raise marked
 }
 
 proc tclerror {msg} {
@@ -630,7 +814,7 @@ proc layout_midnight {} {
     global one_window
     global wlist
 
-    puts $wlist
+    #puts $wlist
     #
     # we want to make the prompt and the input line sunken
     # so we sunk the frame, and set a borderwidth for it
@@ -641,19 +825,21 @@ proc layout_midnight {} {
     pack .p.i0 -side left -expand 1 -fill x -anchor e
     pack .n4 -side bottom -fill x
     pack .p -side bottom -fill x
+
     if $one_window {
 	pack .left -side top -side left -fill both -expand 1
 	pack .right -side top -side right -fill both -expand 1
     }
 }
 
-proc layout_query_xxx {} {
+proc layout_query {} {
     global wlist
 
+    puts "$wlist"
     set t [llength $wlist]
     if {$t == 2} {
-	pack .query.l0 -side top -pady 2m -padx 2m
-	pack .query.b1 -side right  -ipadx 2m -padx 4m -pady 2m -expand 1
+	pack .query.l1 -side top -pady 2m -padx 2m
+	pack .query.b0 -side right  -ipadx 2m -padx 4m -pady 2m -expand 1
     } else {
 	pack .query.l1  -side top -pady 2m -padx 2m
 
@@ -666,73 +852,6 @@ proc layout_query_xxx {} {
     }
 }
 
-proc layout_display {} {
-    pack .display.r.r1 -side left
-    pack .display.r.i2 -side right -fill x -expand 1 -anchor s
-
-    pack .display.m.c3 -side top -anchor w
-    pack .display.m.i4 -side top -fill x -expand 1
-
-    pack .display.b.b5 -side left -padx 2m
-    pack .display.b.b0 -side left -padx 2m
-
-    pack .display.r -side top -fill x -expand 1 -padx 3m -pady 8m
-    pack .display.m -side top -fill x -expand 1 -padx 3m
-    pack .display.b -side top -expand 1 -padx 3m -pady 8m
-}
-
-proc layout_sort_xxx {} {
-    pack .sort.r.r1 -side left -padx 4m
-    pack .sort.r.c2 -side right -padx 4m -pady 4m -anchor n
-    pack .sort.b.b3 -side left -padx 4m
-    pack .sort.b.b0 -side left -padx 4m
-
-    pack .sort.r -side top -pady 8m
-    pack .sort.b -pady 4m -side top
-}
-
-proc layout_findfile {} {
-    label .findfile.l
-    pack .findfile.l -side top
-    pack .findfile.l1 -side top -anchor w -padx 4m
-    pack .findfile.i0 -side top -fill x -expand 1 -anchor w -padx 4m
-    pack .findfile.l2 -side top -anchor w -padx 4m
-    pack .findfile.i3 -side top -fill x -expand 1 -anchor w -padx 4m
-    pack .findfile.b.b4 .findfile.b.b5 .findfile.b.b6 -side left \
-	-expand 1 -padx 4m
-    pack .findfile.b -pady 4m -padx 4m
-}
-
-proc layout_option {} {
-    global wlist
-    puts stderr "$wlist\n\r" 
-    # group name: buttons
-    pack .option.b.b22 .option.b.b23 .option.b.b0 -side left -padx 4m -pady 4m
-    pack .option.b -side bottom  
-    
-    # group name: panel options
-    pack .option.c.l1 -side top -anchor w
-    pack .option.c.c11 .option.c.c12 .option.c.c13 .option.c.c14 \
-	.option.c.c15 .option.c.c16 .option.c.c17 .option.c.c18  \
-	.option.c.c19 .option.c.c20 .option.c.c21 -side top -anchor w
-    .option.c configure -relief sunken 
-    pack .option.c -padx 4m -pady 4m -side right -anchor n -fill y 
-
-    # group name: other options
-    pack .option.o.l10 -side top -anchor w
-    pack .option.o.c2 .option.o.c3 .option.o.c4 .option.o.c5 \
-	.option.o.c6 .option.o.c7 -side top -anchor w
-    .option.o configure -relief sunken 
-    pack .option.o -side top -padx 4m -pady 4m -side top
-
-    # group name: pause after run...
-    pack .option.p.l8 -side top -anchor w
-    pack .option.p.r9 -side top -anchor w
-    .option.p configure -relief sunken
-    pack .option.p -padx 4m -pady 4m -side top
-
-}
-
 proc layout_listbox {} {
     scrollbar .listbox.s -width 3m -command {.listbox.x0 yview}
     .listbox.x0 configure -yscroll {.listbox.s set} 
@@ -740,19 +859,6 @@ proc layout_listbox {} {
     pack .listbox.x0 -expand 1 -fill both -padx 4m -pady 4m -side left
 }
 
-proc layout_mfind {} {
-    scrollbar .mfind.s -width 3m -command {.mfind.x1 yview}
-    .mfind.x1 configure  -yscroll {.mfind.s set} 
-
-    pack .mfind.b.b0 .mfind.b.b3 .mfind.b.b4 .mfind.b.b5 .mfind.b.b6 \
-	-side left -padx 4m -expand 1 -fill x -pady 2m
-
-    pack .mfind.b -fill x -expand 1 -side bottom
-    pack .mfind.l2 -anchor w -expand 1 -anchor w -padx 4m -side bottom
-    pack .mfind.x1 -side left -fill both -expand 1 -pady 2m
-    pack .mfind.s -side right -fill y -expand 1
-}
-
 proc layout_quick_confirm {} {
     pack .quick_confirm.c.c1 .quick_confirm.c.c2 .quick_confirm.c.c3 \
 	-side top -anchor w
@@ -1004,6 +1110,28 @@ proc layout_quick_input {} {
     pack .quick_input.i1 -side bottom -fill x -padx 4m
 }
 
+proc create_drop_target {name icon} {
+    toplevel .drop-$name
+
+    button .drop-$name.b -text "Drop target"
+    pack .drop-$name.b
+    wm group .drop-$name .
+    wm overrideredirect .drop-$name 1
+    wm withdraw .drop-$name
+    wm deiconify .drop-$name
+    wm geometry .drop-$name +0+0
+#    blt_drag&drop target .drop-$name.b handler file mc_drag_target
+#    blt_drag&drop target .drop-$name.b handler text mc_drag_target
+}
+
+#
+# Creates the container
+#
+proc create_container {container} {
+    canvas $container 
+    pack $container -fill both -expand 1
+}
+
 #
 # Removes all of the widgets in a container (.left or .right)
 #
@@ -1021,15 +1149,25 @@ proc container_clean {container} {
 proc bind_setup {win} {
     flush stderr
     bindtags $win {all $win}
+    wm protocol $win WM_DELETE_WINDOW "tkmc e scape"
     bind $win <Leave> 
-    bind $win <Key>              "tkmc r %A"
-    bind $win <Alt-KeyPress>     "tkmc a %A"
-    bind $win <Meta-KeyPress>    "tkmc a %A"
-    bind $win <Control-KeyPress> "tkmc c %A"
+}
+
+proc keyboard_bindings {} {
+    # Remove the Tab binding.
+    bind all <Tab> {}
+
+    bind all <KeyPress>          "tkmc r %A"
+
+    # Remove the Alt-key binding and put a sensible one instead
+    bind all <Alt-KeyPress>      "tkmc a %A"
+    bind all <Control-KeyPress> "tkmc c %A"
+    bind all <Meta-KeyPress>    "tkmc a %A"
+
     foreach i {Left Right Up Down End R13 Home F27 F29 Prior \
 	       Next F35 Return KP_Enter Delete Insert BackSpace \
 	       F1 F2 F3 F4 F5 F6 F7 F8 F9 F10} {
-        bind $win <Key-$i> "tkmc k %K"
+        bind all <Key-$i> "tkmc k %K"
     }
 }
 
@@ -1082,21 +1220,40 @@ proc color_model {} {
 # gray85 is the background for the new tk4
 proc gray_colors {base} {
     global setup
+    global have_blt
 
-    set dark_color [tkDarken $base 90]
+#
 #    set setup(def_back) [tkDarken $base 90]
 #    set setup(def_fore) black
 #    set setup(selected) [tkDarken $base 110]
 #    set setup(marked)   SlateBlue
 #    set setup(high)     $setup(def_back)
 
-    set setup(def_back) [tkDarken $base 110]
+    if {0} {
+	set dark_color [tkDarken $base 90]
+	set setup(def_back) [tkDarken $base 110]
+	set setup(selected) NavyBlue
+	set setup(selected_fg) white
+    } else {
+	set dark_color $base
+	set setup(def_back) #d9d9d9
+	set setup(selected) white
+	set setup(selected_fg) black
+    }
     set setup(def_fore) black
-    set setup(selected) NavyBlue
-    set setup(selected_fg) white
-    set setup(marked)   yellow
     set setup(high)     yellow
-   
+
+#
+# Panel colors:
+#
+    # Marked files
+    set setup(panelcolor,marked)        yellow
+    set setup(panelcolor,directory)     blue
+    set setup(panelcolor,executable)    red
+    set setup(panelcolor,regular)       black
+    set setup(panelcolor,selected_back) white
+    set setup(panelcolor,selected)      black
+
 # Viewer colors
 
     set setup(view_bold)      "-fore yellow -back $dark_color"
@@ -1107,9 +1264,24 @@ proc gray_colors {base} {
 # The percentage bars on info:
     set setup(percolor) "blue"
 
+# The sort bar colors
+    set setup(sort_fg) $setup(def_fore)
+    set setup(sort_bg) $setup(def_back)
+    set setup(with_sortbar) 1
+
+    # We use BLT only for drag and drop, if this is not available, 
+    # then we use the 2nd button for regular file marking.
+    set have_blt 0
+    if $have_blt {
+	set setup(b2_marks) 0
+    } else {
+	set setup(b2_marks) 1
+    }
+
 # The errors
     set setup(errorfore) white
     set setup(errorback) red
+
 }
 
 proc bisque_colors {} {
@@ -1141,22 +1313,16 @@ proc sanity_check {} {
 # bind Entry <Enter> {}
 # bind Entry <FocusIn> {}
 
-# Remove the Tab binding.
-bind all <Tab> {}
+keyboard_bindings 
 
+set setup(tearoff) 0
+set setup(action_foreground) blue
 set setup(lines) 24
 set setup(cols)  40
-
-#
-# Debugging routines
-toplevel .command
-entry .command.i
-pack .command.i -side left
-button .command.ok -command {eval [.command.i get]} -text "Eval"
-pack .command.ok
-button .command.quit -command {exit} -text "Aieee!"
-pack .command.quit
-bind .command.quit <Key-Return> {.command.ok invoke}
+set setup(with_icons) 0
+set setup(widthc) 0
+set setup(heightc) 0
+set setup(real_width) 0
 
 # Determine Tk version
 set beta_4 ![catch tk_bisque]
@@ -1172,20 +1338,43 @@ set mc_repeat {}
 set mc_x 0
 set mc_y 0
 set center_toplevels 1
+#set center_toplevels 0
 
-set setup(panelfont) lucidasanstypewriter-bold-14
-set setup(font) "-*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*"
+# button .testbutton [lindex [.testbutton configure -font] 3]
+#set setup(panelfont) lucidasanstypewriter-bold-14
+set setup(panelfont) lucidasanstypewriter-14
+set setup(paneldir)  lucidasanstypewriter-bold-14
+set setup(font)  "-*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*"
+
+#
+# 
+# This variable if set, will make the program load gui.*.tcl files
+# instead of the gui.tcl file created during instalaltion.
+set use_separate_gui_files 0
 
 if [file exist ~/.tkmc] {source ~/.tkmc}
+
 catch {
-    option add *font $setup(font) userDefault
-    option add *Menu*activeBackground NavyBlue
-    option add *Menu*activeForeground white
-    option add *Menubutton*activeBackground NavyBlue
-    option add *Menubutton*activeForeground white
-    option add *Button*activeBackground NavyBlue
-    option add *Button*activeForeground white
-#    set setup(panelfont) $setup(font)
+#    option add *font $setup(font) userDefault
+#    option add *Menu*activeBackground NavyBlue
+#    option add *Menu*activeForeground white
+#    option add *Menubutton*activeBackground NavyBlue
+#    option add *Menubutton*activeForeground white
+#    option add *Button*activeBackground NavyBlue
+#    option add *Button*activeForeground white
+##    set setup(panelfont) $setup(font)
 }
 
+proc run_gui_design {root} {
+    global components 
+
+    create_workspace $root
+    gui_design $root $components
+}
+
+source $LIBDIR/gd.tcl
+
+set tk_strictMotif 1
 create_top_menu
+
+create_drop_target Hola Zonzo
