@@ -38,10 +38,6 @@
 #include "vfs.h"
 #include "extfs.h"
 
-/* To not include all dialoging stuff, I'm not including dialog.h
-   just for this function */
-extern void *message (int error, char *header, char *text,...);
-
 static struct extfs_archive *first_archive = NULL;
 static int extfserrno = 0;
 static struct stat hstat;		/* Stat struct corresponding */
@@ -53,7 +49,13 @@ static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name
 static char *extfs_prefixes [MAXEXTFS];
 static char extfs_need_archive [MAXEXTFS];
 static int extfs_no = 0;
-static char *extfs_extensions [MAXEXTFS];
+
+struct extfs_ext {
+    char *ext;
+    struct extfs_ext *next;
+};
+
+static struct extfs_ext *extfs_extensions [MAXEXTFS];
 
 void extfs_fill_names (void (*func)(char *))
 {
@@ -61,7 +63,8 @@ void extfs_fill_names (void (*func)(char *))
     char *name;
     
     while (a){
-	name = copy_strings (extfs_prefixes [a->fstype], ":", a->name, "/",
+	name = copy_strings (extfs_prefixes [a->fstype], ":", 
+			    (a->name ? a->name : ""), "/",
 			     a->current_dir->name, 0);
 	(*func)(name);
 	free (name);
@@ -164,7 +167,8 @@ static void free_archive (struct extfs_archive *archive)
             archive->local_stat.st_mtime != my.st_mtime);
         /* ungetlocalcopy frees local_name for us */
     }
-    free (archive->name);
+    if (archive->name)
+	free (archive->name);
     free (archive);
 }
 
@@ -247,7 +251,7 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
     struct extfs_archive *current_archive;
 
     if ((extfsd = open_extfs_archive (fstype, name, &current_archive)) == NULL) {
-        message (1, " Error ", "Couldn't open %s archive\n%s", 
+        message_3s (1, " Error ", "Couldn't open %s archive\n%s", 
             extfs_prefixes [fstype], name);
 	return -1;
     }
@@ -279,8 +283,9 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
                     goto read_extfs_continue;
                 pent = extfs_find_entry (current_archive->root_entry, q, 1);
                 if (pent == NULL) {
-                    message (1, " Error ", "Inconsistent extfs archive");
+                    message_1s (1, " Error ", "Inconsistent extfs archive");
                     /* FIXME: Should clean everything one day */
+                    free (buffer);
                     return -1;
                 }
 		entry = (struct extfs_entry *) xmalloc (sizeof (struct extfs_entry), "Extfs: extfs_entry");
@@ -295,8 +300,9 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
 		if (!S_ISLNK (hstat.st_mode) && current_link_name != NULL) {
 	            pent = extfs_find_entry (current_archive->root_entry, current_link_name, 0);
 	            if (pent == NULL) {
-	                message (1, " Error ", "Inconsistent extfs archive");
+	                message_1s (1, " Error ", "Inconsistent extfs archive");
                     /* FIXME: Should clean everything one day */
+			free (buffer);
 	                return -1;
 	            } else {
 			entry->inode = pent->inode;
@@ -340,15 +346,16 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
     }
     pclose (extfsd);
     *pparc = current_archive;
+    free (buffer);
     return 0;
 }
 
 char *extfs_analysis (char *name, char **archive, int *fstype, int is_dir)
 {
-    char *p, *local, *q;
+    char *p, *local;
     int j;
     char *archive_name = NULL;
-
+    struct extfs_ext *ext;
                                                /*  |  this is len of prefix plus some minimum
                                                 *  v  space needed for the extension */
     *fstype = extfs_prefix_to_type (name);
@@ -364,11 +371,11 @@ char *extfs_analysis (char *name, char **archive, int *fstype, int is_dir)
 	
     for (p = name + strlen (name); p > name; p--)
     	if (*p == '/' || (is_dir && !*p))
-    	    for (q = extfs_extensions [*fstype]; *q; q += j + 1) {
-    	        j = strlen (q);
+    	    for (ext = extfs_extensions [*fstype]; ext; ext = ext->next) {
+    	        j = strlen (ext->ext);
     	        if (p - j < name + 1)
     	            continue;
-    	        if (!strncmp (p - j, q, j)) {
+    	        if (!strncmp (p - j, ext->ext, j)) {
     	            char c = *p;
     	            
     	            *p = 0;
@@ -412,9 +419,11 @@ static char *extfs_get_path (char *inname, struct extfs_archive **archive, int i
      */
     for (parc = first_archive; parc != NULL; parc = parc->next)
 	if (archive_name){
-	    if (!strcmp (parc->name, archive_name)) {
-		vfs_stamp (&extfs_vfs_ops, (vfsid) parc);
-		goto return_success;
+	    if (parc->name) {
+		if (!strcmp (parc->name, archive_name)) {
+		    vfs_stamp (&extfs_vfs_ops, (vfsid) parc);
+		    goto return_success;
+		}
 	    }
 	} else {
 	    if (parc->fstype == fstype){
@@ -724,6 +733,7 @@ static int extfs_close (void *data)
 			    file->entry->inode->local_filename, 0);
 	if (system (cmd))
 	    errno_code = EIO;
+	free (cmd);
     }
     
     file->archive->fd_usage--;
@@ -1151,6 +1161,7 @@ vfs extfs_vfs_ops =
 
     extfs_chmod,		/* unimplemented */
     extfs_chown,		/* unimplemented */
+    NULL,
 
     extfs_readlink,
     
@@ -1174,6 +1185,7 @@ vfs extfs_vfs_ops =
     extfs_mkdir,		/* unimplemented */
     extfs_rmdir,		/* unimplemented */
     NULL,
+    NULL,
     NULL
 #ifdef HAVE_MMAP
     , extfs_mmap,
@@ -1192,7 +1204,6 @@ void extfs_init (void)
     for (extfs_no = 0; keys != NULL && extfs_no < MAXEXTFS; extfs_no++) {
 	keys = profile_iterator_next (keys, &key, &value);
 
-	value = strdup (value);
 	/* Handle those with a trailing ':', those flag that the
 	 * file system does not require an archive to work
 	 */
@@ -1204,26 +1215,18 @@ void extfs_init (void)
 	    extfs_need_archive [extfs_no] = 1;
 	
 	extfs_prefixes [extfs_no] = strdup (key);
-	p = xmalloc (strlen (value) + 2, "Extfs: extensions");
-	extfs_extensions [extfs_no] = p;
-	q = value;
-	for (c = *q; c; q = r + 1) {
-	    while (*q && *q <= 32) q++;
-	    for (r = q; *r > 32; r++);
-	    c = *r;
-	    *r = 0;
-	    if (*q) {
-	        strcpy (p, q);
-	        p = strchr (p, 0) + 1;
-	    }
-	}
- 	free (value);
-	*p = 0;
-	if (p == extfs_extensions [extfs_no]) {
-	    free (extfs_prefixes [extfs_no]);
-	    free (extfs_extensions [extfs_no]);
-	    extfs_no--;
+
+	value = strdup (value);
+	for (q = value; (p = strtok (q, " \t")) != NULL; ){
+	    struct extfs_ext *newn;
+
+	    q = NULL;
+	    newn = xmalloc (sizeof (struct extfs_ext), "prefixes");
+	    newn->next = extfs_extensions [extfs_no];
+	    newn->ext = strdup (p);
+	    extfs_extensions [extfs_no] = newn;
 	}
+	free (value);
     }
     free_profile_name (LIBDIR "extfs/extfs.ini");
 }
@@ -1244,3 +1247,25 @@ char *extfs_get_prefix (int idx)
 {
     return extfs_prefixes [idx];
 }
+
+void extfs_done (void)
+{
+    struct extfs_ext *current, *old;
+    int i;
+
+    for (i = 0; i < extfs_no; i++ ){
+	free (extfs_prefixes [i]);
+	current = extfs_extensions [i];
+	while (current){
+	    old = current;
+	    current = current->next;
+	    free (old->ext);
+	    free (old);
+	}
+    }
+    extfs_no = 0;
+    if (extfs_current_dir)
+	free (extfs_current_dir);
+    extfs_current_dir = 0;
+}
+
