@@ -45,7 +45,7 @@
 #if defined(HAVE_UNISTD_H)
 #include <unistd.h>
 #endif
-#if IS_AIX
+#if HAVE_SYS_SELECT_H
 #   include <sys/select.h>
 #endif
 #include "../src/fs.h"
@@ -79,8 +79,16 @@
 #ifndef MAXHOSTNAMELEN
 #    define MAXHOSTNAMELEN 64
 #endif
+
 #define UPLOAD_ZERO_LENGTH_FILE
 
+/* If defined, to get the proper information on symlinks, the program will
+ * first CD and then LS instead of LS the directory directly
+ */
+#undef CD_THEN_LS_DOT
+
+/* While I make up my mind, this is a no-op */
+#define wipe_password(x)
 void print_vfs_message(char *, ...);
 
 static int ftpfserrno;
@@ -89,6 +97,9 @@ static int code;
 /* Delay to retry a connection */
 int ftpfs_retry_seconds = 30;
 
+/* Method to use to connect to ftp sites */
+int ftpfs_use_passive_connections = 1;
+
 /* Use the ~/.netrc */
 int use_netrc = 1;
 
@@ -101,6 +112,9 @@ int ftpfs_directory_timeout;
 /* Proxy host */
 char *ftpfs_proxy_host = 0;
 
+/* wether we have to use proxy by default? */
+int ftpfs_always_use_proxy;
+
 /* source routing host */
 extern int source_route;
 
@@ -323,9 +337,11 @@ ftpfs_connection_close (void *data)
 {
     struct ftpfs_connection *bucket = data;
 
-    print_vfs_message ("ftpfs: Disconnecting from %s", qhost(bucket));
-    command(bucket, WAIT_REPLY, "QUIT");
-    close(qsock(bucket));
+    if (qsock (bucket) != -1){
+	print_vfs_message ("ftpfs: Disconnecting from %s", qhost(bucket));
+	command(bucket, WAIT_REPLY, "QUIT");
+	close(qsock(bucket));
+    }
 }
 
 static void
@@ -341,6 +357,8 @@ ftpfs_free_bucket (void *data)
     	free(qhome(bucket));
     if (qupdir(bucket))
         free(qupdir(bucket));
+    if (bucket->password)
+	free (bucket->password);
     linklist_destroy(qdcache(bucket), ftpfs_dir_destructor);
     free(bucket);
 }
@@ -382,35 +400,52 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
     char *proxypass, *proxyname;
 #endif
     char *pass;
+    char *op;
     char *name;			/* login user name */
+    int  anon = 0;
     
     if (netrcpass)
-        pass = strdup (netrcpass);
+        op = strdup (netrcpass);
     else {
         if (!strcmp (quser(bucket), "anonymous") || 
-            !strcmp (quser(bucket), "ftp"))
-	    pass = strdup(ftpfs_anonymous_passwd);
-        else {
+            !strcmp (quser(bucket), "ftp")) {
+	    op = strdup(ftpfs_anonymous_passwd);
+	    anon = 1;
+         } else {
             char *p;
 
-	    p = copy_strings (" FTP: Password required for ", quser(bucket), 
-			      " ", NULL);
-            pass = input_dialog (p, "Password:", "");
-            free (p);
-            if (pass == NULL) {
-	        ftpfserrno = EPERM;
-                return 0;
-            }
+	    if (!bucket->password){
+		p = copy_strings (" FTP: Password required for ", quser(bucket), 
+				  " ", NULL);
+		op = input_dialog (p, "Password:", "");
+		free (p);
+		if (op == NULL) {
+		    ftpfserrno = EPERM;
+		    return 0;
+		}
+		bucket->password = strdup (op);
+	    } else
+		op = strdup (bucket->password);
         }
     }
+
+    if (!anon || ftpfs_debug_server_dialog)
+	pass = strdup (op);
+    else
+	pass = copy_strings ("-", op, 0);
+    free (op);
+
     
     /* Proxy server accepts: username@host-we-want-to-connect*/
     if (qproxy (bucket)){
 #if defined(HSC_PROXY)
 	char *p, *host;
 	int port;
-	ftpfs_get_host_and_username(ftpfs_proxy_host, &host, &proxyname,
-				    &port, &proxypass);
+	p = ftpfs_get_host_and_username(ftpfs_proxy_host, &host, &proxyname,
+					&port, &proxypass);
+	if (p)
+	    free (p);
+	
 	free(host);
 	p = copy_strings(" Proxy: Password required for ", proxyname, " ",
 			 NULL);
@@ -422,8 +457,8 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	}
 	name = strdup(quser (bucket));
 #else
-	/* qhost(bucket) has: "!hostname" */
-	name = copy_strings (quser(bucket), "@", qhost(bucket)+1, 0);
+	name = copy_strings (quser(bucket), "@", 
+		qhost(bucket)[0] == '!' ? qhost(bucket)+1 : qhost(bucket), 0);
 #endif
     } else 
 	name = strdup (quser (bucket));
@@ -482,7 +517,9 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 		}
 	} else {
 	    bucket->failed_on_login = 1;
-
+	    free (bucket->password);
+	    bucket->password = 0;
+	    
 	    /* This matches the end of the code below, just to make it
 	     * obvious to the optimizer
 	     */
@@ -523,12 +560,119 @@ setup_source_route (int socket, int dest)
 	ptr++;
     if (setsockopt (socket, IPPROTO_IP, IP_OPTIONS,
 		    buffer, ptr - buffer) < 0)
-	message (1, " Error ", " Could not set source routing (%s)", unix_error_string (errno));
+	message_2s (1, " Error ", " Could not set source routing (%s)", unix_error_string (errno));
 }
 #else
 #define setup_source_route(x,y)
 #endif
     
+static struct no_proxy_entry {
+    char  *domain;
+    void  *next;
+} *no_proxy;
+
+static void
+load_no_proxy_list ()
+{
+    /* FixMe: shouldn't be hardcoded!!! */
+    char		s[258]; /* provide for 256 characters and nl */
+    struct no_proxy_entry *np, *current;
+    FILE	*npf;
+    int		c;
+    char	*p;
+    static int	loaded;
+
+    if (loaded)
+	return;
+
+    if (exist_file (LIBDIR "mc.no_proxy") &&
+	(npf = fopen (LIBDIR "mc.no_proxy", "r"))) {
+	while (fgets (s, 258, npf) || !(feof (npf) || ferror (npf))) {
+	    if (!(p = strchr (s, '\n'))) {	/* skip bogus entries */ 
+		while ((c = getc (npf)) != EOF && c != '\n')
+		    ;
+		continue;
+	    }
+
+	    if (p == s)
+		continue;
+
+	    *p = '\0';
+	    p = xmalloc (strlen (s), "load_no_proxy_list:1");
+	    np = xmalloc (sizeof (*np), "load_no_proxy_list:2");
+	    strcpy (p, s);
+	    np->domain = p;
+	    np->next   = 0;
+	    if (no_proxy)
+		current->next = np;
+	    else
+		no_proxy = np;
+	    current = np;
+	}
+
+	fclose (npf);
+	loaded = 1;
+    }
+}
+
+static int
+ftpfs_check_proxy (char *host)
+{
+    struct no_proxy_entry	*npe;
+
+    if (!ftpfs_proxy_host || !*ftpfs_proxy_host || !host || !*host)
+	return 0;		/* sanity check */
+
+    if (*host == '!')
+	return 1;
+    
+    if (!ftpfs_always_use_proxy)
+	return 0;
+
+    if (!strchr (host, '.'))
+	return 0;
+
+    load_no_proxy_list ();
+    for (npe = no_proxy; npe; npe=npe->next) {
+	char	*domain = npe->domain;
+
+	if (domain[0] == '.') {
+	    int		ld = strlen (domain);
+	    int		lh = strlen (host);
+
+	    while (ld && lh && host[lh - 1] == domain[ld - 1]) {
+		ld--;
+		lh--;
+	    }
+
+	    if (!ld)
+		return 0;
+	} else
+	    if (!strcasecmp (host, domain))
+		return 0;
+    }
+
+    return 1;
+}
+
+static void
+ftpfs_get_proxy_host_and_port (char *proxy, char **host, int *port)
+{
+    char *user, *pass, *dir;
+
+#if defined(HSC_PROXY)
+#define HSC_DEFAULT_PORT "9875"
+    dir = get_host_and_username(proxy, host, &user, port, HSC_DEFAULT_PORT, 1,
+				 &pass);
+#else
+    dir = get_host_and_username(proxy, host, &user, port, 21, 1,
+				 &pass);
+#endif
+    free(user);
+    free(pass);
+    free(dir);
+}
+
 static int
 ftpfs_open_socket(struct ftpfs_connection *bucket)
 {
@@ -547,14 +691,8 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
     }
 
     /* Hosts to connect to that start with a ! should use proxy */
-    if (*host == '!' && ftpfs_proxy_host){
-	bucket->use_proxy = 1;
-#if defined(HSC_PROXY)
+    if (qproxy(bucket))
 	ftpfs_get_proxy_host_and_port (ftpfs_proxy_host, &host, &port);
-#else
-	host = ftpfs_proxy_host;
-#endif
-    }
     
     /* Get host address */
     bzero ((char *) &server_address, sizeof (server_address));
@@ -574,11 +712,8 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
 	       hp->h_length);
     }
 
-#if defined(HSC_PROXY)
 #define THE_PORT qproxy(bucket) ? port : qport (bucket)
-#else
-#define THE_PORT qport(bucket)
-#endif
+
     server_address.sin_port = htons (THE_PORT);
 
     /* Connect */
@@ -639,9 +774,15 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     qupdir(bucket)= 0;
     qdcache(bucket)=0;
     bucket->lock = 0;
-    bucket->use_proxy = 0;
+    bucket->use_proxy = ftpfs_check_proxy (host);
+    bucket->password = 0;
+    bucket->use_passive_connection = ftpfs_use_passive_connections | source_route;
     bucket->use_source_route = source_route;
-    
+
+    /* We do not want to use the passive if we are using proxies */
+    if (bucket->use_proxy)
+	bucket->use_passive_connection = 0;
+
     if ((qdcache(bucket) = linklist_init()) == NULL) {
 	ftpfserrno = ENOMEM;
 	free(host);
@@ -703,17 +844,6 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     return bucket;
 }
 
-#if defined(HSC_PROXY)
-int  ftpfs_get_proxy_host_and_port (char *proxy, char **host, int *port)
-{
-#define HSC_DEFAULT_PORT "9875"
-    char *user, *pass;
-    return get_host_and_username(proxy, host, &user, port, HSC_DEFAULT_PORT, 1,
-				 &pass);
-    free(user);
-    free(pass);
-}
-#endif
 static int
 is_connection_closed(struct ftpfs_connection *bucket)
 {
@@ -901,6 +1031,7 @@ initconn (struct ftpfs_connection *bucket)
         return -1;
     }
 
+#ifdef ORIGINAL_CONNECT_CODE
     if (bucket->use_source_route){
 	int c;
 	
@@ -909,6 +1040,16 @@ initconn (struct ftpfs_connection *bucket)
 	print_vfs_message("ftpfs: could not setup passive mode for source routing");
 	bucket->use_source_route = 0;
     }
+#else
+    if (bucket->use_passive_connection){
+	if ((bucket->use_passive_connection = setup_passive (data, bucket, &data_addr)))
+	    return data;
+
+	bucket->use_source_route = 0;
+	bucket->use_passive_connection = 0;
+	print_vfs_message ("ftpfs: could not setup passive mode");
+    }
+#endif
     /* If passive setup fails, fallback to active connections */
     /* Active FTP connection */
     if (bind (data, (struct sockaddr *)&data_addr, len) < 0)
@@ -951,7 +1092,7 @@ open_data_connection (struct ftpfs_connection *bucket, char *cmd, char *remote,
         return -1;
     }
     enable_interrupt_key();
-    if (bucket->use_source_route)
+    if (bucket->use_passive_connection)
 	data = s;
     else {
 	data = accept (s, (struct sockaddr *)&from, &fromlen);
@@ -966,7 +1107,12 @@ open_data_connection (struct ftpfs_connection *bucket, char *cmd, char *remote,
     return data;
 }
 
-static char *ftpfs_get_path (struct ftpfs_connection **bucket, char *path)
+/* ftpfs_get_path:
+ * makes BUCKET point to the connection bucket descriptor for PATH
+ * returns a malloced string with the pathname relative to BUCKET.
+ */
+static char*
+ftpfs_get_path (struct ftpfs_connection **bucket, char *path)
 {
     char *user, *host, *remote_path, *pass;
     int port;
@@ -991,7 +1137,7 @@ static char *ftpfs_get_path (struct ftpfs_connection **bucket, char *path)
 	return remote_path;
     }
     /* never get here !!! */
-    message(1, " Error ", " Oops, you just hit a bug in the code ");
+    message_1s(1, " Error ", " Oops, you just hit a bug in the code ");
     return NULL;
 }
 
@@ -1057,6 +1203,8 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
 	    flist = flist->next;
 	} while (!S_ISLNK(fe->s.st_mode));
 	while (1) {
+	    if (fgets (buffer, sizeof (buffer), fp) == NULL)
+		goto done;
 	    if (parse_ls_lga (buffer, &s, &filename, NULL)) {
 		int r = strcmp(fe->name, filename);
 		free(filename);
@@ -1071,8 +1219,6 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
 		if (r < 0)
 		    break;
 	    }
-	    if (fgets (buffer, sizeof (buffer), fp) == NULL)
-		goto done;
 	}
     }
 done:
@@ -1150,12 +1296,13 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 
     has_symlinks = 0;
     print_vfs_message("ftpfs: Reading FTP directory...");
+#ifdef CD_THEN_LS_DOT
     if (command(bucket, WAIT_REPLY, "CWD %s", remote_path) != COMPLETE) {
 	ftpfserrno = ENOENT;
 	print_vfs_message("ftpfs: CWD failed.");
 	return NULL;
     }
-	 
+#endif
     file_list = linklist_init();
     if (file_list == NULL) {
 	ftpfserrno = ENOMEM;
@@ -1175,7 +1322,16 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     dcache->file_list = file_list;
     dcache->remote_path = strdup(remote_path);
     dcache->count = 1;
+#ifdef CD_THEN_LS_DOT
     sock = open_data_connection (bucket, "LIST -la", ".", 0);
+#else
+    {
+	char *path = copy_strings (remote_path, PATH_SEP_STR, ".", (char *) 0);
+	
+	sock = open_data_connection (bucket, "LIST -la", path, 0);
+	free (path);
+    }
+#endif
     if (sock == -1)
 	goto error_3;
 
@@ -1203,6 +1359,10 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 #endif
 	int eof = got_intr == 0;
 
+	if (ftpfs_debug_server_dialog){
+	    fputs (buffer, ftpfs_logfile);
+	    fflush (ftpfs_logfile);
+	}
 	if (buffer [0] == 0 && eof)
 	    break;
 	fe = xmalloc(sizeof(struct ftpentry), "struct ftpentry");
@@ -1218,6 +1378,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 	    fe->bucket = bucket;
 	    if (S_ISLNK(fe->s.st_mode))
 		has_symlinks = 1;
+	    
 	    if (!linklist_insert(file_list, fe)) {
 		free(fe);
 		ftpfserrno = ENOMEM;
@@ -1910,7 +2071,8 @@ static int ftpfs_close (void *data)
 	    ftpfserrno = EIO;
 	    result = -1;
 	}
-	flush_all_directory(fp->fe->bucket);
+	if (ftpfs_normal_flush)
+	    flush_all_directory(fp->fe->bucket);
     }
     if (fp->local_handle >= 0)
         close(fp->local_handle);
@@ -2240,6 +2402,44 @@ void ftpfs_set_debug (char *file)
 	ftpfs_debug_server_dialog = 1;
 }
 
+void ftpfs_forget (char *file)
+{
+    struct linklist *l;
+    char *host, *user, *pass, *rp;
+    int port;
+
+    if (strncmp (file, "ftp://", 6) != 0)
+	return;
+
+    file += 6;
+    if (!(rp = ftpfs_get_host_and_username (file, &host, &user, &port, &pass)))
+	return;
+
+    /* we do not care about the path actually */
+    free (rp);
+    
+    for (l = ftpfs_connections_list->next; l != ftpfs_connections_list; l = l->next){
+	struct ftpfs_connection *bucket = l->data;
+	
+	if ((strcmp (host, qhost (bucket)) == 0) &&
+	    (strcmp (user, quser (bucket)) == 0) &&
+	    (port == qport (bucket))){
+	    
+	    /* close socket: the child owns it now */
+	    close (bucket->sock);
+	    bucket->sock = -1;
+
+	    /* reopen the connection */
+	    bucket->sock = ftpfs_open_socket (bucket);
+	    if (bucket->sock != -1)
+		login_server (bucket, pass);
+	    break;
+	}
+    }
+    free (host);
+    free (user);
+}
+
 #ifdef HAVE_MMAP
 caddr_t ftpfs_mmap (caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
 {
@@ -2268,6 +2468,7 @@ vfs ftpfs_vfs_ops = {
 
     ftpfs_chmod,
     ftpfs_chown,
+    NULL,
 
     ftpfs_readlink,
     ftpfs_symlink,
@@ -2290,7 +2491,8 @@ vfs ftpfs_vfs_ops = {
     ftpfs_mkdir,
     ftpfs_rmdir,
     ftpfs_ctl,
-    ftpfs_setctl
+    ftpfs_setctl,
+    ftpfs_forget
 #ifdef HAVE_MMAP
     , ftpfs_mmap,
     ftpfs_munmap
@@ -2403,7 +2605,7 @@ int lookup_netrc (char *host, char **login, char **pass)
 			stat (netrcname, &mystat) >= 0 &&
 			(mystat.st_mode & 077)) {
 			if (be_angry) {
-			    message (1, "Error", "~/.netrc file has not correct mode.\n"
+			    message_1s (1, "Error", "~/.netrc file has not correct mode.\n"
 					         "Remove password or correct mode.");
 			    be_angry = 0;
 			}
@@ -2418,8 +2620,8 @@ int lookup_netrc (char *host, char **login, char **pass)
 		    if (stat (netrcname, &mystat) >= 0 && 
 		        (mystat.st_mode & 077)) {
 			if (be_angry) {
-			    message (1, "Error", "~/.netrc file has not correct mode.\n"
-					         "Remove password or correct mode.");
+			    message_1s (1, "Error", "~/.netrc file has not correct mode.\n"
+					            "Remove password or correct mode.");
 			    be_angry = 0;
 			}
 			free (netrc);
