@@ -32,6 +32,7 @@ struct ftpfs_connection {
     char *current_directory;
     char *home;
     char *updir;
+    char *password;
     int port;
     int sock;
     struct linklist *dcache;
@@ -40,6 +41,7 @@ struct ftpfs_connection {
     int use_proxy;		/* use a proxy server */
     int result_pending;
     int use_source_route;
+    int use_passive_connection;
 };
 
 #define qhost(b) (b)->host
@@ -63,6 +65,7 @@ struct ftpfs_connection {
 extern char *ftpfs_anonymous_passwd;
 extern char *ftpfs_proxy_host;
 extern ftpfs_directory_timeout;
+extern ftpfs_always_use_proxy;
 
 void ftpfs_init_passwd ();
 #endif
