@@ -47,8 +47,22 @@
 #define MCFS_MAX_CONNECTIONS 32
 #define mcserver_port 9876
 
+static mcfs_open_connections = 0;
+struct _mcfs_connection {
+    char *host;
+    char *user;
+    char *home;
+    int  sock;
+    int  port;
+    int  version;
+} mcfs_connections [MCFS_MAX_CONNECTIONS];
+
+typedef struct _mcfs_connection mcfs_connection;
+
+typedef struct { int handle; mcfs_connection *conn; } mcfs_handle;
+
 static int mcfs_errno_var;
-static int current_dir_sock;
+static mcfs_connection *current_dir_connection;
 
 char *mcfs_current_dir = 0;
 
@@ -60,15 +74,6 @@ char *mcfs_get_host_and_username (char *path, char **host, char **user,
     return get_host_and_username (path, host, user, port, 0, 0, pass); 
 }
 
-static mcfs_open_connections = 0;
-struct {
-    char *host;
-    char *user;
-    char *home;
-    int  sock;
-    int  port;
-} mcfs_connections [MCFS_MAX_CONNECTIONS];
-
 void mcfs_fill_names (void (*func)(char *))
 {
     int i;
@@ -94,7 +99,8 @@ void mcfs_free_bucket (int bucket)
     mcfs_connections [bucket].host =
 	mcfs_connections [bucket].user =
 	    mcfs_connections [bucket].home = 0;
-    mcfs_connections [bucket].sock = 0;
+    mcfs_connections [bucket].sock =
+	mcfs_connections [bucket].version = 0;
 }
 
 /* FIXME: This part should go to another c module, perhaps tcp.c */
@@ -128,19 +134,19 @@ extern char *last_current_dir;
 
 /* This routine checks the server RPC version and logs the user in */
 static int mcfs_login_server (int my_socket, char *user, int port,
-			      int port_autodetected, char *netrcpass)
+			      int port_autodetected, char *netrcpass,
+			      int *version)
 {
     int result;
     char *pass;
     
     /* Send the version number */
-    rpc_send (my_socket, RPC_INT, 1, RPC_END);
-    
+    rpc_send (my_socket, RPC_INT, *version, RPC_END);
     if (0 == rpc_get (my_socket, RPC_INT, &result, RPC_END))
 	return 0;
     
     if (result != MC_VERSION_OK){
-	message (1, " MCFS ", " The server does not support this version ");
+	message_1s (1, " MCFS ", " The server does not support this version ");
 	close (my_socket);
 	return 0;
     }
@@ -182,7 +188,7 @@ static int mcfs_login_server (int my_socket, char *user, int port,
 	    return 0;
 	
 	if (result != MC_LOGINOK){
-	    message (1, " MCFS ", " Invalid password ");
+	    message_1s (1, " MCFS ", " Invalid password ");
 	    close (my_socket);
 	    rpc_send (my_socket, RPC_INT, MC_QUIT, RPC_END);
 	    return 0;
@@ -192,12 +198,12 @@ static int mcfs_login_server (int my_socket, char *user, int port,
 }
 
 static int mcfs_open_tcp_link (char *host, char *user, 
-    int *port, char *netrcpass)
+    int *port, char *netrcpass, int *version)
 {
-    int my_socket;
+    int my_socket, result;
     int old_port = *port;
     
-    my_socket = open_tcp_link (host, port, " MCfs ");
+    my_socket = open_tcp_link (host, port, version, " MCfs ");
     if (my_socket <= 0)
 	return 0;
     
@@ -205,11 +211,12 @@ static int mcfs_open_tcp_link (char *host, char *user,
        implements our version of the RPC mechanism and then login
        the user.
     */
-    return mcfs_login_server (my_socket, user, *port, old_port == 0, netrcpass);
+    return mcfs_login_server (my_socket, user, *port, old_port == 0,
+			      netrcpass, version);
 }
 
 static int mcfs_get_free_bucket_init = 1;
-static int mcfs_get_free_bucket ()
+static mcfs_connection *mcfs_get_free_bucket ()
 {
     int i;
     
@@ -220,18 +227,19 @@ static int mcfs_get_free_bucket ()
     }
     for (i = 0; i < MCFS_MAX_CONNECTIONS; i++){
 	if (!mcfs_connections [i].host)
-	    return i;
+	    return &mcfs_connections [i];
     }
     /* This can't happend, since we have checked for max connections before */
     fprintf (stderr, "Internal error: mcfs_get_free_bucket");
-    return -1;
+    return 0;
 }
 
 /* This routine keeps track of open connections */
 /* Returns a connected socket to host */
-static int mcfs_open_link (char *host, char *user, int *port, char *netrcpass)
+static mcfs_connection *mcfs_open_link (char *host, char *user, int *port, char *netrcpass)
 {
-    int i, bucket, sock;
+    int i, sock, version;
+    mcfs_connection *bucket;
 
     /* Is the link actually open? */
     if (mcfs_get_free_bucket_init) {
@@ -243,25 +251,26 @@ static int mcfs_open_link (char *host, char *user, int *port, char *netrcpass)
 	    continue;
 	if ((strcmp (host, mcfs_connections [i].host) == 0) &&
 	    (strcmp (user, mcfs_connections [i].user) == 0))
-	    return mcfs_connections [i].sock;
+	    return &mcfs_connections [i];
     }
     if (mcfs_open_connections == MCFS_MAX_CONNECTIONS){
-	message (1, " Error ", " Too many open connections ");
+	message_1s (1, " Error ", " Too many open connections ");
 	return 0;
     }
 
-    if (!(sock = mcfs_open_tcp_link (host, user, port, netrcpass)))
+    if (!(sock = mcfs_open_tcp_link (host, user, port, netrcpass, &version)))
 	return 0;
     
     bucket = mcfs_get_free_bucket ();
     mcfs_open_connections++;
-    mcfs_connections [bucket].host = strdup (host);
-    mcfs_connections [bucket].user = strdup (user);
-    mcfs_connections [bucket].home = 0;
-    mcfs_connections [bucket].port = *port;
-    mcfs_connections [bucket].sock = sock;
+    bucket->host    = strdup (host);
+    bucket->user    = strdup (user);
+    bucket->home    = 0;
+    bucket->port    = *port;
+    bucket->sock    = sock;
+    bucket->version = version;
 
-    return sock;
+    return bucket;
 }
 
 static int is_error (int result, int errno_num)
@@ -282,40 +291,50 @@ static int the_error (int result, int errno_num)
     return result;
 }
 
-static char *mcfs_get_path (int *sock, char *path)
+static char *mcfs_get_path (mcfs_connection **mc, char *path)
 {
     char *user, *host, *remote_path;
     char *pass;
-    static int last_sock;
     int    port;
 
     /* An absolute path name, try to determine connection socket */
     if (strncmp (path, "mc:", 3) == 0){
 	path += 3;
 
+	/* 1.11.96 bor: add mc:// URL syntax */
+	if (path[0] == '/' && path[1] == '/')
+	    path += 2;
+
 	/* Port = 0 means that open_tcp_link will try to contact the
 	 * remote portmapper to get the port number
 	 */
 	port = 0;
 	if (!(remote_path = mcfs_get_host_and_username
-	      (path, &host, &user, &port, &pass)))
-	    return 0;
-
-	if (!(*sock = mcfs_open_link (host, user, &port, pass))){
+	      (path, &host, &user, &port, &pass))){
 	    free (host);
 	    free (user);
 	    if (pass)
 	        wipe_password (pass);
 	    return 0;
 	}
+
+	if (!(*mc = mcfs_open_link (host, user, &port, pass))){
+	    free (remote_path);
+	    free (host);
+	    free (user);
+	    if (pass)
+	        wipe_password (pass);
+	    return 0;
+	}
+	free (host);
+	free (user);
 	if (pass)
 	    wipe_password (pass);
-	last_sock = *sock;
 	return remote_path;
     }
-    *sock = current_dir_sock;
+    *mc = current_dir_connection;
     
-    return path;
+    return strdup (path);
 }
 
 /* Simple function for routines returning only an integer from the server */
@@ -336,107 +355,112 @@ static int mcfs_handle_simple_error (int sock, int return_status)
 /* Nice wrappers */
 static int mcfs_rpc_two_paths (int command, char *s1, char *s2)
 {
-    int sock;
+    mcfs_connection *mc;
     char *r1, *r2;
 
-    if ((r1 = mcfs_get_path (&sock, s1)) == 0)
+    if ((r1 = mcfs_get_path (&mc, s1)) == 0)
 	return -1;
 
-    if ((r2 = mcfs_get_path (&sock, s2)) == 0)
+    if ((r2 = mcfs_get_path (&mc, s2)) == 0){
+	free (r1);
 	return -1;
+    }
     
-    rpc_send (sock,
+    rpc_send (mc->sock,
 	      RPC_INT, command,
 	      RPC_STRING, r1,
 	      RPC_STRING, r2,
 	      RPC_END);
-
-    return mcfs_handle_simple_error (sock, 0);
+    free (r1);
+    free (r2);
+    return mcfs_handle_simple_error (mc->sock, 0);
 }
 
 static int mcfs_rpc_path (int command, char *path)
 {
-    int sock;
+    mcfs_connection *mc;
     char *remote_file;
 
-    if ((remote_file = mcfs_get_path (&sock, path)) == 0)
+    if ((remote_file = mcfs_get_path (&mc, path)) == 0)
 	return -1;
     
-    rpc_send (sock,
+    rpc_send (mc->sock,
 	      RPC_INT, command,
 	      RPC_STRING, remote_file,
 	      RPC_END);
-    return mcfs_handle_simple_error (sock, 0);
+
+    free (remote_file);
+    return mcfs_handle_simple_error (mc->sock, 0);
 }
 
 static int mcfs_rpc_path_int (int command, char *path, int data)
 {
-    int  sock;
+    mcfs_connection *mc;
     char *remote_file;
 
-    if ((remote_file = mcfs_get_path (&sock, path)) == 0)
+    if ((remote_file = mcfs_get_path (&mc, path)) == 0)
 	return -1;
 
-    rpc_send (sock,
+    rpc_send (mc->sock,
 	      RPC_INT,    command,
 	      RPC_STRING, remote_file,
 	      RPC_INT,    data, RPC_END);
 
-    return mcfs_handle_simple_error (sock, 0);
+    free (remote_file);
+    return mcfs_handle_simple_error (mc->sock, 0);
 }
 
 static int mcfs_rpc_path_int_int (int command, char *path, int n1, int n2)
 {
-    int  sock;
+    mcfs_connection *mc;
     char *remote_file;
 
-    if ((remote_file = mcfs_get_path (&sock, path)) == 0)
+    if ((remote_file = mcfs_get_path (&mc, path)) == 0)
 	return -1;
 
-    rpc_send (sock,
+    rpc_send (mc->sock,
 	      RPC_INT, command,
 	      RPC_STRING, remote_file,
 	      RPC_INT, n1,
 	      RPC_INT, n2,
 	      RPC_END);
 
-    return mcfs_handle_simple_error (sock, 0);
+    free (remote_file);
+    return mcfs_handle_simple_error (mc->sock, 0);
 }
 
 char *mcfs_gethome (char *servername)
 {
     char *remote_file;
     char *buffer;
-    int  sock, i;
+    mcfs_connection *mc;
+    int  i;
 
-    if (!(remote_file = mcfs_get_path (&sock, servername)))
+    if (!(remote_file = mcfs_get_path (&mc, servername)))
 	return 0;
     free (remote_file);
-    for (i = 0; i < MCFS_MAX_CONNECTIONS; i++)
-	if (mcfs_connections[i].sock == sock)
-	    if (mcfs_connections[i].home)
-		return strdup (mcfs_connections[i].home);
-	    else {
-		rpc_send (sock, RPC_INT, MC_GETHOME, RPC_END);
-		if (0 == rpc_get (sock, RPC_STRING, &buffer, RPC_END))
-		    return strdup ("/");
-		mcfs_connections[i].home = buffer;
-		return strdup (buffer);
-	    }
-    return 0;
+    if (mc->home)
+	return strdup (mc->home);
+    else {
+	rpc_send (mc->sock, RPC_INT, MC_GETHOME, RPC_END);
+	if (0 == rpc_get (mc->sock, RPC_STRING, &buffer, RPC_END))
+	    return strdup ("/");
+	mc->home = buffer;
+	return strdup (buffer);
+    }
 }
 
 char *mcfs_getupdir (char *servername)
 {
     char *remote_file;
-    int  sock;
+    mcfs_connection *mc;
     char *buffer;
 
-    if (!(remote_file = mcfs_get_path (&sock, servername)))
+    if (!(remote_file = mcfs_get_path (&mc, servername)))
 	return 0;
     free (remote_file);
-    rpc_send (sock, RPC_INT, MC_GETUPDIR, RPC_END);
-    if (0 == rpc_get (sock, RPC_STRING, &buffer, RPC_END)) {
+    rpc_send (mc->sock, RPC_INT, MC_GETUPDIR, RPC_END);
+    if (0 == rpc_get (mc->sock, RPC_STRING, &buffer, RPC_END)) {
         return strdup ("/");
     }
     return buffer;
@@ -446,47 +470,50 @@ char *mcfs_getupdir (char *servername)
 static void *mcfs_open (char *file, int flags, int mode)
 {
     char *remote_file;
-    int  sock, result, error_num;
-    int  *remote_handle;
+    mcfs_connection *mc;
+    int  result, error_num;
+    mcfs_handle  *remote_handle;
 
-    if (!(remote_file = mcfs_get_path (&sock, file)))
+    if (!(remote_file = mcfs_get_path (&mc, file)))
 	return 0;
 
-    rpc_send (sock, RPC_INT, MC_OPEN, RPC_STRING, remote_file,
+    rpc_send (mc->sock, RPC_INT, MC_OPEN, RPC_STRING, remote_file,
 	      RPC_INT, flags, RPC_INT, mode, RPC_END);
-    
-    if (0 == rpc_get (sock, RPC_INT, &result, RPC_INT, &error_num, RPC_END))
+    free (remote_file);    
+
+    if (0 == rpc_get (mc->sock, RPC_INT, &result, RPC_INT, &error_num, RPC_END))
 	return 0;
 
     if (is_error (result, error_num))
 	return 0;
 
-    remote_handle = (int *) xmalloc (2 * sizeof (int), "int");
-    remote_handle [0] = result;
-    remote_handle [1] = sock;
+    remote_handle = (mcfs_handle *) xmalloc (2 * sizeof (mcfs_handle), "mcfs_handle");
+    remote_handle->handle  = result;
+    remote_handle->conn    = mc;
     
     return remote_handle;
 }
 
 static int mcfs_read (void *data, char *buffer, int count)
 {
-    int *info = (int *) data;
+    mcfs_handle *info = (mcfs_handle *) data;
     int result, error;
-    int handle, sock;
+    int handle;
+    mcfs_connection *mc;
 
-    sock = info [1];
-    handle = info [0];
+    mc =     info->conn;
+    handle = info->handle; 
     
-    rpc_send (sock, RPC_INT, MC_READ, RPC_INT, handle,
+    rpc_send (mc->sock, RPC_INT, MC_READ, RPC_INT, handle,
 	      RPC_INT, count, RPC_END);
     
-    if (0 == rpc_get  (sock, RPC_INT, &result, RPC_INT, &error, RPC_END))
+    if (0 == rpc_get  (mc->sock, RPC_INT, &result, RPC_INT, &error, RPC_END))
 	return the_error (-1, EIO);
     
     if (is_error (result, error))
 	return 0;
     
-    if (0 == rpc_get  (sock, RPC_BLOCK, result, buffer, RPC_END))
+    if (0 == rpc_get  (mc->sock, RPC_BLOCK, result, buffer, RPC_END))
 	return the_error (-1, EIO);
     
     return result;
@@ -494,36 +521,38 @@ static int mcfs_read (void *data, char *buffer, int count)
 
 int mcfs_write (void *data, char *buf, int nbyte)
 {
-    int *info = (int *) data;
-    int handle, sock;
+    mcfs_handle *info = (mcfs_handle *) data;
+    mcfs_connection *mc;
+    int handle;
 
-    sock = info [1];
-    handle = info [0];
+    mc     = info->conn;
+    handle = info->handle;
     
-    rpc_send (sock,
+    rpc_send (mc->sock,
 	      RPC_INT, MC_WRITE,
 	      RPC_INT, handle,
 	      RPC_INT, nbyte,
 	      RPC_BLOCK, nbyte, buf,
 	      RPC_END);
     
-    return mcfs_handle_simple_error (sock, 1);
+    return mcfs_handle_simple_error (mc->sock, 1);
 }
 
 static int mcfs_close (void *data)
 {
-    int *info = (int *) data;
-    int handle, sock, result, error;
+    mcfs_handle *info = (mcfs_handle *) data;
+    mcfs_connection *mc;
+    int handle, result, error;
 
     if (!data)
 	return -1;
     
-    handle = info [0];
-    sock   = info [1];
+    handle = info->handle;
+    mc     = info->conn;
     
-    rpc_send (sock, RPC_INT, MC_CLOSE, RPC_INT, handle, RPC_END);
+    rpc_send (mc->sock, RPC_INT, MC_CLOSE, RPC_INT, handle, RPC_END);
     
-    if (0 == rpc_get (sock, RPC_INT, &result, RPC_INT, &error, RPC_END))
+    if (0 == rpc_get (mc->sock, RPC_INT, &result, RPC_INT, &error, RPC_END))
 	return the_error (-1, EIO);
     
     is_error (result, error);
@@ -545,7 +574,7 @@ typedef struct dir_entry {
 } dir_entry;
 
 typedef struct {
-    int link;
+    mcfs_connection *conn;
     int handle;
     dir_entry *entries;
     dir_entry *current;
@@ -554,16 +583,18 @@ typedef struct {
 static void *mcfs_opendir (char *dirname)
 {
     opendir_info *mcfs_info;
-    int link, handle, error_num;
+    mcfs_connection *mc;
+    int handle, error_num;
     char *remote_dir;
     int  result;
 
-    if (!(remote_dir = mcfs_get_path (&link, dirname)))
+    if (!(remote_dir = mcfs_get_path (&mc, dirname)))
 	return 0;
 
-    rpc_send (link, RPC_INT, MC_OPENDIR, RPC_STRING, remote_dir, RPC_END);
+    rpc_send (mc->sock, RPC_INT, MC_OPENDIR, RPC_STRING, remote_dir, RPC_END);
+    free (remote_dir);
     
-    if (0 == rpc_get  (link, RPC_INT, &result, RPC_INT, &error_num, RPC_END))
+    if (0 == rpc_get  (mc->sock, RPC_INT, &result, RPC_INT, &error_num, RPC_END))
 	return 0;
     
     if (is_error (result, error_num))
@@ -572,7 +603,7 @@ static void *mcfs_opendir (char *dirname)
     handle = result;
 
     mcfs_info = (opendir_info *) xmalloc (sizeof(opendir_info),"mcfs_opendir");
-    mcfs_info->link = link;
+    mcfs_info->conn = mc;
     mcfs_info->handle = handle;
     mcfs_info->entries = 0;
     mcfs_info->current = 0;
@@ -580,12 +611,13 @@ static void *mcfs_opendir (char *dirname)
     return mcfs_info;
 }
 
-static int get_stat_info (int sock, struct stat *buf);
+static int get_stat_info (mcfs_connection *mc, struct stat *buf);
 
 static int mcfs_loaddir (opendir_info *mcfs_info)
 {
     int  status, error;
-    int  link = mcfs_info->link;
+    mcfs_connection  *mc = mcfs_info->conn;
+    int  link = mc->sock;
     int  first = 1;
 
     rpc_send (link, RPC_INT, MC_READDIR, RPC_INT, mcfs_info->handle, RPC_END);
@@ -625,7 +657,7 @@ static int mcfs_loaddir (opendir_info *mcfs_info)
 	    new_entry->merrno = error;
 	else {
 	    new_entry->merrno = 0;
-	    if (!get_stat_info (link, &(new_entry->my_stat)))
+	    if (!get_stat_info (mc, &(new_entry->my_stat)))
 		return 0;
 	}
     } 
@@ -693,7 +725,7 @@ static int mcfs_closedir (void *info)
     opendir_info *mcfs_info = (opendir_info *) info;
     dir_entry *p, *q;
 
-    rpc_send (mcfs_info->link, RPC_INT, MC_CLOSEDIR,
+    rpc_send (mcfs_info->conn->sock, RPC_INT, MC_CLOSEDIR,
 	      RPC_INT, mcfs_info->handle, RPC_END);
     
     for (p = mcfs_info->entries; p;){
@@ -706,27 +738,43 @@ static int mcfs_closedir (void *info)
     return 0;
 }
 
-static time_t mcfs_get_time (int sock)
+static time_t mcfs_get_time (mcfs_connection *mc)
 {
-    struct tm tt;
+    int       sock = mc->sock;
     
-    rpc_get (sock,
-	     RPC_INT, &tt.tm_sec,
-	     RPC_INT, &tt.tm_min,
-	     RPC_INT, &tt.tm_hour,
-	     RPC_INT, &tt.tm_mday,
-	     RPC_INT, &tt.tm_year,
-	     RPC_INT, &tt.tm_mon,
-	     RPC_END);
-    tt.tm_year -= 1900;
-    tt.tm_isdst = 0;
+    if (mc->version == 1) {
+	struct tm tt;
 
-    return mktime (&tt);
+	rpc_get (sock,
+		 RPC_INT, &tt.tm_sec,
+		 RPC_INT, &tt.tm_min,
+		 RPC_INT, &tt.tm_hour,
+		 RPC_INT, &tt.tm_mday,
+		 RPC_INT, &tt.tm_year,
+		 RPC_INT, &tt.tm_mon,
+		 RPC_END);
+	tt.tm_year -= 1900;
+	tt.tm_isdst = 0;
+
+	return mktime (&tt);
+    } else {
+	char  *buf;
+	long  tm;
+
+	rpc_get (sock,
+		 RPC_STRING, &buf,
+		 RPC_END);
+	sscanf (buf, "%lx", &tm);
+	free (buf);
+
+	return (time_t) tm;
+    }
 }
 
-static int get_stat_info (int sock, struct stat *buf)
+static int get_stat_info (mcfs_connection *mc, struct stat *buf)
 {
     long mylong;
+    int  sock = mc->sock;
 
 #ifdef HAVE_ST_RDEV
     buf->st_rdev = 0;
@@ -752,29 +800,30 @@ static int get_stat_info (int sock, struct stat *buf)
 #ifdef HAVE_ST_BLOCKS
     buf->st_blocks = mylong;
 #endif
-    buf->st_atime = mcfs_get_time (sock);
-    buf->st_mtime = mcfs_get_time (sock);
-    buf->st_ctime = mcfs_get_time (sock);
+    buf->st_atime = mcfs_get_time (mc);
+    buf->st_mtime = mcfs_get_time (mc);
+    buf->st_ctime = mcfs_get_time (mc);
     return 1;
 }
 
 static int mcfs_stat_cmd (int cmd, char *path, struct stat *buf)
 {
     char *remote_file;
-    int  sock;
+    mcfs_connection *mc;
     int  status, error;
     
-    if ((remote_file = mcfs_get_path (&sock, path)) == 0)
+    if ((remote_file = mcfs_get_path (&mc, path)) == 0)
 	return -1;
 
-    rpc_send (sock, RPC_INT, cmd, RPC_STRING, remote_file, RPC_END);
-    if (!rpc_get (sock, RPC_INT, &status, RPC_INT, &error, RPC_END))
+    rpc_send (mc->sock, RPC_INT, cmd, RPC_STRING, remote_file, RPC_END);
+    free (remote_file);
+    if (!rpc_get (mc->sock, RPC_INT, &status, RPC_INT, &error, RPC_END))
 	return the_error (-1, errno);
 
     if (is_error (status, error))
 	return -1;
 
-    if (get_stat_info (sock, buf))
+    if (get_stat_info (mc, buf))
 	return 0;
     else
 	return the_error (-1, EIO);
@@ -802,12 +851,12 @@ static int mcfs_lstat (char *path, struct stat *buf)
 
 int mcfs_fstat (void *data, struct stat *buf)
 {
-    int *info = (int *) data;
+    mcfs_handle *info = (mcfs_handle *) data;
     int result, error;
     int handle, sock;
     
-    sock = info [1];
-    handle = info [0];
+    sock = info->conn->sock;
+    handle = info->handle;
     
     rpc_send (sock, RPC_INT, MC_FSTAT, RPC_INT, handle, RPC_END);
     if (!rpc_get (sock, RPC_INT, &result, RPC_INT, &error, RPC_END))
@@ -816,7 +865,7 @@ int mcfs_fstat (void *data, struct stat *buf)
     if (is_error (result, error))
 	return -1;
 
-    if (get_stat_info (sock, buf))
+    if (get_stat_info (info->conn, buf))
 	return 0;
     else
 	return the_error (-1, EIO);
@@ -832,23 +881,57 @@ int mcfs_chown (char *path, int owner, int group)
     return mcfs_rpc_path_int_int (MC_CHOWN, path, owner, group);
 }
 
+int mcfs_utime (char *path, struct utimbuf *times)
+{
+    mcfs_connection   *mc;
+    int   i, status;
+    char *file;
+
+    if (!(file = mcfs_get_path (&mc, path)))
+	return -1;
+
+    status = 0;
+    if (mc->version >= 2) {
+	char   abuf[2*sizeof(long) + 1];
+	char   mbuf[2*sizeof(long) + 1];
+	long   atime, mtime;
+
+	atime = (long) times->actime;
+	mtime = (long) times->modtime;
+
+	sprintf (abuf, "%lx", atime);
+	sprintf (mbuf, "%lx", mtime);
+
+	rpc_send (mc->sock, RPC_INT, MC_UTIME,
+			RPC_STRING, file,
+			RPC_STRING, abuf,
+			RPC_STRING, mbuf,
+			RPC_END);
+	status = mcfs_handle_simple_error (mc->sock, 0);
+    
+    }
+    free (file);
+    return (status);
+}
+
 static int mcfs_readlink (char *path, char *buf, int size)
 {
     char *remote_file, *stat_str;
     int  status, error;
-    int  sock;
+    mcfs_connection *mc;
 
-    if (!(remote_file = mcfs_get_path (&sock, path)))
+    if (!(remote_file = mcfs_get_path (&mc, path)))
 	return -1;
 
-    rpc_send (sock, RPC_INT, MC_READLINK, RPC_STRING, remote_file, RPC_END);
-    if (!rpc_get (sock, RPC_INT, &status, RPC_INT, &error, RPC_END))
+    rpc_send (mc->sock, RPC_INT, MC_READLINK, RPC_STRING, remote_file, RPC_END);
+    free (remote_file);
+    if (!rpc_get (mc->sock, RPC_INT, &status, RPC_INT, &error, RPC_END))
 	return the_error (-1, EIO);
 
     if (is_error (status, errno))
 	return -1;
 
-    if (!rpc_get (sock, RPC_STRING, &stat_str, RPC_END))
+    if (!rpc_get (mc->sock, RPC_STRING, &stat_str, RPC_END))
 	return the_error (-1, EIO);
     
     strncpy (buf, stat_str, size);
@@ -874,9 +957,10 @@ int mcfs_rename (char *a, char *b)
 static int mcfs_chdir (char *path)
 {
     char *remote_dir;
-    int  sock, status, error;
+    mcfs_connection *mc;
+    int  status, error;
 
-    if (!(remote_dir = mcfs_get_path (&sock, path)))
+    if (!(remote_dir = mcfs_get_path (&mc, path)))
 	return -1;
 
     if (mcfs_current_dir)
@@ -884,9 +968,10 @@ static int mcfs_chdir (char *path)
     
     mcfs_current_dir = strdup (path);
     
-    current_dir_sock = sock;
-    rpc_send (sock, RPC_INT, MC_CHDIR, RPC_STRING, remote_dir, RPC_END);
-    if (!rpc_get (sock, RPC_INT, &status, RPC_INT, &error, RPC_END))
+    current_dir_connection = mc;
+    rpc_send (mc->sock, RPC_INT, MC_CHDIR, RPC_STRING, remote_dir, RPC_END);
+    free (remote_dir);
+    if (!rpc_get (mc->sock, RPC_INT, &status, RPC_INT, &error, RPC_END))
 	return the_error (-1, EIO);
     
     if (is_error (status, error))
@@ -896,11 +981,11 @@ static int mcfs_chdir (char *path)
 
 int mcfs_lseek (void *data, off_t offset, int whence)
 {
-    int *info = (int *) data;
+    mcfs_handle *info = (mcfs_handle *) data;
     int handle, sock;
 
-    sock = info [1];
-    handle = info [0];
+    sock = info->conn->sock; 
+    handle = info->handle;
 
     rpc_send (sock,
 	      RPC_INT, MC_LSEEK,
@@ -961,6 +1046,47 @@ static void mcfs_ungetlocalcopy (char *path, char *local, int has_changed)
     mc_def_ungetlocalcopy (path, local, has_changed);
 }
 
+/* Gives up on a socket and reopnes the connection, the child own the socket
+ * now
+ */
+void
+mcfs_forget (char *path)
+{
+    char *host, *user, *pass, *p;
+    int  port, i, vers;
+    
+    if (strncmp (path, "mc:", 3) != 0)
+	return;
+    path += 3;
+    if (path[0] == '/' && path[1] == '/')
+	path += 2;
+
+    if ((p = mcfs_get_host_and_username (path, &host, &user, &port, &pass)) == 0) {
+	free (host);
+	free (user);
+	if (pass)
+	    wipe_password (pass);
+	return;
+    }
+    for (i = 0; i < MCFS_MAX_CONNECTIONS; i++){
+	if ((strcmp (host, mcfs_connections [i].host) == 0) &&
+	    (strcmp (user, mcfs_connections [i].user) == 0) &&
+	    (port == mcfs_connections [i].port)){
+
+	    /* close socket: the child owns it now */
+	    close (mcfs_connections [i].sock);
+
+	    /* reopen the connection */
+	    mcfs_connections [i].sock = mcfs_open_tcp_link (host, user, &port, pass, &vers);
+	}
+    }
+    free (p);
+    free (host);
+    free (user);
+    if (pass)
+	wipe_password (pass);
+}
+
 #ifdef HAVE_MMAP
 caddr_t mcfs_mmap (caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
 {
@@ -989,6 +1115,7 @@ vfs mcfs_vfs_ops = {
 
     mcfs_chmod,
     mcfs_chown,
+    mcfs_utime,
 
     mcfs_readlink,
     mcfs_symlink,
@@ -1011,7 +1138,8 @@ vfs mcfs_vfs_ops = {
     mcfs_mkdir,
     mcfs_rmdir,
     NULL,
-    NULL
+    NULL,
+    mcfs_forget
 #ifdef HAVE_MMAP
     , mcfs_mmap,
     mcfs_munmap
