@@ -2,7 +2,7 @@
 
 /* This number was registered for program "mcfs" with rpc@Sun.COM */
 #define RPC_PROGNUM 300516
-#define RPC_PROGVER 1
+#define RPC_PROGVER 2
 
 /* this constants must be kept in sync with mcserv.c commands */
 /* They are the messages sent on the link connection */
@@ -36,12 +36,16 @@ enum {
     MC_LOGIN,
     MC_QUIT,
     
+    MC_UTIME,           /* it has to go here for compatibility with old
+			   servers/clients. sigh ... */
+
     MC_INVALID_PASS = 0x1000,
     MC_NEED_PASSWORD,
     MC_LOGINOK,
     MC_VERSION_OK,
     MC_VERSION_MISMATCH,
     MC_PASS
+
 };
 
 extern char *mcfs_current_dir;
