@@ -62,12 +62,16 @@
 #include <errno.h>
 #include <signal.h>
 #include <time.h>
+#include <utime.h>
 
 /* Network include files */
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#if 0
+#include <sys/byteorder.h>
+#endif
 #ifdef HAVE_PMAP_SET
 #include <rpc/rpc.h>
 #include <rpc/pmap_prot.h>
@@ -90,6 +94,9 @@
 /* The socket from which we accept commands */
 int msock;
 
+/* Requested version number from client */
+static int clnt_version;
+
 /* If non zero, we accept further commands */
 int logged_in = 0;
 
@@ -121,8 +128,17 @@ int r_auth = 0;
 
 #define OPENDIR_HANDLES 8
 
+#define DO_QUIT() \
+do { \
+  quit_server = 1; \
+  return_code = 1; \
+  return; \
+} while (0)
+
 char buffer [4096];
 int debug = 1;
+static int quit_server;
+static int return_code;
 
 /* }}} */
 
@@ -218,40 +234,50 @@ void do_close (void)
 
 void send_time (int sock, time_t time)
 {
-    char   *ct;
-    int    month;
+    if (clnt_version == 1) {
+	char   *ct;
+	int    month;
     
-    ct = ctime (&time);
-    ct [3] = ct [10] = ct [13] = ct [16] = ct [19] = 0;
-    
-    /* Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */
-    if (ct [4] == 'J'){
-	if (ct [5] == 'a'){
-	    month = 0;
+	ct = ctime (&time);
+	ct [3] = ct [10] = ct [13] = ct [16] = ct [19] = 0;
+	
+	/* Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */
+	if (ct [4] == 'J'){
+	    if (ct [5] == 'a'){
+		month = 0;
+	    } else
+		month = (ct [6] == 'n') ? 5 : 6;
+	} else if (ct [4] == 'F'){
+	    month = 1;
+	} else if (ct [4] == 'M'){
+	    month = (ct [6] == 'r') ? 2 : 5;
+	} else if (ct [4] == 'A'){
+	    month = (ct [5] == 'p') ? 3 : 7;
+	} else if (ct [4] == 'S'){
+	    month = 8;
+	} else if (ct [4] == 'O'){
+	    month = 9;
+	} else if (ct [4] == 'N'){
+	    month = 10;
 	} else
-	    month = (ct [6] == 'n') ? 5 : 6;
-    } else if (ct [4] == 'F'){
-	month = 1;
-    } else if (ct [4] == 'M'){
-	month = (ct [6] == 'r') ? 2 : 5;
-    } else if (ct [4] == 'A'){
-	month = (ct [5] == 'p') ? 3 : 7;
-    } else if (ct [4] == 'S'){
-	month = 8;
-    } else if (ct [4] == 'O'){
-	month = 9;
-    } else if (ct [4] == 'N'){
-	month = 10;
-    } else
 	month = 11;
-    rpc_send (msock,
-	      RPC_INT, atoi (&ct [17]),  	/* sec */
-	      RPC_INT, atoi (&ct [14]),         /* min */
-	      RPC_INT, atoi (&ct [11]),         /* hour */
-  	      RPC_INT, atoi (&ct [8]),          /* mday */
-              RPC_INT, atoi (&ct [20]),         /* year */
-              RPC_INT, month,	                /* month */
-	      RPC_END);
+	rpc_send (msock,
+		  RPC_INT, atoi (&ct [17]),  	/* sec */
+		  RPC_INT, atoi (&ct [14]),         /* min */
+		  RPC_INT, atoi (&ct [11]),         /* hour */
+		  RPC_INT, atoi (&ct [8]),          /* mday */
+		  RPC_INT, atoi (&ct [20]),         /* year */
+		  RPC_INT, month,	                /* month */
+		  RPC_END);
+    } else {
+	long ltime = (long) time;
+	char buf[2*sizeof(long) + 1];
+
+	sprintf (buf, "%lx", ltime);
+	rpc_send (msock,
+		  RPC_STRING, buf,
+		  RPC_END);
+    }
 }
 
 void send_stat_info (struct stat *st)
@@ -581,12 +607,36 @@ void do_chown (void)
     free (file);
 }
 
-void do_quit (int result)
+void do_utime (void)
 {
-#ifdef HAVE_PMAP_SET
-    pmap_unset (RPC_PROGNUM, RPC_PROGVER);
-#endif
-    exit (result);
+    char *file;
+    int  status;
+    long atime;
+    long mtime;
+    char *as;
+    char *ms;
+    struct utimbuf times;
+    
+    rpc_get (msock, RPC_STRING, &file,
+		    RPC_STRING, &as,
+		    RPC_STRING, &ms,
+		    RPC_END);
+    sscanf (as, "%lx", &atime);
+    sscanf (ms, "%lx", &mtime);
+    if (verbose) printf ("Got a = %s, m = %s, comp a = %ld, m = %ld\n",
+			  as, ms, atime, mtime);
+    free (as);
+    free (ms);
+    times.actime  = (time_t) atime;
+    times.modtime = (time_t) mtime;
+    status = utime (file, &times);
+    send_status (status, errno);
+    free (file);
+}
+
+void do_quit ()
+{
+    quit_server = 1;
 }
 
 #ifdef HAVE_PAM
@@ -921,10 +971,10 @@ void do_login ()
     rpc_send (msock, RPC_INT, MC_NEED_PASSWORD, RPC_END);
     rpc_get (msock, RPC_INT, &result, RPC_END);
     if (result == MC_QUIT)
-	do_quit (0);
+	DO_QUIT ();
     if (result != MC_PASS){
 	if (verbose) printf ("do_login: Unknown response: %d\n", result);
-	do_quit (0);
+	DO_QUIT ();
     }
     rpc_get (msock, RPC_STRING, &password, RPC_END);
     logged_in = do_auth (username, password);
@@ -938,7 +988,7 @@ void do_login ()
 
 /* This structure must be kept in synch with mcfs.h enums */
 
-struct {
+struct _command {
     char *command;
     void (*callback)(void);
 } commands [] = {
@@ -967,14 +1017,19 @@ struct {
     { "gethome",    do_gethome },
     { "getupdir",   do_getupdir },
     { "login",      do_login },
-    { "quit",       do_quit }
+    { "quit",       do_quit },
+    { "utime",      do_utime },
 };
 
+static ncommands = sizeof(commands)/sizeof(struct _command);
+
 void exec_command (int command)
 {
-    if (commands [command].command == 0){
+    if (command < 0 ||
+	command >= ncommands ||
+	commands [command].command == 0){
 	fprintf (stderr, "Got unknown command: %d\n", command);
-	do_quit (1);
+	DO_QUIT ();
     }
     if (verbose) printf ("Command: %s\n", commands [command].command);
     (*commands [command].callback)();
@@ -985,17 +1040,20 @@ void check_version ()
     int version;
     
     rpc_get (msock, RPC_INT, &version, RPC_END);
-    if (version == 1)
+    if (version >= 1 &&
+	version <= RPC_PROGVER)
 	rpc_send (msock, RPC_INT, MC_VERSION_OK, RPC_END);
     else
-	rpc_send (msock, RPC_INT, MC_VERSION_MISMATCH);
+	rpc_send (msock, RPC_INT, MC_VERSION_MISMATCH, RPC_END);
+    
+    clnt_version = version;
 }
 
 /* This routine is called by rpc_get/rpc_send when the connection is closed */
 void tcp_invalidate_socket (int sock)
 {
-    printf ("Connection closed\n");
-    do_quit (1);
+    if (verbose) printf ("Connection closed\n");
+    DO_QUIT();
 }
 
 void server (int sock)
@@ -1003,13 +1061,14 @@ void server (int sock)
     int command;
     
     msock = sock;
+    quit_server = 0;
 
     check_version ();
-    for (;;){
-	rpc_get (sock, RPC_INT, &command, RPC_END);
-	if (logged_in || command == MC_LOGIN)
+    do {
+	if (rpc_get (sock, RPC_INT, &command, RPC_END) &&
+	    (logged_in || command == MC_LOGIN))
 	    exec_command (command);
-    }
+    } while (!quit_server);
 }
 
 /* }}} */
@@ -1050,28 +1109,19 @@ char *get_client (int portnum)
 
     for (;;){
 	clilen = sizeof (client_address);
-
-	if (verbose)
-	    printf ("Waiting for %sconnection...", isDaemon ? "":"just one ");
-	fflush (stdout);
 	newsocket = accept (sock, (struct sockaddr *) &client_address,
 			    &clilen);
-	if (verbose) printf ("ok\n");
-	if (isDaemon){
-	    if (fork ()){
-		close (newsocket);
-		/* This is needed to avoid zombies */
-		if (fork () != 0)
-		    exit (0);
-	    } else {
-		close (sock);
-		server (newsocket);
-		exit (0);
-	    }
-	} else {
-	    server (newsocket);
+
+	if (isDaemon && fork()) {
 	    close (newsocket);
+	    continue;
 	}
+
+	if (isDaemon && fork()) exit (0);
+
+	server (newsocket);
+	close (newsocket);
+	return 0;
     }
 }
 
@@ -1097,7 +1147,7 @@ int get_port_number ()
     if (port == -1){
 	if (geteuid () == 0){
 	    fprintf (stderr, "Could not bind the server on a reserved port\n");
-	    do_quit (1);
+	    DO_QUIT ();
 	}
 	port = 0;
     }
@@ -1187,6 +1237,8 @@ int main (int argc, char *argv [])
 	}
     }
 
+    if (isDaemon && fork()) exit (0);
+
     if (portnum == 0)
 	portnum = get_port_number ();
     register_port (portnum, 0);
@@ -1195,7 +1247,11 @@ int main (int argc, char *argv [])
 	printf ("Using port %d\n", portnum);
     if ((result = get_client (portnum)))
 	perror (result);
-    do_quit (0);
+#ifdef HAVE_PMAP_SET
+    if (!isDaemon)
+	pmap_unset (RPC_PROGNUM, RPC_PROGVER);
+#endif
+    exit (return_code);
 }
 
 /* This functions are not used */
