@@ -44,10 +44,6 @@ int tar_gzipped_memlimit = 1*1024*1024;
 /* used to rotate the dash */
 int dash_number = 0;
 
-/* In order not to include all dialoging stuff, I'm not including dialog.h
-   just for this function */
-extern void *message (int error, char *header, char *text,...);
-
 #define	isodigit(c)	( ((c) >= '0') && ((c) <= '7') )
 /*
  * Quick and dirty octal conversion.
@@ -339,6 +335,12 @@ static INLINE int uncompress_tar_file  (struct tarfs_archive *current_archive,
 	if ((dash_number++ % 64) == 0)
 	    rotate_dash ();
 	fwrite (buffer, 1, i, f);
+	if (ferror (f)) {
+            pclose (f);
+            mc_close (fd);
+            free_archive (current_archive);
+            return -1;
+        }
     }
     
     pclose (f);
@@ -535,7 +537,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	     size -= written) {
 	    data = get_next_record (archive, tard)->charptr;
 	    if (data == NULL) {
-		message (1, " Error ", "Unexpected EOF on archive file");
+		message_1s (1, " Error ", "Unexpected EOF on archive file");
 		return 0;
 	    }
 	    written = RECORDSIZE;
@@ -586,7 +588,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	    
 	pent = tarfs_find_entry (archive->root_entry, q, 1);
 	if (pent == NULL) {
-	    message (1, " Error ", "Inconsistent tar archive");
+	    message_1s (1, " Error ", "Inconsistent tar archive");
 	}
 
 	entry = (struct tarfs_entry *) xmalloc (sizeof (struct tarfs_entry), "Tar: tarfs_entry");
@@ -607,7 +609,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	if (header->header.linkflag == LF_LINK) {
 	    pent = tarfs_find_entry (archive->root_entry, current_link_name, 0);
 	    if (pent == NULL) {
-	        message (1, " Error ", "Inconsistent tar archive");
+	        message_1s (1, " Error ", "Inconsistent tar archive");
 	    } else {
 		entry->inode = pent->inode;
 		pent->inode->nlink++;
@@ -687,7 +689,7 @@ int read_tar_archive (char *name, struct tarfs_archive **pparc)
     struct tarfs_archive *archive;
 
     if ((tard = open_tar_archive (name, &archive)) == -1) {	/* Open for reading */
-        message (1, " Error ", "Couldn't open tar archive\n%s", name);
+        message_2s (1, " Error ", "Couldn't open tar archive\n%s", name);
 	return -1;
     }
 
@@ -706,12 +708,12 @@ int read_tar_archive (char *name, struct tarfs_archive **pparc)
 	case 0:		/* Invalid header */
 	    switch (prev_status) {
 	    case 3:		/* Error on first record */
-		message (1, " Error ", "Hmm,...\n%s\ndoesn't look like a tar archive.", name);
+		message_2s (1, " Error ", "Hmm,...\n%s\ndoesn't look like a tar archive.", name);
 		/* FALL THRU */
 	    case 2:		/* Error after record of zeroes */
 	    case 1:		/* Error after header rec */
 #if 0
-		message (0, " Warning ", "Skipping to next file header...");
+		message_1s (0, " Warning ", "Skipping to next file header...");
 #endif
 	    case 0:		/* Error after error */
 		return -1;
@@ -978,7 +980,7 @@ static void *tar_open (char *file, int flags, int mode)
     	tarerrno = EISDIR;
     	return NULL;
     }
-    if (flags & O_ACCMODE != O_RDONLY) {
+    if ((flags & O_ACCMODE) != O_RDONLY) {
     	tarerrno = EROFS; /* At the moment we are RO */
     	return NULL;
     }
@@ -1267,7 +1269,7 @@ static int tar_rename (char *a, char *b)
 static int tar_chdir (char *path)
 {
     struct tarfs_archive *archive;
-    char *p, *q, *res;
+    char *p, *q;
     struct tarfs_entry *entry;
 
     tarerrno = ENOTDIR;
@@ -1289,7 +1291,6 @@ static int tar_chdir (char *path)
     	return -1;
     }
     entry->inode->archive->current_dir = entry;
-    res = copy_strings ("tar:", entry->inode->archive->name, p, NULL);
     free (p);
     tarerrno = 0;
     return 0;
@@ -1469,6 +1470,7 @@ vfs tarfs_vfs_ops =
 
     tar_chmod,
     tar_chown,
+    NULL,
 
     tar_readlink,
     tar_symlink,
@@ -1491,6 +1493,7 @@ vfs tarfs_vfs_ops =
     tar_mkdir,
     tar_rmdir,
     NULL,
+    NULL,
     NULL
 #ifdef HAVE_MMAP
     , tar_mmap,
