@@ -253,19 +253,31 @@ void tcp_init (void)
     sigaction (SIGPIPE, &sa, NULL);
 }
 
-int get_remote_port (struct sockaddr_in *sin)
+int get_remote_port (struct sockaddr_in *sin, int *version)
 {
-    int port;
+    int              port;
+#ifdef HAVE_PMAP_GETMAPS
+    struct pmaplist  *pl;
     
-#ifdef HAVE_PMAP_GETPORT
-    port = pmap_getport (sin, RPC_PROGNUM, RPC_PROGVER, IPPROTO_TCP);
-    if (port == 0)
-	return mcserver_port;
-    else
-	return port;
+    *version = 1;
+    port     = mcserver_port;
+    for (pl = pmap_getmaps (sin); pl; pl = pl->pml_next)
+	if (pl->pml_map.pm_prog == RPC_PROGNUM &&
+	    pl->pml_map.pm_prot == IPPROTO_TCP &&
+	    pl->pml_map.pm_vers >= *version) {
+	    *version = pl->pml_map.pm_vers;
+	    port     = pl->pml_map.pm_port;
+	}
+    return port;
 #else
+#ifdef HAVE_PMAP_GETPORT
+    for (*version = RPC_PROGVER; *version >= 1; (*version)--)
+	if (port = pmap_getport (sin, RPC_PROGNUM, *version, IPPROTO_TCP))
+	    return port;
+#endif /* HAVE_PMAP_GETPORT */
+#endif /* HAVE_PMAP_GETMAPS */
+    *version = 1;
     return mcserver_port;
-#endif
 }
 
 
