@@ -16,7 +16,7 @@ int socket_read_block (int sock, char *dest, int len);
 int socket_write_block (int sock, char *buffer, int len);
 int send_string (int sock, char *string);
 void tcp_init (void);
-int open_tcp_link  (char *host, int *port, char *caller);
+int open_tcp_link  (char *host, int *port, int *version, char *caller);
 char *get_host_and_username (char *path, char **host, char **user, int *port,
 			     int default_port, int default_to_anon, char **pass);
 
