@@ -466,6 +466,8 @@ char *mc_get_current_wd (char *buffer, int size)
 	if (my_stat.st_ino != my_stat2.st_ino ||
 	    my_stat.st_dev != my_stat2.st_dev ||
 	    !cd_symlinks) {
+	    if (last_current_dir != current_dir)
+		free (last_current_dir);
 	    free (current_dir);
 	    current_dir = strdup (p);
 	    last_current_dir = current_dir;
@@ -508,6 +510,26 @@ int mc_chown (char *path, int owner, int group)
     return result;
 }
 
+int mc_utime (char *path, struct utimbuf *times)
+{
+    vfs *vfs;
+    int result;
+    
+    path = vfs_canon (path);
+    vfs = vfs_type (path);    
+    if (!vfs->utime) {
+	free (path);
+	return 0;
+    }
+    result = (*vfs->utime)(vfs_name (path), times);
+    free (path);
+    if (result == -1){
+	errno = (*vfs->ferrno)();
+	return -1;
+    }
+    return result;
+}
+
 int mc_readlink(char *path, char *buf, int bufsiz)
 {
     vfs *vfs;
@@ -724,7 +746,10 @@ char *vfs_canon (char *path)
 	    prefixsh = 6;
 	    isftp = 1;
 	} else {
-	    prefixsh = 3;
+	    if (path[3] == '/' && path[4] == '/')
+		prefixsh = 5;
+	    else
+		prefixsh = 3;
 	    isftp = 0;
 	}
 	
@@ -823,7 +848,8 @@ char *vfs_canon (char *path)
 		    result = strdup (path);
 	    }
 	    free (local_path);
-	    free (arc_name);
+	    if (arc_name)
+    	        free (arc_name);
 	    return result;
 	} else {
 	    /* We want to move outside of the archive */
@@ -971,6 +997,8 @@ int mc_chdir (char *path)
     oldvfsid = vfs_ncs_getid (oldvfs, current_dir, &parent);
     
     a = current_dir; /* Save a copy for case of failure */
+    if (last_current_dir != current_dir)
+	free (last_current_dir);
     last_current_dir = a;
     current_dir = vfs_canon (path);
     current_vfs = vfs_type (current_dir);
@@ -1006,6 +1034,11 @@ int vfs_current_is_extfs (void)
     return current_vfs == &extfs_vfs_ops;
 }
 
+int vfs_current_is_tarfs (void)
+{
+    return current_vfs == &tarfs_vfs_ops;
+}
+
 int vfs_file_is_local (char *filename)
 {
     vfs *vfs;
@@ -1095,18 +1128,18 @@ int mc_mknod (char *path, int mode, int dev)
 }
 
 #ifdef HAVE_MMAP
-struct mc_mmaping {
+struct mc_mmapping {
     caddr_t addr;
     void *vfs_info;
     vfs *vfs;
-    struct mc_mmaping *next;
+    struct mc_mmapping *next;
 } *mc_mmaparray = NULL;
 
 caddr_t mc_mmap (caddr_t addr, size_t len, int prot, int flags, int fd, off_t offset)
 {
     vfs *vfs;
     caddr_t result;
-    struct mc_mmaping *mcm;
+    struct mc_mmapping *mcm;
 
     if (fd == -1)
 	return (caddr_t) -1;
@@ -1117,7 +1150,7 @@ caddr_t mc_mmap (caddr_t addr, size_t len, int prot, int flags, int fd, off_t of
 	errno = (*vfs->ferrno)();
 	return (caddr_t)-1;
     }
-    mcm = (struct mc_mmaping *) xmalloc (sizeof (struct mc_mmaping), "vfs: mmap handling");
+    mcm = (struct mc_mmapping *) xmalloc (sizeof (struct mc_mmapping), "vfs: mmap handling");
     mcm->addr = result;
     mcm->vfs_info = vfs_info (fd);
     mcm->vfs = vfs;
@@ -1128,7 +1161,7 @@ caddr_t mc_mmap (caddr_t addr, size_t len, int prot, int flags, int fd, off_t of
 
 int mc_munmap (caddr_t addr, size_t len)
 {
-    struct mc_mmaping *mcm, *mcm2 = NULL;
+    struct mc_mmapping *mcm, *mcm2 = NULL;
     
     for (mcm = mc_mmaparray; mcm != NULL; mcm2 = mcm, mcm = mcm->next) {
         if (mcm->addr == addr) {
@@ -1305,9 +1338,13 @@ void vfs_shut (void)
     for (stamp = stamps; stamp != NULL; stamp = stamp->next)
 	(*stamp->v->free)(stamp->id);
 
+    if (last_current_dir && last_current_dir != current_dir)
+	free (last_current_dir);
     if (current_dir)
 	free (current_dir);
 
+    extfs_done ();
+
 #ifdef USE_NETCODE
     ftpfs_done();
 #endif
@@ -1654,3 +1691,15 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
     }
     free_and_return (1);
 }
+
+void
+vfs_force_expire (char *pathname)
+{
+    vfs *vfs;
+    
+    pathname = vfs_canon (pathname);
+    vfs = vfs_type (pathname);
+    if (vfs->forget_about)
+	(*vfs->forget_about) (pathname);
+    free (pathname);
+}
