@@ -8,6 +8,7 @@
 #endif
 
 #include <sys/time.h>
+#include <utime.h>
 
     /* Our virtual file system layer */
     
@@ -31,6 +32,7 @@
 
 	int  (*chmod)(char *path, int mode);
 	int  (*chown)(char *path, int owner, int group);
+	int  (*utime)(char *path, struct utimbuf *times);
 
 	int  (*readlink)(char *path, char *buf, int size);
 	int  (*symlink)(char *n1, char *n2);
@@ -54,6 +56,7 @@
 	
 	int  (*ctl)(void *vfs_info, int ctlop, int arg);
 	int  (*setctl)(char *path, int ctlop, char *arg);
+	void (*forget_about)(char *path);
 #ifdef HAVE_MMAP
 	caddr_t (*mmap)(caddr_t addr, size_t len, int prot, int flags, void *vfs_info, off_t offset);
 	int (*munmap)(caddr_t addr, size_t len, void *vfs_info);
@@ -89,6 +92,7 @@
     char *mc_get_current_wd (char *buffer, int bufsize);
     int vfs_current_is_local (void);
     int vfs_current_is_extfs (void);
+    int vfs_current_is_tarfs (void);
     int vfs_file_is_local (char *name);
     char *vfs_get_current_dir (void);
     
@@ -100,6 +104,7 @@
     void vfs_free_resources(char *path);
     void vfs_timeout_handler ();
     int vfs_timeouts ();
+    void vfs_force_expire (char *pathname);
 
     void vfs_fill_names (void (*)(char *));
 
@@ -214,6 +219,7 @@
 #   define vfs_free_resources()
 #   define vfs_timeout_handler()
 #   define vfs_timeouts() 0
+#   define vfs_force_expire () 
 
     typedef int vfs;
     
